package jp.oarts.pirka.iop.tool.core.business;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.TreeMap;

import jp.oarts.pirka.iop.tool.core.general.vo.AttributeData;
import jp.oarts.pirka.iop.tool.core.general.vo.IncludeInfo;
import jp.oarts.pirka.iop.tool.core.general.vo.InterfaceDataItem;
import jp.oarts.pirka.iop.tool.core.general.vo.InterfaceDataItemAttribute;
import jp.oarts.pirka.iop.tool.core.general.vo.InterfaceDataMyItem;
import jp.oarts.pirka.iop.tool.core.tools.xml.XmlCreatorNode;
import jp.oarts.pirka.iop.tool.core.tools.xml.XmlParser;
import jp.oarts.pirka.iop.tool.core.tools.xml.XmlParserData;

/**
 * C^[tF[X
 *
 * @author ito
 *
 */
public class InterfaceData implements Cloneable, Serializable {

	/**
	 *
	 */
	private static final long serialVersionUID = -3585092795400490964L;

	/** IuWFNgIDEvWFNgŃj[NȃIuWFNgID */
	protected int dataId;

	/** C^[tF[XEvWFNgŃj[NȃC^[tF[X̖ */
	protected String name;

	/** C^[tF[X{ꖼ */
	protected String nameJp;

	/** Rg */
	protected String comment;

	/** ő̃ACeԍ */
	protected int maxItemId;

	/** ő̃CN[hԍԍ */
	protected int maxIncludeId;

	/**
	 * C^[tF[Xڃ}bv<br>
	 * L[Fj[NACeL[Af[^FC^[tF[XڃIuWFNg
	 */
	protected TreeMap<Integer, InterfaceDataItem> itemMap;

	/**
	 * CN[hꂽC^[tF[X}bv<br>
	 * L[FCN[hC^[tF[XIDAf[^FCN[hC^[tF[X
	 */
	protected TreeMap<Integer, IncludeInfo> includeMap;

	/**
	 * Agr[g}bv<br>
	 * L[Fj[NACeL[Af[^FAgr[gIuWFNg
	 */
	protected TreeMap<Integer, InterfaceDataItemAttribute> attributeMap;

	/**
	 * C^[tF[Xڂ̕я<br>
	 * vf̓tB[hACeID
	 */
	protected LinkedList<Integer> itemSeqList;

	/**
	 * A^b`ꂽǗ
	 */
	protected LinkedHashSet<String> attachAttributeSet;

	/**
	 * C^[tF[X}l[W[
	 */
	protected InterfaceDataManager interfaceDataManager;

	/**
	 * RXgN^
	 *
	 * @param dataId
	 *            ID
	 */
	private InterfaceData() {
		includeMap = new TreeMap<Integer, IncludeInfo>();
		itemMap = new TreeMap<Integer, InterfaceDataItem>();
		attributeMap = new TreeMap<Integer, InterfaceDataItemAttribute>();
		itemSeqList = new LinkedList<Integer>();
		attachAttributeSet = new LinkedHashSet<String>();
	}

	/**
	 * RXgN^
	 *
	 * @param interfaceDataManager
	 *            C^[tF[X}l[W[
	 */
	protected InterfaceData(InterfaceDataManager interfaceDataManager) {
		this.interfaceDataManager = interfaceDataManager;
		this.dataId = interfaceDataManager.createNewId();

		includeMap = new TreeMap<Integer, IncludeInfo>();
		itemMap = new TreeMap<Integer, InterfaceDataItem>();
		attributeMap = new TreeMap<Integer, InterfaceDataItemAttribute>();
		itemSeqList = new LinkedList<Integer>();

		maxItemId = 0;
		maxIncludeId = 0;

		name = "interface " + dataId;
		nameJp = "C^[tF[X " + dataId;

		attachAttributeSet = new LinkedHashSet<String>();
	}

	/**
	 * VC^[tF[XID擾
	 *
	 * @return C^[tF[XID
	 */
	private int createNewItemId() {
		return ++maxItemId;
	}

	/**
	 * VCN[hID擾
	 *
	 * @return C^[tF[XID
	 */
	private int createNewIncludeId() {
		return ++maxIncludeId;
	}

	/**
	 * `ꂽC^[tF[XID̈ꗗ擾 o^
	 *
	 * @return `ꂽC^[tF[Xڈꗗ
	 */
	public Integer[] getMyItemIDList() {
		List<Integer> list = new ArrayList<Integer>();
		for (InterfaceDataItem item : itemMap.values()) {
			if (item.isBaseItem()) {
				list.add(item.getItemId());
			}
		}
		return list.toArray(new Integer[0]);
	}

	/**
	 * `ꂽC^[tF[Xڂ擾
	 *
	 * @param itemId
	 *            C^[tF[XID
	 * @return C^[tF[X
	 * @throws InterfaceException
	 */
	public InterfaceDataMyItem getMyItem(int itemId) throws InterfaceException {

		if (!itemMap.containsKey(itemId)) {
			throw new InterfaceException("w肳ꂽC^[tF[XID݂͑܂ " + itemId);
		}

		InterfaceDataItem item = itemMap.get(itemId);
		if (!item.isBaseItem()) {
			throw new InterfaceException("w肳ꂽC^[tF[XID͎`ꂽC^[tF[Xڂł͂܂ " + itemId);
		}
		return new InterfaceDataMyItem(item.getItemId(), item.getName(), item.getNameJp(), item.getType(), item.getLength(),
				item.getSubLength(), item.isPrimaryKey());
	}

	/**
	 * `C^[tF[Xڂǉ
	 *
	 * @param item
	 *            C^[tF[X
	 * @throws InterfaceException
	 */
	public int addMyItem(InterfaceDataMyItem item) throws InterfaceException {

		if (item == null) {
			throw new InterfaceException("ȃC^[tF[Xڂł");
		}

		int itemId = createNewItemId();

		InterfaceDataItem saveItem = new InterfaceDataItem(dataId, itemId);
		saveItem.setBaseItem(true);
		saveItem.setIncludeId(0);
		saveItem.setIncludeDataId(0);
		saveItem.setIncludeItemId(0);
		saveItem.setName(item.getName());
		saveItem.setNameJp(item.getNameJp());
		saveItem.setType(item.getType());
		saveItem.setLength(item.getLength());
		saveItem.setSubLength(item.getSubLength());
		saveItem.setEnable(true);
		saveItem.setUseName(true);
		saveItem.setUseNameJp(true);
		saveItem.setUseType(true);
		saveItem.setUseLength(true);
		saveItem.setUseMultiRecordType(true);
		saveItem.setMultiRecordType(false);
		saveItem.setPrimaryKey(item.isPrimaryKey());


		itemMap.put(itemId, saveItem);
		attributeMap.put(itemId, new InterfaceDataItemAttribute(itemId));
		itemSeqList.add(itemId);

		return itemId;
	}

	/**
	 * `C^[tF[XڂύX
	 *
	 * @param item
	 *            C^[tF[X
	 * @throws InterfaceException
	 */
	public void updateMyItem(InterfaceDataMyItem item) throws InterfaceException {

		if (item == null) {
			throw new InterfaceException("ȃC^[tF[Xڂł");
		}

		InterfaceDataItem saveItem = itemMap.get(item.getId());

		if (saveItem == null) {
			throw new InterfaceException("L[̃C^[tF[Xڂ݂܂ " + item.getId());
		}

		if (!saveItem.isBaseItem()) {
			throw new InterfaceException("`C^[tF[Xڂł͂܂ " + item.getId());
		}

		saveItem.setIncludeId(0);
		saveItem.setIncludeDataId(0);
		saveItem.setIncludeItemId(0);
		saveItem.setName(item.getName());
		saveItem.setNameJp(item.getNameJp());
		saveItem.setType(item.getType());
		saveItem.setLength(item.getLength());
		saveItem.setSubLength(item.getSubLength());
		saveItem.setEnable(true);
		saveItem.setUseName(true);
		saveItem.setUseNameJp(true);
		saveItem.setUseType(true);
		saveItem.setUseLength(true);
		saveItem.setUseMultiRecordType(true);
		saveItem.setMultiRecordType(false);
		saveItem.setPrimaryKey(item.isPrimaryKey());

		itemMap.put(item.getId(), saveItem);
	}

	/**
	 * `C^[tF[Xڂ폜
	 *
	 * @param itemId
	 *            C^[tF[XID
	 * @throws InterfaceException
	 */
	public void removeMyItem(int itemId) throws InterfaceException {

		InterfaceDataItem saveItem = itemMap.get(itemId);

		if (saveItem == null) {
			throw new InterfaceException("L[̃C^[tF[Xڂ݂܂");
		}

		if (!saveItem.isBaseItem()) {
			throw new InterfaceException("`C^[tF[Xڂł͂܂ " + itemId);
		}

		itemMap.remove(itemId);
		attributeMap.remove(itemId);
		itemSeqList.remove((Object) itemId);
	}

	/**
	 * w肳ꂽCN[hC^[tF[XɑC^[tF[XID̈ꗗ擾
	 *
	 * @param includeId
	 *            CN[hID
	 * @return C^[tF[Xڈꗗ
	 */
	public Integer[] getIncludeItemIdList(int includeId) {

		List<Integer> list = new ArrayList<Integer>();
		for (InterfaceDataItem item : itemMap.values()) {
			if (!item.isBaseItem() && item.getIncludeId() == includeId) {
				list.add(item.getItemId());
			}
		}
		return list.toArray(new Integer[0]);
	}

	/**
	 * C^[tF[Xڂ擾<br>
	 * ̃\bhĂяoƃf[^̍ŐVs
	 *
	 * @param itemId
	 *            C^[tF[XID
	 * @return C^[tF[X
	 * @throws InterfaceException
	 */
	public InterfaceDataItem getItem(int itemId) throws InterfaceException {

		InterfaceDataItem item = getItemMain(itemId);

		if (item == null) {
			return null;
		}

		return (InterfaceDataItem) item.clone();
	}

	/**
	 * C^[tF[Xڂ݂邩mF
	 *
	 * @param itemId
	 *            C^[tF[XID
	 * @return true=݂
	 */
	protected boolean isExists(int itemId) {

		return itemMap.containsKey(itemId);
	}

	/**
	 * C^[tF[Xڂ擾<br>
	 * ̃\bhĂяoƃf[^̍ŐVs
	 *
	 * @param itemId
	 *            C^[tF[XID
	 * @return C^[tF[X
	 * @throws InterfaceException
	 */
	protected InterfaceDataItem getItemMain(int itemId) throws InterfaceException {

		if (!itemMap.containsKey(itemId)) {
			throw new InterfaceException("w肳ꂽC^[tF[XID݂͑܂ " + itemId);
		}

		InterfaceDataItem item = itemMap.get(itemId);
		if (item.isBaseItem()) {
			return item;
		}

		InterfaceData interfaceData = interfaceDataManager.getInterfaceDataMain(item.getIncludeDataId());

		if (interfaceData == null) {
			return null;
		}

		InterfaceDataItem includeItem = interfaceData.getItemMain(item.getIncludeItemId());

		if (includeItem == null) {
			return null;
		}
		// if (!item.isEnable()) {
		// return null;
		// }

		if (!item.isUseName()) {
			item.setName(includeItem.getName());
		}

		if (!item.isUseNameJp()) {
			item.setNameJp(includeItem.getNameJp());
		}

		if (!item.isUseType()) {
			item.setType(includeItem.getType());
		}

		if (!item.isUseLength()) {
			item.setLength(includeItem.getLength());
			item.setSubLength(includeItem.getSubLength());
		}

		if (!item.isUseMultiRecordType()) {
			item.setMultiRecordType(includeItem.isMultiRecordType());
		}

		return item;
	}

	/**
	 * LȃC^[tF[XڂID̈ꗗ擾<br>
	 * [U[`я
	 *
	 * @return@LȃC^[tF[Xڂ̈ꗗ
	 */
	public Integer[] getEnableItemIdList() {

		List<Integer> list = new LinkedList<Integer>();

		for (Integer itemId : itemSeqList) {
			InterfaceDataItem item = itemMap.get(itemId);
			if (item.isEnable()) {
				list.add(item.getItemId());
			}
		}

		return list.toArray(new Integer[0]);
	}

	/**
	 * SC^[tF[XڂID̈ꗗ擾<br>
	 * [U[`я
	 *
	 * @return@SC^[tF[Xڂ̈ꗗ
	 */
	public Integer[] getAllItemIdList() {

		List<Integer> list = new LinkedList<Integer>();

		for (Integer itemId : itemSeqList) {
			InterfaceDataItem item = itemMap.get(itemId);
			list.add(item.getItemId());
		}

		return list.toArray(new Integer[0]);
	}

	/**
	 * CN[hꂽC^[tF[X̍XVs
	 *
	 * @param item
	 * @throws InterfaceException
	 */
	public void updateItem(InterfaceDataItem item) throws InterfaceException {

		if (item == null) {
			throw new InterfaceException("ȃC^[tF[Xڂł");
		}

		InterfaceDataItem orgItem = itemMap.get(item.getItemId());

		if (orgItem == null) {
			throw new InterfaceException("L[̃C^[tF[Xڂ݂܂");
		}

		if (orgItem.isBaseItem()) {
			throw new InterfaceException("CN[hꂽC^[tF[Xł͂܂");
		}

		if (orgItem.isBaseItem() != item.isBaseItem()) {
			throw new InterfaceException("ꍀڃtO͕ύXł܂");
		}

		if (orgItem.getIncludeId() != item.getIncludeId()) {
			throw new InterfaceException("CN[hID͕ύXł܂");
		}

		if (orgItem.getIncludeDataId() != item.getIncludeDataId()) {
			throw new InterfaceException("CN[hC^[tF[XID͕ύXł܂");
		}

		if (orgItem.getIncludeItemId() != item.getIncludeItemId()) {
			throw new InterfaceException("CN[hC^[tF[XID͕ύXł܂");
		}

		if (orgItem.getDataId() != item.getDataId()) {
			throw new InterfaceException("C^[tF[XID͕ύXł܂");
		}

		itemMap.put(item.getItemId(), (InterfaceDataItem) item.clone());

	}

	/**
	 * w肳ĂC^[tF[Xڂ̕яPɂ
	 *
	 * @param itemId
	 *            C^[tF[XID
	 * @return true=Afakse=ړs\
	 */
	public boolean upInterfaceItem(int itemId) {
		int index = itemSeqList.indexOf(itemId);

		if (index <= 0) {
			return false;
		}

		Integer wk = itemSeqList.get(index);
		itemSeqList.set(index, itemSeqList.get(index - 1));
		itemSeqList.set(index - 1, wk);
		return true;
	}

	/**
	 * w肳ĂC^[tF[Xڂ̕яPɂ
	 *
	 * @param itemId
	 *            C^[tF[XID
	 * @return true=Afakse=ړs\
	 */
	public boolean downInterfaceItem(int itemId) {
		int index = itemSeqList.indexOf(itemId);

		if (index >= (itemSeqList.size() - 1)) {
			return false;
		}

		Integer wk = itemSeqList.get(index);
		itemSeqList.set(index, itemSeqList.get(index + 1));
		itemSeqList.set(index + 1, wk);
		return true;

	}

	/**
	 * w肳ꂽC^[tF[XIDCN[h\ׂ
	 *
	 * @param dataId
	 *            C^[tF[XID
	 * @return true=CN[h\
	 * @throws InterfaceException
	 */
	public boolean canInclude(int dataId) throws InterfaceException {

		if (this.dataId == dataId) {
			return false;
		}

		for (IncludeInfo info : includeMap.values()) {
			InterfaceData interfaceData = interfaceDataManager.getInterfaceData(info.getIncludeDataId());
			if (interfaceData != null) {
				if (interfaceData.canInclude(dataId) == false) {
					return false;
				}
			}
		}
		return true;

	}

	/**
	 * CN[hꗗ擾
	 *
	 * @return CN[hꗗ
	 */
	public IncludeInfo[] getIncludeIfoList() {

		List<IncludeInfo> list = new LinkedList<IncludeInfo>();

		for (IncludeInfo includeInfo : includeMap.values()) {
			if (interfaceDataManager.getInterfaceDataMain(includeInfo.getIncludeDataId()) != null) {
				list.add(includeInfo);
			}
		}

		return list.toArray(new IncludeInfo[0]);

	}

	/**
	 * CN[h擾
	 *
	 * @return CN[h
	 */
	public IncludeInfo getIncludeIfo(int includeId) {

		return includeMap.get(includeId);
	}

	/**
	 * C^[tF[XCN[h
	 *
	 * @param dataId
	 *            CN[hC^[tF[XID
	 * @throws InterfaceException
	 */
	public void includeInterfaceData(int dataId) throws InterfaceException {

		if (interfaceDataManager.getInterfaceDataMain(dataId) == null) {
			throw new InterfaceException("w肳ꂽC^[tF[X݂܂ " + dataId);
		}

		int seqNo = 0;
		for (IncludeInfo includeInfo : includeMap.values()) {
			if (includeInfo.getIncludeDataId() == dataId) {
				if (seqNo < includeInfo.getSeqNo()) {
					seqNo = includeInfo.getSeqNo();
				}
			}
		}

		int includeId = createNewIncludeId();

		seqNo++;
		includeMap.put(includeId, new IncludeInfo(includeId, dataId, seqNo));

		remake();
	}

	/**
	 * CN[hĂC^[tF[X폜
	 *
	 * @param includeId
	 *            폜CN[hID
	 * @throws InterfaceException
	 */
	public void removeIncludeInterfaceData(int includeId) throws InterfaceException {

		if (!includeMap.containsKey(includeId)) {
			throw new InterfaceException("w肳ꂽCN[hID݂܂ " + includeId);
		}

		includeMap.remove(includeId);

		remake();
	}

	/**
	 * č\z
	 *
	 * @throws InterfaceException
	 */
	@SuppressWarnings("unchecked")
	public void remake() throws InterfaceException {

		TreeMap<Integer, InterfaceDataItem> tempItemMap = (TreeMap<Integer, InterfaceDataItem>) itemMap.clone();
		TreeMap<Integer, IncludeInfo> tempIncludeMap = (TreeMap<Integer, IncludeInfo>) includeMap.clone();
		TreeMap<Integer, InterfaceDataItemAttribute> tempAttributeMap = (TreeMap<Integer, InterfaceDataItemAttribute>) attributeMap
				.clone();
		LinkedList<Integer> tempFieldItemSeq = (LinkedList<Integer>) itemSeqList.clone();

		HashMap<Integer, HashSet<Integer>> includeIdItemIdSetMap = new HashMap<Integer, HashSet<Integer>>();

		// Ǘe[uNA
		itemMap.clear();
		includeMap.clear();
		attributeMap.clear();
		itemSeqList.clear();

		// CN[hID,CN[hID`FbN}bv쐬
		for (InterfaceDataItem item : tempItemMap.values()) {
			if (!item.isBaseItem()) {
				HashSet<Integer> itemIdSet = null;
				if (!includeIdItemIdSetMap.containsKey(item.getIncludeId())) {
					itemIdSet = new HashSet<Integer>();
					includeIdItemIdSetMap.put(item.getIncludeId(), itemIdSet);
				} else {
					itemIdSet = includeIdItemIdSetMap.get(item.getIncludeId());
				}
				itemIdSet.add(item.getIncludeItemId());
			}
		}

		// CN[h̍č\z
		for (IncludeInfo includeInfo : tempIncludeMap.values()) {
			InterfaceData includeData = interfaceDataManager.getInterfaceDataMain(includeInfo.getIncludeDataId());
			if (includeData != null) {

				// CN[h̃C^[tF[XCN
				includeData.remake();

				// C^[tF[X}bv֓o^
				includeMap.put(includeInfo.getIncludeId(), includeInfo);

				// CN[h̃C^[tF[Xɂ݂͑邪ACN[hɂ݂͑ȂC^[tF[[Xڂǉ
				HashSet<Integer> itemIdSet = includeIdItemIdSetMap.get(includeInfo.getIncludeId());

				boolean multi = false;	//	CN[hR[hw肩̔
				for (InterfaceDataItem item : tempItemMap.values()) {
					if (item.getIncludeId() == includeInfo.getIncludeId() && item.isMultiRecordType()) {
						multi = true;
						break;
					}
				}

				for (Integer includeItemId : includeData.getEnableItemIdList()) {
					if (itemIdSet == null || !itemIdSet.contains(includeItemId)) {

						InterfaceDataItem includeItem = includeData.getItemMain(includeItemId);
						int itemId = createNewItemId();

						InterfaceDataItem saveItem = new InterfaceDataItem(dataId, itemId);

						saveItem.setIncludeId(includeInfo.getIncludeId());
						saveItem.setIncludeDataId(includeData.getDataId());
						saveItem.setIncludeItemId(includeItemId);
						saveItem.setBaseItem(false);
						saveItem.setName(includeItem.getName());
						saveItem.setNameJp(includeItem.getNameJp());
						saveItem.setType(includeItem.getType());
						saveItem.setLength(includeItem.getLength());
						saveItem.setSubLength(includeItem.getSubLength());
						if(multi){
							saveItem.setEnable(false);	//	̃CN[hR[hw肳ĂƂ̓CN[hɐVɕtꂽڂ͌Ƃ
						}else{
							saveItem.setEnable(true);
						}
						saveItem.setUseName(false);
						saveItem.setUseNameJp(false);
						saveItem.setUseType(false);
						saveItem.setUseLength(false);
						saveItem.setUseMultiRecordType(false);
						saveItem.setMultiRecordType(false);

						saveItem.setMultiRecordType(includeItem.isMultiRecordType());

						itemMap.put(itemId, saveItem);
						tempAttributeMap.put(itemId, new InterfaceDataItemAttribute(itemId));
						tempFieldItemSeq.add(itemId);
					}
				}

			}

		}

		tempIncludeMap = null;

		// `ς݃C^[tF[Xڂ̍č\z
		for (InterfaceDataItem item : tempItemMap.values()) {
			if (item.isBaseItem()) {
				itemMap.put(item.getItemId(), item);
			} else {
				if (includeMap.containsKey(item.getIncludeId())) {
					InterfaceData includeData = interfaceDataManager.getInterfaceDataMain(item.getIncludeDataId());

					if (includeData != null) {
						if (includeData.isExists(item.getIncludeItemId())
								&& includeData.getItem(item.getIncludeItemId()).isEnable()) {
							itemMap.put(item.getItemId(), item);
						}
					}

				}
			}
		}

		// Agr[gč\z
		for (Integer itemId : tempAttributeMap.keySet()) {
			if (itemMap.containsKey(itemId)) {
				attributeMap.put(itemId, tempAttributeMap.get(itemId));
			}
		}

		// C^[tF[Xڂ̕яč\z
		for (Integer itemId : tempFieldItemSeq) {
			if (itemMap.containsKey(itemId)) {
				itemSeqList.add(itemId);
			}
		}

		// C^[tF[Xړf[^̍ŐV
		for (InterfaceDataItem item : itemMap.values()) {
			// getItem\bh͓eŐV
			getItemMain(item.getItemId());
		}

	}

	// //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

	/**
	 * ǗA^b`
	 *
	 * @param attributeManagerName
	 *            Ǘ
	 */
	public void attach(String attributeManagerName) {
		if (!attachAttributeSet.contains(attributeManagerName)) {
			attachAttributeSet.add(attributeManagerName);
		}
	}

	/**
	 * Ǘf^b`
	 *
	 * @param attributeManagerName
	 *            Ǘ
	 */
	public void dettach(String attributeManagerName) {
		attachAttributeSet.remove(attributeManagerName);
		for (Integer itemId : itemMap.keySet()) {
			InterfaceDataItemAttribute attribute = attributeMap.get(itemId);
			if (attribute != null) {
				attribute.remove(attributeManagerName);
			}
		}
	}

	/**
	 * ǗA^b`ς݂mF
	 *
	 * @return true=A^b`ς
	 */
	public boolean isAttached(String attributeManagerName) {
		return attachAttributeSet.contains(attributeManagerName);
	}

	/**
	 * Ǘ̈ꗗ擾
	 *
	 * @return Ǘ̈ꗗ
	 */
	public String[] getAttachList() {
		return attachAttributeSet.toArray(new String[0]);
	}

	/**
	 * 擾
	 *
	 * @param itemId
	 *            C^[tF[XID
	 * @param attributeManagerName
	 *            Ǘ
	 * @param key
	 *            L[
	 * @return l
	 */
	public AttributeData getAttribute(int itemId, String attributeManagerName, String key) {

		InterfaceDataItemAttribute attribute = attributeMap.get(itemId);
		if (attribute == null) {
			return new AttributeData(); // ɂ͗Ȃ͂
		}

		String value = attribute.getAttribute(attributeManagerName, key);

		if (value == null) {
			FileCreatorSampleModelerPlugin plugin = PluginManager.getInstance().getAttachPlugin(attributeManagerName);
			if (plugin != null) {
				AttributeItem aItem = plugin.getAttributeItem(key);
				if (aItem != null) {
					AttributeData defaultValue = aItem.getDefaultValue();
					if (defaultValue != null) {
						return defaultValue;
					}
				}
			}
			return new AttributeData();

		}

		return new AttributeData(value);
	}

	/**
	 * ݒ肷
	 *
	 * @param itemId
	 *            C^[tF[XID
	 * @param attributeManagerName
	 *            Ǘ
	 * @param key
	 *            L[
	 * @param value
	 *            l
	 */
	public void setAttribute(int itemId, String attributeManagerName, String key, AttributeData value) {

		String str = "";
		if (value != null) {
			str = value.getValue();
		}

		InterfaceDataItemAttribute attribute = attributeMap.get(itemId);
		if (attribute == null) {
			attribute = new InterfaceDataItemAttribute(itemId);
			attributeMap.put(itemId, attribute);
		}

		attribute.setAttribute(attributeManagerName, key, str);
	}

	/*
	 * ( Javadoc)
	 *
	 * @see java.lang.Object#clone()
	 */
	public Object clone(InterfaceDataManager interfaceDataManager) {
		InterfaceData cloneObj = new InterfaceData();

		cloneObj.dataId = dataId;
		cloneObj.name = name;
		cloneObj.nameJp = nameJp;
		cloneObj.comment = comment;
		cloneObj.maxItemId = maxItemId;
		cloneObj.maxIncludeId = maxIncludeId;

		for (Integer key : itemMap.keySet()) {
			InterfaceDataItem item = (InterfaceDataItem) itemMap.get(key).clone();
			item.setDataId(cloneObj.getDataId());
			cloneObj.itemMap.put(key, item);
		}

		for (Integer key : includeMap.keySet()) {
			cloneObj.includeMap.put(key, (IncludeInfo) includeMap.get(key).clone());
		}

		for (Integer key : attributeMap.keySet()) {
			cloneObj.attributeMap.put(key, (InterfaceDataItemAttribute) attributeMap.get(key).clone());
		}

		for (Integer itemId : itemSeqList) {
			cloneObj.itemSeqList.add(itemId);
		}

		for (String name : attachAttributeSet) {
			cloneObj.attachAttributeSet.add(name);
		}

		cloneObj.interfaceDataManager = interfaceDataManager;

		return cloneObj;
	}

	/**
	 * C^[tF[XǗɃN[쐬
	 *
	 * @param copyAttribute
	 *            true=Rs[Afalse=̓Rs[Ȃ
	 * @return InterfaceDataIuWFNg̃N[
	 */
	public InterfaceData clone(boolean copyAttribute) {

		InterfaceData cloneObj = interfaceDataManager.createInterfaceData();

		cloneObj.name = name;
		cloneObj.nameJp = nameJp;
		cloneObj.comment = comment;
		cloneObj.maxItemId = maxItemId;
		cloneObj.maxIncludeId = maxIncludeId;

		for (Integer key : itemMap.keySet()) {
			InterfaceDataItem item = (InterfaceDataItem) itemMap.get(key).clone();
			item.setDataId(cloneObj.getDataId());
			cloneObj.itemMap.put(key, item);
		}

		for (Integer key : includeMap.keySet()) {
			cloneObj.includeMap.put(key, (IncludeInfo) includeMap.get(key).clone());
		}

		for (Integer key : attributeMap.keySet()) {
			if (copyAttribute) {
				cloneObj.attributeMap.put(key, (InterfaceDataItemAttribute) attributeMap.get(key).clone());
			} else {
				cloneObj.attributeMap.put(key, new InterfaceDataItemAttribute(key));
			}
		}

		for (Integer itemId : itemSeqList) {
			cloneObj.itemSeqList.add(itemId);
		}

		if (copyAttribute) {
			for (String name : attachAttributeSet) {
				cloneObj.attachAttributeSet.add(name);
			}
		}

		return cloneObj;
	}

	/**
	 * N[쐬<br>
	 * Rs[ɃCN[hꂽC^[tF[Xڂ͑SĎ`̍ڂƂč쐬
	 *
	 * @param copyAttribute
	 *            true=Rs[Afalse=̓Rs[Ȃ
	 * @return InterfaceDataIuWFNg̃N[
	 * @throws InterfaceException
	 */
	public InterfaceData expandcClone(boolean copyAttribute) throws InterfaceException {

		InterfaceData cloneObj = interfaceDataManager.createInterfaceData();

		cloneObj.name = name;
		cloneObj.nameJp = nameJp;
		cloneObj.comment = comment;

		for (Integer itemId : getEnableItemIdList()) {

			InterfaceDataItem wkItem = getItem(itemId);
			InterfaceDataMyItem myItem = new InterfaceDataMyItem(0, wkItem.getName(), wkItem.getNameJp(), wkItem.getType(),
					wkItem.getLength(), wkItem.getSubLength(), wkItem.isPrimaryKey());

			int newItemId = cloneObj.addMyItem(myItem);

			if (copyAttribute) {
				InterfaceDataItemAttribute itemAttribute = attributeMap.get(itemId);
				if (itemAttribute != null) {
					itemAttribute = (InterfaceDataItemAttribute) itemAttribute.clone();
					itemAttribute.setItemId(newItemId);
					cloneObj.attributeMap.put(newItemId, itemAttribute);
				}
			}
		}

		if (copyAttribute) {
			for (String name : attachAttributeSet) {
				cloneObj.attachAttributeSet.add(name);
			}
		}

		return cloneObj;
	}

	/*
	 * ( Javadoc)
	 *
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {

		StringBuilder sb = new StringBuilder();

		sb.append("dataId=");
		sb.append(dataId);
		sb.append(", ");

		sb.append("name=");
		sb.append(name);
		sb.append(", ");

		sb.append("nameJp=");
		sb.append(nameJp);
		sb.append(", ");

		sb.append("comment=");
		sb.append(comment);
		sb.append(", ");

		sb.append("maxItemId=");
		sb.append(maxItemId);
		sb.append(", ");

		sb.append("maxIncludeId=");
		sb.append(maxIncludeId);
		sb.append(", ");

		sb.append("itemMap=y");
		for (InterfaceDataItem item : itemMap.values()) {
			sb.append("(");
			sb.append(item.toString());
			sb.append("), ");
		}
		sb.append("z, ");

		sb.append("includeMap=y");
		for (IncludeInfo info : includeMap.values()) {
			sb.append("(");
			sb.append(info.toString());
			sb.append("), ");
		}
		sb.append("z, ");

		sb.append("attributeMap=y");
		for (InterfaceDataItemAttribute attribute : attributeMap.values()) {
			sb.append("w");
			sb.append(attribute.toString());
			sb.append("x, ");
		}
		sb.append("z, ");

		sb.append("itemSeqList=y");
		for (Integer itemSeq : itemSeqList) {
			sb.append(itemSeq);
			sb.append(", ");
		}
		sb.append("z, ");

		sb.append("attachAttributeSet=y");
		for (String attach : attachAttributeSet) {
			sb.append(attach);
			sb.append(", ");
		}
		sb.append("z, ");

		return sb.toString();
	}

	/**
	 * ̃IuWFNgXML֊i[
	 *
	 * @param baseNode
	 */
	public void toXml(XmlCreatorNode baseNode) {

		baseNode.appendChild(new XmlCreatorNode("dataId", dataId));
		baseNode.appendChild(new XmlCreatorNode("name", name));
		baseNode.appendChild(new XmlCreatorNode("nameJp", nameJp));
		baseNode.appendChild(new XmlCreatorNode("comment", comment));
		baseNode.appendChild(new XmlCreatorNode("maxItemId", maxItemId));
		baseNode.appendChild(new XmlCreatorNode("maxIncludeId", maxIncludeId));

		XmlCreatorNode itemMapNode = new XmlCreatorNode("itemMap");
		baseNode.appendChild(itemMapNode);
		for (Integer key : itemMap.keySet()) {

			XmlCreatorNode entryNode = new XmlCreatorNode("entry");
			itemMapNode.appendChild(entryNode);

			entryNode.appendChild(new XmlCreatorNode("key", key));

			XmlCreatorNode valueNode = new XmlCreatorNode("value");
			entryNode.appendChild(valueNode);
			itemMap.get(key).toXml(valueNode);
		}

		XmlCreatorNode includeMapNode = new XmlCreatorNode("includeMap");
		baseNode.appendChild(includeMapNode);
		for (Integer key : includeMap.keySet()) {
			XmlCreatorNode entryNode = new XmlCreatorNode("entry");
			includeMapNode.appendChild(entryNode);
			entryNode.appendChild(new XmlCreatorNode("key", key));

			IncludeInfo info = includeMap.get(key);
			XmlCreatorNode valueNode = new XmlCreatorNode("value");
			entryNode.appendChild(valueNode);
			info.toXml(valueNode);
		}

		XmlCreatorNode attributeMapNode = new XmlCreatorNode("attributeMap");
		baseNode.appendChild(attributeMapNode);
		for (Integer key : attributeMap.keySet()) {
			XmlCreatorNode entryNode = new XmlCreatorNode("entry");
			attributeMapNode.appendChild(entryNode);
			entryNode.appendChild(new XmlCreatorNode("key", key));

			InterfaceDataItemAttribute attribute = attributeMap.get(key);
			XmlCreatorNode valueNode = new XmlCreatorNode("value");
			entryNode.appendChild(valueNode);
			attribute.toXml(valueNode);
		}

		XmlCreatorNode itemSeqListNode = new XmlCreatorNode("itemSeqList");
		baseNode.appendChild(itemSeqListNode);
		for (Integer value : itemSeqList) {
			itemSeqListNode.appendChild(new XmlCreatorNode("value", value));
		}

		XmlCreatorNode attachAttributeSetNode = new XmlCreatorNode("attachAttributeSet");
		baseNode.appendChild(attachAttributeSetNode);
		for (String value : attachAttributeSet) {
			attachAttributeSetNode.appendChild(new XmlCreatorNode("value", value));
		}
	}

	/**
	 * XMLf[^莩IuWFNg쐬
	 *
	 * @param xml
	 *            XMLf[^
	 * @return IuWFNg
	 * @throws InterfaceException
	 * @throws NumberFormatException
	 */
	public static InterfaceData fromXml(XmlParserData xml, InterfaceDataManager interfaceDataManager)
			throws NumberFormatException, InterfaceException {

		InterfaceData obj = new InterfaceData();

		obj.interfaceDataManager = interfaceDataManager;
		obj.dataId = xml.getInt("dataId");
		obj.name = xml.getString("name");
		obj.nameJp = xml.getString("nameJp");
		try {
			obj.comment = xml.getString("comment");
		} catch (Exception e) {
			obj.comment = ""; // ʌ݊
		}
		obj.maxItemId = xml.getInt("maxItemId");
		obj.maxIncludeId = xml.getInt("maxIncludeId");

		int index = 0;
		XmlParserData itemMapXml = xml.getXmlParserData("itemMap");
		while (true) {
			String entry = XmlParser.createKey("entry" + XmlParser.INDEX_MARKER, index++);
			if (!itemMapXml.isExists(entry)) {
				break;
			}
			XmlParserData entryXml = itemMapXml.getXmlParserData(entry);
			obj.itemMap.put(entryXml.getInt("key"), InterfaceDataItem.fromXml(entryXml.getXmlParserData("value")));
		}

		index = 0;
		XmlParserData includeMapXml = xml.getXmlParserData("includeMap");
		while (true) {
			String entry = XmlParser.createKey("entry" + XmlParser.INDEX_MARKER, index++);
			if (!includeMapXml.isExists(entry)) {
				break;
			}
			XmlParserData entryXml = includeMapXml.getXmlParserData(entry);
			obj.includeMap.put(entryXml.getInt("key"), IncludeInfo.fromXml(entryXml.getXmlParserData("value")));
		}

		index = 0;
		XmlParserData attributeMapXml = xml.getXmlParserData("attributeMap");
		while (true) {
			String entry = XmlParser.createKey("entry" + XmlParser.INDEX_MARKER, index++);
			if (!attributeMapXml.isExists(entry)) {
				break;
			}
			XmlParserData entryXml = attributeMapXml.getXmlParserData(entry);
			obj.attributeMap
					.put(entryXml.getInt("key"), InterfaceDataItemAttribute.fromXml(entryXml.getXmlParserData("value")));
		}

		index = 0;
		XmlParserData itemSeqListXml = xml.getXmlParserData("itemSeqList");
		while (true) {
			String value = XmlParser.createKey("value" + XmlParser.INDEX_MARKER, index++);
			if (!itemSeqListXml.isExists(value)) {
				break;
			}
			obj.itemSeqList.add(itemSeqListXml.getInt(value));
		}

		index = 0;
		XmlParserData attachAttributeSetXml = xml.getXmlParserData("attachAttributeSet");
		while (true) {
			String value = XmlParser.createKey("value" + XmlParser.INDEX_MARKER, index++);
			if (!attachAttributeSetXml.isExists(value)) {
				break;
			}
			obj.attachAttributeSet.add(attachAttributeSetXml.getString(value));
		}

		return obj;
	}

	public void sysout() {

		StringBuilder sb = new StringBuilder();

		sb.append("dataId=");
		sb.append(dataId);
		sb.append(", ");

		sb.append("name=");
		sb.append(name);
		sb.append(", ");

		sb.append("nameJp=");
		sb.append(nameJp);
		sb.append(", ");

		sb.append("comment=");
		sb.append(comment);
		sb.append(", ");

		sb.append("maxItemId=");
		sb.append(maxItemId);
		sb.append(", ");

		sb.append("maxIncludeId=");
		sb.append(maxIncludeId);

		System.out.println(sb.toString());

		System.out.println("itemMap=");
		for (InterfaceDataItem item : itemMap.values()) {
			System.out.print("  ");
			System.out.println(item.toString());
		}

		System.out.println("includeMap=");
		for (IncludeInfo info : includeMap.values()) {
			System.out.print("  ");
			System.out.println(info.toString());
		}

		System.out.println("attributeMap=");
		for (InterfaceDataItemAttribute attribute : attributeMap.values()) {
			System.out.print("  ");
			System.out.println(attribute.toString());
		}

		System.out.println("itemSeqList=");
		System.out.print("  ");
		for (Integer itemSeq : itemSeqList) {
			System.out.print(itemSeq);
			System.out.print(", ");
		}
		System.out.println("");

		System.out.println("attachAttributeSet=");
		for (String attach : attachAttributeSet) {
			System.out.print("  ");
			System.out.println(attach);
		}
	}

	// /@̃Qb^[^Zb^[@////////////////////////////////////////////////////////////////////////////////////////////////////////////

	/**
	 * @return maxItemId
	 */
	protected int getMaxItemId() {
		return maxItemId;
	}

	/**
	 * @return maxIncludeId
	 */
	protected int getMaxIncludeId() {
		return maxIncludeId;
	}

	/**
	 * @param maxItemId
	 *            Zbg maxItemId
	 */
	protected void setMaxItemId(int maxItemId) {
		this.maxItemId = maxItemId;
	}

	/**
	 * @param maxIncludeId
	 *            Zbg maxIncludeId
	 */
	protected void setMaxIncludeId(int maxIncludeId) {
		this.maxIncludeId = maxIncludeId;
	}

	/**
	 * @return id
	 */
	public int getDataId() {
		return dataId;
	}

	/**
	 * @return name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return nameJp
	 */
	public String getNameJp() {
		return nameJp;
	}

	/**
	 * @param name
	 *            Zbg name
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @param nameJp
	 *            Zbg nameJp
	 */
	public void setNameJp(String nameJp) {
		this.nameJp = nameJp;
	}

	/**
	 * @return comment
	 */
	public String getComment() {
		return comment;
	}

	/**
	 * @param comment
	 *            Zbg comment
	 */
	public void setComment(String comment) {
		this.comment = comment;
	}

}
