package jp.oarts.pirka.iop.tool.core.business;

import java.io.Serializable;
import java.util.Arrays;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.TreeMap;

import jp.oarts.pirka.iop.tool.core.tools.xml.XmlCreatorNode;
import jp.oarts.pirka.iop.tool.core.tools.xml.XmlParser;
import jp.oarts.pirka.iop.tool.core.tools.xml.XmlParserData;

/**
 * C^[tF[X}l[W[
 *
 * @author ito
 *
 */
public class InterfaceDataManager implements Cloneable, Serializable {

	/**
	 *
	 */
	private static final long serialVersionUID = 8965892729365291436L;

	/**
	 * C^[tF[XID̊Jnl
	 */
	private static final int START_ID = 0;

	/** őID */
	private int maxId;

	/** C^[tF[X}bv */
	private TreeMap<Integer, InterfaceData> dataMap;

	/**
	 * RXgN^
	 */
	public InterfaceDataManager() {
		maxId = START_ID;
		dataMap = new TreeMap<Integer, InterfaceData>();
	}

	/**
	 * C^[tF[X쐬
	 *
	 * @return C^[tF[X
	 */
	public InterfaceData createInterfaceData() {
		InterfaceData data = new InterfaceData(this);
		dataMap.put(data.getDataId(), data);

		return data;
	}

	/**
	 * C^[tF[XID̈ꗗ擾<br>
	 * o^
	 *
	 * @return C^[tF[XID̈ꗗ
	 */
	public Integer[] getInterfaceDataIdList() {
		return dataMap.keySet().toArray(new Integer[0]);
	}

	/**
	 * C^[tF[XID̈ꗗ擾<br>
	 * C^[tF[X
	 *
	 * @return C^[tF[XID̈ꗗ
	 */
	public Integer[] getInterfaceDataIdListOrderByName() {

		InterfaceData[] datas = dataMap.values().toArray(new InterfaceData[0]);

		Arrays.sort(datas, new Comparator<InterfaceData>() {

			@Override
			public int compare(InterfaceData o1, InterfaceData o2) {

				int ret = o1.getName().compareToIgnoreCase(o2.getName());

				if (ret != 0) {
					return ret;
				}

				if (o1.getDataId() < o2.getDataId()) {
					return -1;
				} else if (o1.getDataId() > o2.getDataId()) {
					return 1;
				}

				return 0;
			}
		});

		List<Integer> list = new LinkedList<Integer>();

		for (InterfaceData data : datas) {
			list.add(data.getDataId());
		}

		return list.toArray(new Integer[0]);
	}

	/**
	 * CN[h\ȃC^[tF[XID̈ꗗ擾<br>
	 * C^[tF[X
	 *
	 * @return C^[tF[XID̈ꗗ
	 * @throws InterfaceException
	 */
	public Integer[] getInterfaceDataIdListInclude(int targetDataId) throws InterfaceException {

		Integer[] dataIds = getInterfaceDataIdListOrderByName();

		List<Integer> list = new LinkedList<Integer>();

		for (Integer dataId : dataIds) {
			InterfaceData interfaceData = getInterfaceData(dataId);
			if (interfaceData.canInclude(targetDataId)) {
				list.add(interfaceData.getDataId());
			}
		}

		return list.toArray(new Integer[0]);
	}

	/**
	 * C^[tF[X擾
	 *
	 * @param id
	 *            C^[tF[XID
	 * @return	C^[tF[X
	 * @throws InterfaceException
	 */
	public InterfaceData getInterfaceData(int id) throws InterfaceException {
		InterfaceData data = getInterfaceDataMain(id);
		if(data!=null){
			data.remake();
		}
		return data;
	}

	/**
	 * C^[tF[X擾
	 *
	 * @param id
	 *            C^[tF[XID
	 * @return	C^[tF[X
	 */
	public InterfaceData getInterfaceDataMain(int id) {
		return dataMap.get(id);
	}

	/**
	 * C^[tF[X폜
	 *
	 * @param id
	 *            C^[tF[XID
	 */
	public void removeInterfaceData(int id) {
		dataMap.remove(id);
	}

	@Override
	public String toString() {

		StringBuilder sb = new StringBuilder();

		sb.append("maxId=");
		sb.append(maxId);
		sb.append(", ");

		sb.append("dataMap=y");
		for (InterfaceData data : dataMap.values()) {
			sb.append("(");
			sb.append(data.toString());
			sb.append("), ");
		}
		sb.append("z, ");

		return sb.toString();
	}

	/**
	 * fobOpo
	 */
	public void sysout() {

		System.out.print("maxId=");
		System.out.println(maxId);

		System.out.println("dataMap=");
		for (InterfaceData data : dataMap.values()) {
			System.out.println("--------------------------------------------------");
			data.sysout();
		}

	}

	/**
	 * ̃IuWFNgXML֊i[
	 *
	 * @param baseNode
	 */
	public void toXml(XmlCreatorNode baseNode) {

		XmlCreatorNode InterfaceDataManagerNode = new XmlCreatorNode("InterfaceDataManager");
		baseNode.appendChild(InterfaceDataManagerNode);

		InterfaceDataManagerNode.appendChild(new XmlCreatorNode("maxId", maxId));

		XmlCreatorNode dataMapNode = new XmlCreatorNode("dataMap");
		InterfaceDataManagerNode.appendChild(dataMapNode);
		for (Integer key : dataMap.keySet()) {
			XmlCreatorNode entryNode = new XmlCreatorNode("entry");
			dataMapNode.appendChild(entryNode);

			entryNode.appendChild(new XmlCreatorNode("key", key));

			InterfaceData interfaceData = dataMap.get(key);
			XmlCreatorNode valueNode = new XmlCreatorNode("value");
			entryNode.appendChild(valueNode);
			interfaceData.toXml(valueNode);
		}

	}

	/**
	 * XMLf[^莩IuWFNg쐬
	 *
	 * @param xml
	 *            XMLf[^
	 * @return IuWFNg
	 * @throws InterfaceException
	 * @throws NumberFormatException
	 */
	public static InterfaceDataManager fromXml(XmlParserData xml) throws NumberFormatException, InterfaceException {

		InterfaceDataManager obj = new InterfaceDataManager();

		xml = xml.getXmlParserData("InterfaceDataManager");

		obj.maxId = xml.getInt("maxId");

		int index = 0;
		XmlParserData dataMapXml = xml.getXmlParserData("dataMap");
		while (true) {
			String entry = XmlParser.createKey("entry" + XmlParser.INDEX_MARKER, index++);
			if (!dataMapXml.isExists(entry)) {
				break;
			}
			XmlParserData entryXml = dataMapXml.getXmlParserData(entry);
			obj.dataMap.put(entryXml.getInt("key"), InterfaceData.fromXml(entryXml.getXmlParserData("value"), obj));
		}

		return obj;
	}

	@Override
	public Object clone() {
		InterfaceDataManager cloneObj = new InterfaceDataManager();

		cloneObj.maxId = maxId;
		for (Integer id : dataMap.keySet()) {
			cloneObj.dataMap.put(id, (InterfaceData) dataMap.get(id).clone(cloneObj));
		}

		return cloneObj;

	}

	/**
	 * őID̃NA
	 */
	public void clearId() {
		maxId = START_ID;
	}

	/**
	 * ID̃Zbg őIDID傫ꍇ̂ݍőIDɃZbg
	 *
	 * @param id
	 *            ID
	 */
	public void setId(int id) {
		if (id > maxId) {
			maxId = id;
		}
	}

	/**
	 * VKID̍쐬
	 *
	 * @return VKID
	 */
	public int createNewId() {
		maxId++;

		return maxId;
	}

	/**
	 * őID擾
	 *
	 * @return őID
	 */
	public int getMaxId() {
		return maxId;
	}

	/**
	 * őIDݒ
	 *
	 * @param maxId
	 *            őID
	 */
	public void setMaxId(int maxId) {
		this.maxId = maxId;
	}
}
