package jp.oarts.pirka.iop.tool.core.general.vo;

import java.io.Serializable;
import java.util.HashMap;
import java.util.HashSet;

import jp.oarts.pirka.iop.tool.core.business.InterfaceException;
import jp.oarts.pirka.iop.tool.core.tools.xml.XmlCreatorNode;
import jp.oarts.pirka.iop.tool.core.tools.xml.XmlParser;
import jp.oarts.pirka.iop.tool.core.tools.xml.XmlParserData;

/**
 * Ǘpf[^
 *
 * @author ito
 *
 */
public class InterfaceDataItemAttribute implements Cloneable, Serializable {

	/**
	 *
	 */
	private static final long serialVersionUID = -7893989733746752636L;

	/**
	 * C^[tF[XID
	 */
	private int itemId;

	/** l}bv */
	private HashMap<AttributeKey, String> attributeMap;

	/** ǗEL[}bv */
	private HashMap<String, HashSet<AttributeKey>> attributeManagerNameMap;

	/**
	 * RXgN^
	 */
	public InterfaceDataItemAttribute(int itemId) {
		this.itemId = itemId;
		attributeMap = new HashMap<AttributeKey, String>();
		attributeManagerNameMap = new HashMap<String, HashSet<AttributeKey>>();
	}

	@Override
	public String toString() {

		StringBuilder sb = new StringBuilder();

		sb.append("itemId=");
		sb.append(itemId);
		sb.append(", ");

		sb.append("attributeMap=[");
		for (AttributeKey key : attributeMap.keySet()) {
			sb.append("<");
			sb.append(key.toString());
			sb.append(">=");

			sb.append(attributeMap.get(key));
			sb.append(", ");
		}
		sb.append("], ");

		sb.append("attributeManagerNameMap=[");
		for (String key : attributeManagerNameMap.keySet()) {
			sb.append(key);
			sb.append("=<");
			HashSet<AttributeKey> set = attributeManagerNameMap.get(key);
			for (AttributeKey ak : set) {
				sb.append("(");
				sb.append(attributeMap.get(ak));
				sb.append("), ");
			}
			sb.append(">, ");
		}
		sb.append("], ");

		return sb.toString();
	}

	/**
	 * ̃IuWFNgXML֊i[
	 *
	 * @param baseNode
	 */
	public void toXml(XmlCreatorNode baseNode) {

		baseNode.appendChild(new XmlCreatorNode("itemId", itemId));

		XmlCreatorNode attributeMapNode = new XmlCreatorNode("attributeMap");
		baseNode.appendChild(attributeMapNode);
		for (AttributeKey key : attributeMap.keySet()) {
			XmlCreatorNode entryNode = new XmlCreatorNode("entry");
			attributeMapNode.appendChild(entryNode);

			XmlCreatorNode keyNode = new XmlCreatorNode("key");
			entryNode.appendChild(keyNode);
			key.toXml(keyNode);

			entryNode.appendChild(new XmlCreatorNode("value", attributeMap.get(key)));
		}
	}

	/**
	 * XMLf[^莩IuWFNg쐬
	 *
	 * @param xml
	 *            XMLf[^
	 * @return IuWFNg
	 * @throws InterfaceException
	 * @throws NumberFormatException
	 */
	public static InterfaceDataItemAttribute fromXml(XmlParserData xml) throws NumberFormatException, InterfaceException {

		InterfaceDataItemAttribute obj = new InterfaceDataItemAttribute(xml.getInt("itemId"));

		XmlParserData attributeMapXml = xml.getXmlParserData("attributeMap");

		int index = 0;

		while (true) {

			String entry = XmlParser.createKey("entry" + XmlParser.INDEX_MARKER, index++);

			if (!attributeMapXml.isExists(entry)) {
				break;
			}

			XmlParserData entryXml = attributeMapXml.getXmlParserData(entry);
			AttributeKey key = obj.new AttributeKey(entryXml.getXmlParserData("key"));
			obj.attributeMap.put(key, entryXml.getString("value"));

			HashSet<AttributeKey> attributeKeySet = null;
			if (obj.attributeManagerNameMap.containsKey(key.attributeManagerName)) {
				attributeKeySet = obj.attributeManagerNameMap.get(key.attributeManagerName);
			} else {
				attributeKeySet = new HashSet<AttributeKey>();
				obj.attributeManagerNameMap.put(key.attributeManagerName, attributeKeySet);
			}
			attributeKeySet.add(key);
		}

		return obj;
	}

	/**
	 * ̐ݒ
	 *
	 * @param attributeManagerName
	 *            Ǘ
	 * @param keyName
	 *            L[
	 * @param value
	 *            l
	 */
	public void setAttribute(String attributeManagerName, String keyName, String value) {

		AttributeKey key = new AttributeKey(attributeManagerName, keyName);
		attributeMap.put(key, value);

		HashSet<AttributeKey> keySet = attributeManagerNameMap.get(attributeManagerName);
		if (keySet == null) {
			keySet = new HashSet<AttributeKey>();
			attributeManagerNameMap.put(attributeManagerName, keySet);
		}
		keySet.add(key);
	}

	/**
	 * 擾
	 *
	 * @param attributeManagerName
	 *            Ǘ
	 * @param keyName
	 *            L[
	 * @return l
	 */
	public String getAttribute(String attributeManagerName, String keyName) {
		return attributeMap.get(new AttributeKey(attributeManagerName, keyName));
	}

	/**
	 * 폜
	 *
	 * @param attributeManagerName
	 *            Ǘ
	 */
	public void remove(String attributeManagerName) {
		HashSet<AttributeKey> keySet = attributeManagerNameMap.get(attributeManagerName);
		if (keySet == null) {
			return;
		}

		for (AttributeKey key : keySet) {
			attributeMap.remove(key);
		}

	}

	/*
	 * ( Javadoc)
	 *
	 * @see java.lang.Object#clone()
	 */
	@Override
	public Object clone() {

		InterfaceDataItemAttribute cloneObj = new InterfaceDataItemAttribute(itemId);

		for (AttributeKey key : attributeMap.keySet()) {
			cloneObj.attributeMap.put(key, attributeMap.get(key));
		}

		for (String key : attributeManagerNameMap.keySet()) {
			cloneObj.attributeManagerNameMap.put(key, attributeManagerNameMap.get(key));
		}

		return cloneObj;
	}

	/**
	 * L[NX
	 *
	 * @author ito
	 *
	 */
	@SuppressWarnings("serial")
	public class AttributeKey implements Serializable {

		/** Ǘ */
		public String attributeManagerName;

		/** L[ */
		public String keyName;

		/**
		 * RXgN^
		 *
		 * @param attributeManagerName
		 *            Ǘ
		 * @param keyName
		 *            L[
		 */
		public AttributeKey(String attributeManagerName, String keyName) {
			this.attributeManagerName = attributeManagerName;
			this.keyName = keyName;
		}

		/**
		 * RXgN^
		 *
		 * @param xml
		 *            XMLf[^
		 * @throws InterfaceException
		 */
		public AttributeKey(XmlParserData xml) throws InterfaceException {
			this(xml.getString("attributeManagerName"), xml.getString("keyName"));
		}

		@Override
		public String toString() {

			StringBuilder sb = new StringBuilder();

			sb.append("attributeManagerName=");
			sb.append(attributeManagerName);
			sb.append(", ");

			sb.append("keyName=");
			sb.append(keyName);

			return sb.toString();
		}

		/**
		 * ̃IuWFNgXML֊i[
		 *
		 * @param baseNode
		 */
		public void toXml(XmlCreatorNode baseNode) {

			baseNode.appendChild(new XmlCreatorNode("attributeManagerName", attributeManagerName));
			baseNode.appendChild(new XmlCreatorNode("keyName", keyName));
		}

		/**
		 * XMLf[^莩IuWFNg쐬
		 *
		 * @param xml
		 *            XMLf[^
		 * @return IuWFNg
		 * @throws InterfaceException
		 */
		public AttributeKey fromXml(XmlParserData xml) throws InterfaceException {
			return new AttributeKey(xml.getString("attributeManagerName"), xml.getString("keyName"));
		}

		@Override
		public int hashCode() {

			if (attributeManagerName != null && keyName != null) {
				return attributeManagerName.hashCode() % 100000 + keyName.hashCode() % 100000;
			} else if (attributeManagerName == null && keyName != null) {
				return keyName.hashCode() % 100000;
			} else if (attributeManagerName != null && keyName == null) {
				return attributeManagerName.hashCode() % 100000;
			}

			return 0;
		}

		@Override
		public boolean equals(Object obj) {
			if (!(obj instanceof AttributeKey)) {
				return false;
			}

			AttributeKey target = (AttributeKey) obj;
			return target.attributeManagerName.equals(attributeManagerName) && target.keyName.equals(keyName);
		}

	}

	/**
	 * @return itemId
	 */
	public int getItemId() {
		return itemId;
	}

	/**
	 * @param itemId
	 *            Zbg itemId
	 */
	public void setItemId(int itemId) {
		this.itemId = itemId;
	}
}
