package jp.oarts.pirka.iop.tool.core.plugin.file.csv;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.List;

import jp.oarts.pirka.iop.tool.core.business.AttributeItem;
import jp.oarts.pirka.iop.tool.core.business.FileCreatorSampleModelerSimplePlugin;
import jp.oarts.pirka.iop.tool.core.business.InterfaceData;
import jp.oarts.pirka.iop.tool.core.business.InterfaceException;
import jp.oarts.pirka.iop.tool.core.business.InterfaceProject;
import jp.oarts.pirka.iop.tool.core.business.ParameterChecker;
import jp.oarts.pirka.iop.tool.core.business.ParameterItem;
import jp.oarts.pirka.iop.tool.core.business.ParameterSimpleItem;
import jp.oarts.pirka.iop.tool.core.general.constants.FieldType;
import jp.oarts.pirka.iop.tool.core.general.constants.ParameterType;
import jp.oarts.pirka.iop.tool.core.general.constants.PluginType;
import jp.oarts.pirka.iop.tool.core.general.vo.AttributeFileImage;
import jp.oarts.pirka.iop.tool.core.general.vo.InterfaceDataItem;
import jp.oarts.pirka.iop.tool.core.general.vo.ParameterData;
import jp.oarts.pirka.iop.tool.core.general.vo.Selecter;

/**
 * CSVt@C쐬Et@CNG[^[vOC
 *
 * @author ito
 *
 */
public class CsvFileCreator extends FileCreatorSampleModelerSimplePlugin {

	/**
	 *
	 */
	private static final long serialVersionUID = 1L;

	// vOCʖ
	public static final String PLUGIN_NAME = "jp.oarts.ifop.tool.core.plugin.file.csv#CsvFileCreator";

	// vOC{ꖼ
	public static final String PLUGIN_NAME_JP = "C^[tF[XCSVt@C쐬";

	// p[^`
	// ///////////////////////////////////////////////////////////////////
	@SuppressWarnings("serial")
	private static ParameterItem[] parameterItems = new ParameterItem[] { //

			//
			new ParameterSimpleItem("targetInterface", "o͂sC^[tF[X", "o͑Ώۂ̃C^[tF[Xw肵܂", ParameterType.INTERFACE, 0,
					null, new ParameterChecker() {

						@Override
						public void check(String value) throws InterfaceException {
							if (value == null || value.length() <= 0) {
								throw new InterfaceException("͂ł");
							}
						}
					}),

			//
			new ParameterSimpleItem("existsHeaderLine", "CSVwb_sL", "o̓t@CCSVwb_so͂Ƃ̓`FbN܂", ParameterType.CHECK,
					new ParameterData(true), null),
			new ParameterSimpleItem("outputInterfaceInfo", "C^[tF[X̏o͂", "CSVt@CɃC^[tF[X̏o͂Ƃ̓`FbN܂",
					ParameterType.CHECK, new ParameterData(false), null),

			//
			new ParameterSimpleItem("charSet", "R[h", "o̓t@C̕R[hI܂", ParameterType.SELECT, new ParameterData("MS932"),
					new ParameterChecker() {

						@Override
						public void check(String value) throws InterfaceException {
							if (value == null || value.length() <= 0) {
								throw new InterfaceException("͂ł");
							}
						}
					}, new Selecter("MS932", "SHIFT-JIS"), new Selecter("EUC-JP", "EUC"), new Selecter("UTF8", "UTF8")),

			//
			new ParameterSimpleItem("newLine", "sR[h", "o̓t@C̉sR[hI܂", ParameterType.SELECT, new ParameterData("CRLF"),
					new ParameterChecker() {

						@Override
						public void check(String value) throws InterfaceException {
							if (value == null || value.length() <= 0) {
								throw new InterfaceException("͂ł");
							}
						}
					}, new Selecter("CELF", "CR LF"), new Selecter("CR", "CR"), new Selecter("LF", "LF")),
	//
	};

	// /////////////////////////////////////////////////////////////////////////////////////

	public CsvFileCreator() {
		super(PLUGIN_NAME, PLUGIN_NAME_JP, PluginType.FILE_CREATOR, false, true, true);
	}

	@Override
	public ParameterItem[] getParameterItems() {
		return parameterItems;
	}

	@Override
	public AttributeItem[] getAttributeItems() {
		return null;
	}

	@Override
	public String getComment() {
		return "w肳ꂽC^[tF[X̓eCSVt@Co͂܂";
	}

	@Override
	public String getAttributeComment() {
		return "";
	}

	@Override
	public String getParameterComment() {
		return "w肳ꂽC^[tF[X̓eCSVt@Co͂܂B\n" //
				+ "A^b`ꂽl͏o͂܂B\n" //
				+ "o͂ꂽt@ĆwCSVt@CC^[tF[X쐬x\n" //
				+ "ǂݍ݉\łB";
	}

	@Override
	public void checkParameter(HashMap<String, ParameterData> dataMap, InterfaceProject interfaceProject,
			List<String> errrorKeyList) throws InterfaceException {

	}

	@Override
	public AttributeFileImage create(HashMap<String, ParameterData> dataMap, InterfaceProject interfaceProject)
			throws InterfaceException {

		int targetInterfaceId = dataMap.get("targetInterface").getValueInt();
		boolean existsHeaderLine = dataMap.get("existsHeaderLine").getValueBoolean();
		boolean outputInterfaceInfo = dataMap.get("outputInterfaceInfo").getValueBoolean();
		String charSet = dataMap.get("charSet").getValue();
		String newLine = dataMap.get("newLine").getValue();

		byte[] newLineCode;
		if ("LF".equalsIgnoreCase(newLine)) {
			newLineCode = new byte[] { 0x0a };
		} else if ("CR".equalsIgnoreCase(newLine)) {
			newLineCode = new byte[] { 0x0d };
		} else {
			newLineCode = new byte[] { 0x0d, 0x0a };
		}

		InterfaceData interfaceData = interfaceProject.getInterfaceDataManager().getInterfaceData(targetInterfaceId);
		if (interfaceData == null) {
			throw new InterfaceException("C^[tF[X̎擾Ɏs܂");
		}

		StringBuilder sb = new StringBuilder();
		ByteArrayOutputStream baos = null;

		try {
			try {

				baos = new ByteArrayOutputStream();

				if (outputInterfaceInfo) {
					DecimalFormat fmt = new DecimalFormat("0000");
					String Info = interfaceData.getName() + " ( " + fmt.format(interfaceData.getDataId()) + " : "
							+ interfaceData.getNameJp() + " )";
					baos.write(Info.getBytes(charSet));
					baos.write(newLineCode);
					baos.write(newLineCode);
				}

				if (existsHeaderLine) {
					String hd = createSafeCsvField("ږ") + "," + createSafeCsvField("ږ({)") + ","
							+ createSafeCsvField("^") + "," + createSafeCsvField("ڒ") + "," + createSafeCsvField("");
					baos.write(hd.getBytes(charSet));
					baos.write(newLineCode);
				}

				for (Integer itemId : interfaceData.getEnableItemIdList()) {

					InterfaceDataItem item = interfaceData.getItem(itemId);

					sb.setLength(0);
					sb.append(createSafeCsvField(item.getName()));
					sb.append(',');
					sb.append(createSafeCsvField(item.getNameJp()));
					sb.append(',');
					sb.append(createSafeCsvField(item.getType()));
					sb.append(',');
					sb.append(createSafeCsvField(item.getLength()));
					sb.append(',');
					sb.append(createSafeCsvField(item.getSubLength()));

					baos.write(sb.toString().getBytes(charSet));
					baos.write(newLineCode);
				}

				baos.flush();

			} finally {
				if (baos != null) {
					baos.close();
				}
			}
			return new AttributeFileImage(interfaceData.getName() + ".csv", baos.toByteArray());

		} catch (IOException e) {
			throw new InterfaceException("쐬ɗ\ȂG[܂");
		}

	}

	/**
	 * CSVt@C1ڂ̍쐬<BR>
	 * w"xwCx␳
	 *
	 * @param value
	 *            ڒl
	 * @return ␳ꂽڒl
	 */
	private String createSafeCsvField(Object value) {

		if (value == null) {
			return "";
		}

		String str = "";
		if (value instanceof Integer) {
			str = ((Integer) value).toString();
		} else if (value instanceof Long) {
			str = ((Long) value).toString();
		} else if (value instanceof FieldType) {
			str = ((FieldType) value).toString();
		} else {
			str = value.toString();
		}

		// s̓Xy[Xɕϊ
		str = str.replaceAll("\\r\\n", " ");
		str = str.replaceAll("\\r", " ");
		str = str.replaceAll("\\n", " ");

		boolean fenceFlag = false;

		// w"x́w""xɕϊ_uR[e[Vň͂
		if (str.indexOf('\"') >= 0) {
			str = str.replaceAll("\"", "\"\"");
			fenceFlag = true;
		}

		// w,x݂_uR[e[Vň͂
		if (str.indexOf(',') >= 0) {
			fenceFlag = true;
		}

		if (fenceFlag) {
			str = "\"" + str + "\"";
		}
		return str;
	}

}
