package jp.oarts.pirka.iop.tool.core.plugin.in;

import java.io.StringReader;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import jp.oarts.pirka.core.analyzer.html.HtmlFieldMapAnalyzer;
import jp.oarts.pirka.core.analyzer.html.HtmlMockupAnalyzer;
import jp.oarts.pirka.core.analyzer.html.HtmlSplitAnalyzer;
import jp.oarts.pirka.core.app.OfflineEntryPoint;
import jp.oarts.pirka.core.ctrl.proc.CtrlProcesser;
import jp.oarts.pirka.core.general.HtmlCtrlParts;
import jp.oarts.pirka.core.general.HtmlParts;
import jp.oarts.pirka.core.kernel.FieldMap;
import jp.oarts.pirka.core.util.field.FieldItem;
import jp.oarts.pirka.iop.tool.core.business.InterfaceData;
import jp.oarts.pirka.iop.tool.core.business.InterfaceDesignerPlugin;
import jp.oarts.pirka.iop.tool.core.business.InterfaceException;
import jp.oarts.pirka.iop.tool.core.business.InterfaceProject;
import jp.oarts.pirka.iop.tool.core.business.ParameterChecker;
import jp.oarts.pirka.iop.tool.core.business.ParameterItem;
import jp.oarts.pirka.iop.tool.core.business.ParameterSimpleItem;
import jp.oarts.pirka.iop.tool.core.general.constants.FieldType;
import jp.oarts.pirka.iop.tool.core.general.constants.ParameterType;
import jp.oarts.pirka.iop.tool.core.general.vo.InterfaceDataMyItem;
import jp.oarts.pirka.iop.tool.core.general.vo.ParameterData;
import jp.oarts.pirka.iop.tool.core.tools.InterfaceTools;

/**
 * PirkapHTMLt@CǍEt@CNG[^[vOC
 *
 * @author ito
 *
 */
public class PirkaHtmlFileInterfaceDesigner implements InterfaceDesignerPlugin {

	/**
	 *
	 */
	private static final long serialVersionUID = 1900563334848140695L;

	// p[^`
	@SuppressWarnings("serial")
	private static ParameterItem[] parameterItems = new ParameterItem[] { //

			//

			new ParameterSimpleItem("file", "HTMLt@C(pirkapHTML)", "ǍspirkapHTMt@Cw肵܂", ParameterType.FILE, null,
					new ParameterChecker() {

						@Override
						public void check(String value) throws InterfaceException {
							if (value == null || value.length() <= 0) {
								throw new InterfaceException("͂ł");
							}
						}
					}),
			//

			new ParameterSimpleItem("outputTag", "\p^OΏۂƂ", "\^OC^[tF[XڂƂĎ荞ނƂɃ`FbN܂", ParameterType.CHECK,
					new ParameterData(true), null),
			//

			new ParameterSimpleItem("inputTag", "̓^OΏۂƂ", "̓^OC^[tF[XڂƂĎ荞ނƂɃ`FbN܂", ParameterType.CHECK,
					new ParameterData(true), null),

			//
			new ParameterSimpleItem("actionTag", "ANV^Oi{^AAJ[jΏۂƂ", "ANV^Oi{^AAJ[jC^[tF[XڂƂĎ荞ނƂɃ`FbN܂",
			//

					ParameterType.CHECK, new ParameterData(true), null),
			new ParameterSimpleItem("etcTag", "̑̃^OΏۂƂ", "̑̃^OC^[tF[XڂƂĎ荞ނƂɃ`FbN܂", ParameterType.CHECK,
					new ParameterData(false), null),
			//

			new ParameterSimpleItem("interfaceName", "C^[tF[X", "V쐬C^[tFX͂܂", ParameterType.TEXT,
					new ParameterData("newInterface"), new ParameterChecker() {

						@Override
						public void check(String value) throws InterfaceException {
							String str = (String) value;
							if (str == null || str.length() <= 0) {
								throw new InterfaceException("͂ł");
							}

							if (!InterfaceTools.isSafeInterfaceDataNmae(str)) {
								throw new InterfaceException("gpłȂ͂܂");
							}
						}
					}),
			//

			new ParameterSimpleItem("interfaceNameJp", "C^[tF[Xi{ꖼj", "V쐬C^[tFX{ꖼ͂܂", ParameterType.TEXT,
					new ParameterData("VC^[tF[X"), new ParameterChecker() {

						@Override
						public void check(String value) throws InterfaceException {
							String str = (String) value;
							if (str == null || str.length() <= 0) {
								throw new InterfaceException("͂ł");
							}
						}
					}),
			//

			new ParameterSimpleItem("comment", "C^tF[XRg", "V쐬C^[tFX̃Rg͂܂", ParameterType.TEXT_AREA, null, null),

	};

	@Override
	public String createInterfaceDesign(HashMap<String, ParameterData> dataMap, InterfaceProject project) throws InterfaceException {

		byte[] file = dataMap.get("file").getFileImage();
		boolean outputTag = dataMap.get("outputTag").getValueBoolean();
		boolean inputTag = dataMap.get("inputTag").getValueBoolean();
		boolean actionTag = dataMap.get("actionTag").getValueBoolean();
		boolean etcTag = dataMap.get("etcTag").getValueBoolean();
		String interfaceName = dataMap.get("interfaceName").getValue();
		String interfaceNameJp = dataMap.get("interfaceNameJp").getValue();
		String comment = dataMap.get("comment").getValue();

		List<InterfaceDataMyItem> fieldList = new LinkedList<InterfaceDataMyItem>();

		FieldMap fieldMap = null;
		try {
			List<HtmlParts> htmlPartsList = null;
			StringReader sr = null;

			try {
				String html = new String(file, "JISAutoDetect");
				sr = new StringReader(html);
				htmlPartsList = HtmlSplitAnalyzer.getHtmlParts(sr);
				htmlPartsList = HtmlMockupAnalyzer.cutMockup(htmlPartsList);
				sr.close();
				sr = null;
				fieldMap = HtmlFieldMapAnalyzer.getFieldMap(htmlPartsList, OfflineEntryPoint.getInstance()
						.getCtrlProcesserMap());

			} finally {
				if (sr != null) {
					sr.close();
				}
			}

		} catch (Throwable e) {
			throw new InterfaceException("HTML@C͂Ɏs܂", e);
		}

		if (fieldMap == null) {
			throw new InterfaceException("HTML@C͂Ɏs܂");
		}

		createFeldList(outputTag, inputTag, actionTag, etcTag, fieldList, fieldMap);

		InterfaceData newInterfaceData = project.getInterfaceDataManager().createInterfaceData();

		newInterfaceData.setName(interfaceName);
		newInterfaceData.setNameJp(interfaceNameJp);
		newInterfaceData.setComment(comment);

		for (InterfaceDataMyItem item : fieldList) {
			newInterfaceData.addMyItem(item);
		}

		return null;
	}

	@SuppressWarnings( { "unchecked", "deprecation" })
	protected void createFeldList(boolean outputTag, boolean inputTag, boolean actionTag, boolean etcTag,
			List<InterfaceDataMyItem> fieldList, FieldMap fieldMap) {

		int count = 0;
		for (String fieldName : fieldMap.getFieldNames()) {

			count++;

			FieldItem item = fieldMap.getFieldItem(fieldName);
			HtmlCtrlParts ctrlParts = item.getCtrl();
			CtrlProcesser ctrlProcesser = ctrlParts.getCtrlProcesser();

			String nameJp = "";
			boolean addFlag = false;
			if (ctrlProcesser.isInputCtrl() || ctrlProcesser.isCheckCtrl()) {
				if (inputTag) {
					addFlag = true;
					nameJp = " " + fieldName;
				}
			} else if (ctrlProcesser.isActionCtrl()) {
				if (actionTag) {
					addFlag = true;
					nameJp = "ANV " + fieldName;
					if (item.getValue() != null && (item.getValue() instanceof String)
							&& ((String) item.getValue()).length() >= 0) {
						nameJp = (String) item.getValue();
					}
				}
			} else if ("span".equalsIgnoreCase(ctrlParts.getTagName()) || "label".equalsIgnoreCase(ctrlParts.getTagName())) {
				if (outputTag) {
					addFlag = true;
					nameJp = "\ " + fieldName;
				}
			} else if (etcTag) {
				addFlag = true;
				nameJp = "̑ " + fieldName;
			}

			if (addFlag) {
				InterfaceDataMyItem myItem = new InterfaceDataMyItem(InterfaceTools.changeSafeInterfaceDataItemNmae(fieldName,
						"item" + (count)), nameJp, FieldType.STRING, 10L, 0, false);
				fieldList.add(myItem);
			}

			if (ctrlProcesser.isLoop() && item.getValue() != null && item.getValue() instanceof List) {
				List<FieldMap> list = (List<FieldMap>) item.getValue();
				for (FieldMap childMap : list) {
					createFeldList(outputTag, inputTag, actionTag, etcTag, fieldList, childMap);
				}
			}

		}

	}

	@Override
	public ParameterItem[] getParameterItems() {
		return parameterItems;
	}

	@Override
	public String getComment() {
		return "pirkapɍ쐬ꂽHTMLt@Cǂݍ݃C^[tF[X쐬܂";
	}

	@Override
	public String getParameterComment() {
		return "pirkapɍ쐬ꂽHTMLt@Cǂݍ݃C^[tF[X쐬܂B\n" //
				+ "HTMLincludew͖ƂȂ܂B";
	}

	@Override
	public String getName() {
		return "jp.oarts.ifop.tool.core.plugin.in#PirkaHtmlFileInterfaceDesigner";
	}

	@Override
	public String getNameJp() {
		return "HTMLt@CiprikapjC^[tF[X쐬";
	}

	@Override
	public void checkParameter(HashMap<String, ParameterData> dataMap, InterfaceProject interfaceProject,
			List<String> errrorKeyList) throws InterfaceException {

	}

	@Override
	public boolean useSaveAndLoadParameter() {
		return false;
	}

}
