package jp.oarts.pirka.iop.tool.core.plugin.in.database;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;

import jp.oarts.pirka.iop.tool.core.business.ClickListener;
import jp.oarts.pirka.iop.tool.core.business.InterfaceData;
import jp.oarts.pirka.iop.tool.core.business.InterfaceDesignerPlugin;
import jp.oarts.pirka.iop.tool.core.business.InterfaceException;
import jp.oarts.pirka.iop.tool.core.business.InterfaceProject;
import jp.oarts.pirka.iop.tool.core.business.ParameterChecker;
import jp.oarts.pirka.iop.tool.core.business.ParameterItem;
import jp.oarts.pirka.iop.tool.core.business.ParameterSimpleItem;
import jp.oarts.pirka.iop.tool.core.general.constants.DbType;
import jp.oarts.pirka.iop.tool.core.general.constants.FieldType;
import jp.oarts.pirka.iop.tool.core.general.constants.ParameterType;
import jp.oarts.pirka.iop.tool.core.general.vo.AttributeData;
import jp.oarts.pirka.iop.tool.core.general.vo.InterfaceDataMyItem;
import jp.oarts.pirka.iop.tool.core.general.vo.ParameterData;
import jp.oarts.pirka.iop.tool.core.general.vo.Selecter;
import jp.oarts.pirka.iop.tool.core.plugin.file.db.DataBaseFileCreator;

/**
 * f[^x[X^ǍEt@CfUCi[C^[tF[X
 *
 * @author ito
 *
 */
public class DatabaseInterfaceDesigner implements InterfaceDesignerPlugin {

	/*
	 * f[^x[XEtB[h^ꗗ
	 *
	 * yoraclez NUMBER[(precision [,scale])] BINARY_FLOAT BINARY_DOUBLE
	 * CHAR[(size [BYTE|CHAR])] NCHAR[(size)] NVARCHAR2(size) VARCHAR2(size
	 * [BYTE|CHAR]) DATE TIMESTAMP[(fractional_seconds)] [WITH [LOCAL] TIMEZONE]
	 * INTERVAL YEAR[(year_precision)] TO MONTH INTERVAL DAY[(day_precision)] TO
	 * SECOND[(fractional_seconds)] LONG LONG RAW RAW(s) BFILE BLOB CLOB NCLOB
	 * ROWID[(size)]
	 *
	 * ypostgreSQLz bigint int8 bigserial serial8 bit[(n)] bit varying [(n)]
	 * varbit[(n)] boolean bool box bytea character varying [ (n) ] varchar [
	 * (n) ] character [ (n) ] char [ (n) ] cidr circle date double precision
	 * float8 inet integer int int4 interval [ (p) ] line lseg macaddr money
	 * numeric [ (p, s) ] decimal [ (p, s) ] path point polygon real float4
	 * smallint int2 serial serial4 text time [ (p) ] [ without time zone ] time
	 * [ (p) ] with time zone timetz [ (p) ] timestamp [ (p) ] [ without time
	 * zone ] timestamp [ (p) ] with time zone timestamptz [ (p) ] tsquery
	 * tsvector txid_snapshot uuid xml
	 *
	 * fields YEAR MONTH DAY HOUR MINUTE SECOND YEAR TO MONTH DAY TO HOUR DAY
	 * TO MINUTE DAY TO SECOND HOUR TO MINUTE MINUTE TO SECOND
	 *
	 * yH2z INT INTEGER MEDIUMINT INT4 SIGNED BOOLEAN BIT BOOL TINYINT SMALLINT
	 * INT2 YEAR BIGINT INT8 IDENTITY DECIMAL( precisionInt [, scaleInt] )
	 * NUMBER ( precisionInt [, scaleInt] ) DEC ( precisionInt [, scaleInt] )
	 * NUMERIC( precisionInt [, scaleInt] ) DOUBLE [PRECISION] FLOAT FLOAT4
	 * FLOAT8 REAL TIME DATE TIMESTAMP DATETIME SMALLDATETIME BINARY
	 * [(precisionInt )] VARBINARY [( precisionInt )] LONGVARBINARY
	 * [(precisionInt )] RAW [( precisionInt )] BYTEA [( precisionInt )] OTHER
	 * VARCHAR [( precisionInt )] LONGVARCHAR [( precisionInt )] VARCHAR2
	 * [(precisionInt )] NVARCHAR [( precisionInt )] NVARCHAR2 [( precisionInt
	 * )] VARCHAR_CASESENSITIVE [( precisionInt )] VARCHAR_IGNORECASE
	 * [(precisionInt )] CHAR [( precisionInt )] CHARACTER [( precisionInt )]
	 * NCHAR [( precisionInt )] BLOB [( precisionInt )] TINYBLOB [(
	 * precisionInt)] MEDIUMBLOB [( precisionInt )] LONGBLOB [( precisionInt )]
	 * IMAGE [(precisionInt )] OID [( precisionInt )] CLOB [( precisionInt )]
	 * TINYTEXT [( precisionInt )] TEXT [( precisionInt )] MEDIUMTEXT [(
	 * precisionInt )] LONGTEXT [( precisionInt )] NTEXT [( precisionInt )]
	 * NCLOB[( precisionInt)] UUID ARRAY
	 */

	private static final HashMap<DbType, String> databaseDriverNameMap = new HashMap<DbType, String>();
	private static final HashSet<Integer> varcharSet = new HashSet<Integer>();
	private static final HashMap<Integer, FieldType> dbtype2FieldTypeMap = new HashMap<Integer, FieldType>();
	private static final HashMap<DbType, HashMap<String, FieldType>> evryDbDbtype2FieldTypeMap = new HashMap<DbType, HashMap<String, FieldType>>();

	private static final HashMap<DbType, ReviseType> reviseTypeMap = new HashMap<DbType, ReviseType>();

	static {
		// DBhCo[
		databaseDriverNameMap.put(DbType.h2, "org.h2.Driver");
		databaseDriverNameMap.put(DbType.oracle, "oracle.jdbc.driver.OracleDriver");
		databaseDriverNameMap.put(DbType.postgreSQL, "org.postgresql.Driver");

		reviseTypeMap.put(DbType.postgreSQL, new PostgreSQLReviseType());
		reviseTypeMap.put(DbType.h2, new H2ReviseType());
		reviseTypeMap.put(DbType.oracle, new OracleReviseType());

		// ϕ̃Zbg
		varcharSet.add(Types.LONGNVARCHAR);
		varcharSet.add(Types.LONGVARCHAR);
		varcharSet.add(Types.NVARCHAR);
		varcharSet.add(Types.VARCHAR);

		// DBtB[h^CvioptB[h^CvɃ}bv(DB)
		dbtype2FieldTypeMap.put(Types.BIGINT, FieldType.NUM);
		dbtype2FieldTypeMap.put(Types.BINARY, FieldType.IMAGE);
		dbtype2FieldTypeMap.put(Types.BIT, FieldType.BOOLEAN);
		dbtype2FieldTypeMap.put(Types.BLOB, FieldType.IMAGE);
		dbtype2FieldTypeMap.put(Types.BOOLEAN, FieldType.BOOLEAN);
		dbtype2FieldTypeMap.put(Types.CHAR, FieldType.STRING);
		dbtype2FieldTypeMap.put(Types.DATE, FieldType.DATE);
		dbtype2FieldTypeMap.put(Types.DECIMAL, FieldType.NUM);
		dbtype2FieldTypeMap.put(Types.INTEGER, FieldType.NUM);
		dbtype2FieldTypeMap.put(Types.LONGNVARCHAR, FieldType.STRING);
		dbtype2FieldTypeMap.put(Types.LONGVARBINARY, FieldType.IMAGE);
		dbtype2FieldTypeMap.put(Types.LONGVARCHAR, FieldType.STRING);
		dbtype2FieldTypeMap.put(Types.NCHAR, FieldType.STRING);
		dbtype2FieldTypeMap.put(Types.NCLOB, FieldType.STRING);
		dbtype2FieldTypeMap.put(Types.NUMERIC, FieldType.NUM);
		dbtype2FieldTypeMap.put(Types.NVARCHAR, FieldType.STRING);
		dbtype2FieldTypeMap.put(Types.SMALLINT, FieldType.NUM);
		dbtype2FieldTypeMap.put(Types.TIME, FieldType.TIME);
		dbtype2FieldTypeMap.put(Types.TIMESTAMP, FieldType.DATE_TIME);
		dbtype2FieldTypeMap.put(Types.TINYINT, FieldType.NUM);
		dbtype2FieldTypeMap.put(Types.VARBINARY, FieldType.IMAGE);
		dbtype2FieldTypeMap.put(Types.VARCHAR, FieldType.STRING);
		dbtype2FieldTypeMap.put(Types.DOUBLE, FieldType.NUM); // DOUBLEiopł͖T|[gȂ̂NUMɕϊ
		dbtype2FieldTypeMap.put(Types.FLOAT, FieldType.NUM); // FLOATiopł͖T|[gȂ̂NUMɕϊ
		dbtype2FieldTypeMap.put(Types.REAL, FieldType.NUM); // REALiopł͖T|[gȂ̂NUMɕϊ
		dbtype2FieldTypeMap.put(Types.CLOB, FieldType.STRING); // CLOBiopł͖T|[gȂ̂IMAGEɕϊ
		dbtype2FieldTypeMap.put(Types.ROWID, FieldType.NUM); // ROWIDiopł͖T|[gȂ̂NUMɕϊ
		dbtype2FieldTypeMap.put(Types.SQLXML, FieldType.STRING); // SQLXMLiopł͖T|[gȂ̂IMAGEɕϊ

		HashMap<String, FieldType> postgreSqlDbtype2FieldTypeMap = new HashMap<String, FieldType>();
		postgreSqlDbtype2FieldTypeMap.put("bit".toLowerCase(), FieldType.STRING);
		postgreSqlDbtype2FieldTypeMap.put("varbit".toLowerCase(), FieldType.STRING);
		evryDbDbtype2FieldTypeMap.put(DbType.postgreSQL, postgreSqlDbtype2FieldTypeMap);

		// T|[gDBtB[h^CvioptB[h^CvɃ}bv
		// noSupportDbtype2FieldTypeMap.put(Types.ARRAY, FieldType.STRING); //
		// ARRAYiopł͖T|[g
		// noSupportDbtype2FieldTypeMap.put(Types.DATALINK, FieldType.IMAGE); //
		// DATALINKiopł͖T|[gȂ̂IMAGEɕϊ
		// noSupportDbtype2FieldTypeMap.put(Types.JAVA_OBJECT, FieldType.IMAGE);
		// // JAVA_OBJECTiopł͖T|[gȂ̂IMAGEɕϊ
		// noSupportDbtype2FieldTypeMap.put(Types.OTHER, FieldType.IMAGE); //
		// OTHERiopł͖T|[gȂ̂IMAGEɕϊ
		// noSupportDbtype2FieldTypeMap.put(Types.REF, FieldType.IMAGE); //
		// REFiopł͖T|[gȂ̂IMAGELɕϊ
		// noSupportDbtype2FieldTypeMap.put(Types.STRUCT, FieldType.IMAGE); //
		// STRUCTiopł͖T|[gȂ̂IMAGEɕϊ

		// f[^vݒf[^^}bviDBʁj
		// HashMap<String, Integer> setLengthMapItem;
		// // PostgreSQL
		// setLengthMapItem = new HashMap<String, Integer>();
		// setLengthMapItem.put("character varying".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("varchar".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("character".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("char".toLowerCase(), SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("numeric".toLowerCase(),
		// SET_LENGTH_MODE_LEN_AND_DEC);
		// setLengthMapItem.put("decimal".toLowerCase(),
		// SET_LENGTH_MODE_LEN_AND_DEC);
		// setLengthMapItem.put("bit".toLowerCase(), SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("bit varying".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("varbit".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("interval".toLowerCase(),
		// SET_LENGTH_MODE_DEC_ONLY);
		// setLengthMapItem.put("TIME".toLowerCase(), SET_LENGTH_MODE_DEC_ONLY);
		// setLengthMapItem.put("TIME without time zone".toLowerCase(),
		// SET_LENGTH_MODE_DEC_ONLY);
		// setLengthMapItem.put("TIME with time zone".toLowerCase(),
		// SET_LENGTH_MODE_DEC_ONLY);
		// setLengthMapItem.put("timetz".toLowerCase(),
		// SET_LENGTH_MODE_DEC_ONLY);
		// setLengthMapItem.put("TIMESTAMP".toLowerCase(),
		// SET_LENGTH_MODE_DEC_ONLY);
		// setLengthMapItem.put("TIMESTAMP without time zone".toLowerCase(),
		// SET_LENGTH_MODE_DEC_ONLY);
		// setLengthMapItem.put("TIMESTAMP with time zone".toLowerCase(),
		// SET_LENGTH_MODE_DEC_ONLY);
		// setLengthMapItem.put("timestamptz".toLowerCase(),
		// SET_LENGTH_MODE_DEC_ONLY);
		// setLengthMap.put(DbType.postgreSQL, setLengthMapItem);
		//
		// // Oracle
		// setLengthMapItem = new HashMap<String, Integer>();
		// setLengthMapItem.put("CHAR".toLowerCase(), SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("VARCHAR2".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("NCHAR".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("NVARCHAR2".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("NUMBER".toLowerCase(),
		// SET_LENGTH_MODE_LEN_AND_DEC);
		// setLengthMapItem.put("TIMESTAMP".toLowerCase(),
		// SET_LENGTH_MODE_DEC_ONLY);
		// setLengthMapItem.put("TIMESTAMP WITH TIMEZONE".toLowerCase(),
		// SET_LENGTH_MODE_DEC_ONLY);
		// setLengthMapItem.put("TIMESTAMP WITH LOCAL TIMEZONE".toLowerCase(),
		// SET_LENGTH_MODE_DEC_ONLY);
		// setLengthMapItem.put("INTERVAL YEAR TO MONTH".toLowerCase(),
		// SET_LENGTH_MODE_DEC_ONLY);
		// setLengthMapItem.put("INTERVAL DAY TO SECOND".toLowerCase(),
		// SET_LENGTH_MODE_DEC_ONLY);
		// setLengthMapItem.put("RAW".toLowerCase(), SET_LENGTH_MODE_DEC_ONLY);
		// setLengthMapItem.put("ROWID".toLowerCase(),
		// SET_LENGTH_MODE_DEC_ONLY);
		// setLengthMap.put(DbType.oracle, setLengthMapItem);
		//
		// // H2
		// setLengthMapItem = new HashMap<String, Integer>();
		// setLengthMapItem.put("DECIMAL".toLowerCase(),
		// SET_LENGTH_MODE_LEN_AND_DEC);
		// setLengthMapItem.put("NUMBER".toLowerCase(),
		// SET_LENGTH_MODE_LEN_AND_DEC);
		// setLengthMapItem.put("DEC".toLowerCase(),
		// SET_LENGTH_MODE_LEN_AND_DEC);
		// setLengthMapItem.put("NUMERIC".toLowerCase(),
		// SET_LENGTH_MODE_LEN_AND_DEC);
		// setLengthMapItem.put("BINARY".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("VARBINARY".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("LONGVARBINARY".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("RAW".toLowerCase(), SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("BYTEA".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("VARCHAR".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("LONGVARCHAR".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("VARCHAR2".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("NVARCHAR".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("NVARCHAR2".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("VARCHAR_CASESENSITIVE".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("VARCHAR_IGNORECASE".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("CHAR".toLowerCase(), SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("CHARACTER".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("NCHAR".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("BLOB".toLowerCase(), SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("TINYBLOB".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("MEDIUMBLOB".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("LONGBLOB".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("IMAGE".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("OID".toLowerCase(), SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("CLOB".toLowerCase(), SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("TINYTEXT".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("TEXT".toLowerCase(), SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("MEDIUMTEXT".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("LONGTEXT".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("NTEXT".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMapItem.put("NCLOB".toLowerCase(),
		// SET_LENGTH_MODE_LEN_ONLY);
		// setLengthMap.put(DbType.h2, setLengthMapItem);

	}

	/**
	 *
	 */
	private static final long serialVersionUID = -4181482973945241760L;

	// p[^`
	@SuppressWarnings("serial")
	private static ParameterItem[] parameterItems = new ParameterItem[] { //

			new ParameterSimpleItem("db", "f[^x[X", "^͂f[^x[Xw肵܂", ParameterType.DB, 0, null, new ParameterChecker() {

				@Override
				public void check(String value) throws InterfaceException {
					if (value == null || value.length() <= 0) {
						throw new InterfaceException("͂ł");
					}
				}
			}),
			//

			new ParameterSimpleItem("connectionString", "ڑ", "f[^x[XɐڑƂ̐ڑ͂܂", ParameterType.TEXT,
					new ParameterData(""), new ParameterChecker() {

						@Override
						public void check(String str) throws InterfaceException {
							if (str == null || str.length() <= 0) {
								throw new InterfaceException("͂ł");
							}
						}
					}),
			//

			new ParameterSimpleItem("loginId", "OCID", "f[^x[XɐڑƂ̃OCID͂܂", ParameterType.TEXT,
					new ParameterData(""), new ParameterChecker() {

						@Override
						public void check(String str) throws InterfaceException {
							if (str == null || str.length() <= 0) {
								throw new InterfaceException("͂ł");
							}
						}
					}),
			//

			new ParameterSimpleItem("passwd", "pX[h", "f[^x[XɐڑƂ̃pX[h͂܂", ParameterType.TEXT, new ParameterData(""),
					new ParameterChecker() {

						@Override
						public void check(String str) throws InterfaceException {
						}
					}),
			//
			new ParameterSimpleItem("makeTableList", "e[uꗗ쐬", "f[^x[XڑɂƂe[uꗗ擾܂", new ClickListener() {
				public void run(HashMap<String, ParameterItem> paramMap, HashMap<String, ParameterData> dataMap)
						throws InterfaceException {
					if (dataMap.get("db") == null || dataMap.get("db").getValue().length() <= 0) {
						throw new InterfaceException("f[^x[XIĂ");
					}
					DbType db = DbType.valueOf(dataMap.get("db").getValue());

					String connectionString = dataMap.get("connectionString").getValue();
					if (connectionString.length() <= 0) {
						throw new InterfaceException("ڑ͂Ă");
					}

					String loginId = dataMap.get("loginId").getValue();
					if (loginId.length() <= 0) {
						throw new InterfaceException("OCID͂Ă");
					}

					String loginPassword = dataMap.get("passwd").getValue();
					try {
						String[] tableNames = DatabaseInterfaceDesigner.readDatabaseTableInfo(db, connectionString, loginId,
								loginPassword);

						List<Selecter> list = new LinkedList<Selecter>();
						for (String tableName : tableNames) {
							list.add(new Selecter(tableName, tableName));
						}

						paramMap.get("interfaceName").setSelecter(list.toArray(new Selecter[0]));
					} catch (SQLException e) {
						throw new InterfaceException("e[uꗗ̎擾Ɏs܂ " + e.getMessage());
					}

				}
			}),

			new ParameterSimpleItem("interfaceName", "e[u", "͂se[uI܂ 쐬C^[tF[X̓e[uƂȂ܂",
					ParameterType.SELECT, new ParameterData(""), new ParameterChecker() {

						@Override
						public void check(String str) throws InterfaceException {
							if (str == null || str.length() <= 0) {
								throw new InterfaceException("͂ł");
							}
							//
							// if (!InterfaceTools.isSafeInterfaceDataNmae(str))
							// {
							// throw new InterfaceException("gpłȂ͂܂");
							// }
						}
					}, new Selecter("", "")),
			//

			new ParameterSimpleItem("interfaceNameJp", "C^[tF[Xi{ꖼj", "V쐬C^[tFX{ꖼ͂܂", ParameterType.TEXT,
					new ParameterData(""), new ParameterChecker() {

						@Override
						public void check(String str) throws InterfaceException {
							if (str == null || str.length() <= 0) {
								throw new InterfaceException("͂ł");
							}
						}
					}),
			//

			new ParameterSimpleItem("undefineInputMode", "T|[gOf[^^̃tB[h", "T|[gOf[^^̃tB[hɑ΂Ă̏I܂",
					ParameterType.SELECT, new ParameterData("inputString"), new ParameterChecker() {

						@Override
						public void check(String value) throws InterfaceException {
							if (value == null || value.length() <= 0) {
								throw new InterfaceException("͂ł");
							}
						}
					}, new Selecter("uninput", "荞܂Ȃ"), new Selecter("inputString", "iSTRINGjƂĎ荞")),

			new ParameterSimpleItem("comment", "C^[tF[XRg", "V쐬C^[tFX̃Rg͂܂", ParameterType.TEXT_AREA, null,
					null),

	};

	@Override
	public String createInterfaceDesign(HashMap<String, ParameterData> dataMap, InterfaceProject project)
			throws InterfaceException {

		// Ab^`p
		DataBaseFileCreator dataBaseFileCreator = new DataBaseFileCreator();

		try {

			// p[^̎擾
			DbType db = DbType.valueOf(dataMap.get("db").getValue());
			String connectionString = dataMap.get("connectionString").getValue();
			String loginId = dataMap.get("loginId").getValue();
			String passwd = dataMap.get("passwd").getValue();
			String inputMode = dataMap.get("undefineInputMode").getValue();
			String interfaceName = dataMap.get("interfaceName").getValue();
			String interfaceNameJp = dataMap.get("interfaceNameJp").getValue();
			String comment = dataMap.get("comment").getValue();

			// ^f[^擾
			DatabaseMetaInfo[] fieldList = readDatabaseMetaInfo(db, connectionString, loginId, passwd, interfaceName, inputMode);

			// C^[tF[X̐VK쐬
			InterfaceData newInterfaceData = project.getInterfaceDataManager().createInterfaceData();

			project.getInterfaceDataManager().getInterfaceData(newInterfaceData.getDataId()).attach(
					dataBaseFileCreator.getName());
			newInterfaceData.setName(interfaceName);
			newInterfaceData.setNameJp(interfaceNameJp);
			newInterfaceData.setComment(comment);

			for (DatabaseMetaInfo item : fieldList) {
				InterfaceDataMyItem myItem = new InterfaceDataMyItem(item.getName(), item.getNameJp(), item.getType(), item
						.getLength(), item.getSubLength(), item.isPrimaryKey());
//				if (item.isPrimaryKey()) {
//					myItem.setPrimaryKey(true);
//				}

				int itemId = newInterfaceData.addMyItem(myItem);

				newInterfaceData.setAttribute(itemId, dataBaseFileCreator.getName(), "primaryKey", new AttributeData(item
						.isPrimaryKey()));

				newInterfaceData.setAttribute(itemId, dataBaseFileCreator.getName(), "notNull", new AttributeData(item
						.isNotNull()));

				newInterfaceData.setAttribute(itemId, dataBaseFileCreator.getName(), "default", new AttributeData(item
						.getDefaultValue()));

				newInterfaceData.setAttribute(itemId, dataBaseFileCreator.getName(), "varchar", new AttributeData(
						isVarchar(item.getTypesType())));

				newInterfaceData.setAttribute(itemId, dataBaseFileCreator.getName(), "dokujiType", new AttributeData(item
						.getFieldType()));

				newInterfaceData.setAttribute(itemId, dataBaseFileCreator.getName(), "autoIncrement", new AttributeData(item
						.isAutoIncrement()));

				newInterfaceData.setAttribute(itemId, dataBaseFileCreator.getName(), "sequenceName", new AttributeData(item
						.getSequenceName()));

			}

		} catch (SQLException e) {
			return "G[܂B\n" + e.getMessage();
		}

		return null;
	}

	@Override
	public ParameterItem[] getParameterItems() {
		return parameterItems;
	}

	@Override
	public String getComment() {
		return " f[^x[X^ǂݍ݃C^[tF[X쐬܂";
	}

	@Override
	public String getParameterComment() {
		return " f[^x[X^ǂݍ݃C^[tF[X쐬܂B\n" //
				+ "f[^x[Xiop삵ĂT[o[ANZX\łȂ΂Ȃ܂B\n";
	}

	@Override
	public String getName() {
		return "jp.oarts.ifop.tool.core.plugin.in#DatabaseInterfaceDesigner";
	}

	@Override
	public String getNameJp() {
		return "f[^x[X^񂩂C^[tF[X쐬";
	}

	@Override
	public void checkParameter(HashMap<String, ParameterData> dataMap, InterfaceProject interfaceProject,
			List<String> errrorKeyList) throws InterfaceException {

	}

	@Override
	public boolean useSaveAndLoadParameter() {
		return false;
	}

	/**
	 * f[^x[Xe[uꗗ擾
	 *
	 * @param db
	 *            f[^[x[X^Cv
	 * @param connectionString
	 *            ڑ
	 * @param loginId
	 *            OCID
	 * @param loginPassword
	 *            pX[h
	 * @return e[uꗗ
	 * @throws SQLException
	 */
	public static String[] readDatabaseTableInfo(DbType db, String connectionString, String loginId, String loginPassword)
			throws SQLException {

		List<String> tableList = new LinkedList<String>();

		// tableList.add("");

		// JDBChCo[擾
		try {
			Class.forName(databaseDriverNameMap.get(db));
		} catch (ClassNotFoundException e) {
			throw new SQLException("JDBCgpł܂", e);
		}

		Connection con = null;
		try {
			con = DriverManager.getConnection(connectionString, loginId, loginPassword);

			ResultSet rs = null;
			try {
				rs = con.getMetaData().getTables(null, null, "%", new String[] { "TABLE" });

				while (rs.next()) {
					tableList.add(rs.getString("TABLE_NAME"));
				}

				String[] tableNames = tableList.toArray(new String[0]);
				Arrays.sort(tableNames);
				return tableNames;
			} finally {
				if (rs != null) {
					rs.close();
				}
			}

		} finally {
			if (con != null) {
				con.close();
			}
		}
	}

	/**
	 * f[^x[Xe[ũ^擾
	 *
	 * @param db
	 *            f[^[x[X^Cv
	 * @param connectionString
	 *            ڑ
	 * @param loginId
	 *            OCID
	 * @param loginPassword
	 *            pX[h
	 * @param tableName
	 *            e[u
	 * @return e[u^
	 * @throws SQLException
	 */
	protected DatabaseMetaInfo[] readDatabaseMetaInfo(DbType db, String connectionString, String loginId, String loginPassword,
			String tableName, String inputMode) throws SQLException {

		ReviseType reviseType = reviseTypeMap.get(db);

		// JDBChCo[擾
		try {
			Class.forName(databaseDriverNameMap.get(db));
		} catch (ClassNotFoundException e) {
			throw new SQLException("JDBCgpł܂", e);
		}

		Connection con = null;
		try {
			con = DriverManager.getConnection(connectionString, loginId, loginPassword);

			ResultSet rs = null;
			try {
				rs = con.getMetaData().getTables(null, null, tableName, new String[] { "TABLE" });

				if (!rs.next()) {
					throw new SQLException("e[u݂܂");
				}

				return getTableField(con, tableName, inputMode, reviseType, evryDbDbtype2FieldTypeMap.get(db));
			} finally {
				if (rs != null) {
					rs.close();
				}
			}
		} finally {
			if (con != null) {
				con.close();
			}
		}
	}

	/**
	 * e[ũtB[h^̎擾
	 *
	 * @param con
	 *            f[^[x[XRlNV
	 * @param tableNameParam
	 *            e[u
	 * @return ^
	 * @throws SQLException
	 */
	protected DatabaseMetaInfo[] getTableField(Connection con, String tableNameParam, String inputMode, ReviseType reviseType,
			HashMap<String, FieldType> evryDbTyreDbtype2FieldType) throws SQLException {

		List<DatabaseMetaInfo> databaseMetaInfoList = new LinkedList<DatabaseMetaInfo>();

		ResultSet rsc = null;
		try {
			rsc = con.getMetaData().getColumns(null, null, tableNameParam, "%");
			while (rsc.next()) {

				DatabaseMetaInfo info = new DatabaseMetaInfo();

				info.setName(rsc.getString("COLUMN_NAME"));
				info.setNameJp(rsc.getString("COLUMN_NAME"));
				info.setNotNull("NO".equals(rsc.getString("IS_NULLABLE")));
				info.setDefaultValue(nz(rsc.getString("COLUMN_DEF")));

				long len = 0;
				try {
					// len = Long.parseLong(rsc.getString("CHAR_OCTET_LENGTH"));
					len = Long.parseLong(rsc.getString("COLUMN_SIZE"));
				} catch (NumberFormatException e) {
				}
				info.setLength(len);

				int subLen = rsc.getInt("DECIMAL_DIGITS");
				info.setSubLength(subLen);
				info.setSubLenWasNull(rsc.wasNull());

				int typesType = rsc.getInt("DATA_TYPE");
				info.setTypesType(typesType);

				String typeName = rsc.getString("TYPE_NAME");
				info.setFieldType(typeName);

				// int num_prec_radix = rsc.getInt("NUM_PREC_RADIX");

				FieldType iopFieldType = FieldType.STRING;

				if (evryDbTyreDbtype2FieldType != null && evryDbTyreDbtype2FieldType.containsKey(typeName.toLowerCase())) {
					iopFieldType = evryDbTyreDbtype2FieldType.get(typeName.toLowerCase());
				} else {
					if (dbtype2FieldTypeMap.containsKey(typesType)) {
						iopFieldType = dbtype2FieldTypeMap.get(typesType);
					} else {
						if ("uninput".equals(inputMode)) {
							continue;
						} else {
							iopFieldType = FieldType.STRING;
						}
					}
				}
				info.setType(iopFieldType);

				// try {
				// String isAutoIncrement = rsc.getString("IS_AUTOINCREMENT");
				// if ("yes".equalsIgnoreCase(isAutoIncrement)) {
				// info.setAutoIncrement(true);
				// } else if ("no".equalsIgnoreCase(isAutoIncrement)) {
				// info.setAutoIncrement(false);
				// } else {
				// info.setAutoIncrement(null);
				// }
				// } catch (SQLException e) {
				// // IS_AUTOINCREMENTUgɑݑ݂ȂƂ
				// info.setAutoIncrement(null);
				// }

				databaseMetaInfoList.add(info);
			}

		} finally {
			if (rsc != null) {
				rsc.close();
			}
		}

		HashSet<String> primaryKeySet = new HashSet<String>();
		ResultSet rsp = null;
		try {
			rsp = con.getMetaData().getPrimaryKeys(null, null, tableNameParam);

			while (rsp.next()) {
				primaryKeySet.add(rsp.getString("COLUMN_NAME").toLowerCase());
			}

		} finally {
			if (rsp != null) {
				rsp.close();
			}
		}

		for (DatabaseMetaInfo info : databaseMetaInfoList) {
			if (primaryKeySet.contains(info.getName().toLowerCase())) {
				info.setPrimaryKey(true);
			} else {
				info.setPrimaryKey(false);
			}

			if (reviseType != null) {
				reviseType.reviseType(info);
			}
		}

		return databaseMetaInfoList.toArray(new DatabaseMetaInfo[0]);

	}

	/**
	 * w肳ꂽnull̂Ƃ͋󕶎AnullȊÔƂ͎w肳ꂽԂ
	 *
	 * @param str
	 * @return 
	 */
	protected String nz(String str) {
		if (str == null) {
			return "";
		}

		return str;
	}

	/**
	 * w肳ꂽtB[h^Cvϕ񂩊mF
	 *
	 * @param fieldType
	 *            tB[h^Cv(sava.sql.Types)
	 * @return true=ϕ
	 */
	protected boolean isVarchar(int fieldType) {
		return varcharSet.contains(fieldType);

	}

	/**
	 * DB␳IuWFNgXg擾
	 *
	 * @param db
	 * @return
	 */
	public static ReviseType getReviseType(DbType db) {
		return reviseTypeMap.get(db);
	}

	/**
	 * f[^[x[X^
	 */
	class DatabaseMetaInfo {

		/** O */
		private String name;

		/** {ꖼ */
		private String nameJp;

		/** tB[h^Cv */
		private FieldType type;

		/** ڒ */
		private long length;

		/** Tuڒiȉ̌j */
		private int subLength;

		/** L[ */
		private boolean primaryKey;

		/** NULL֎~ */
		private boolean notNull;

		/** default */
		private String defaultValue;

		/** ϕ */
		private String variableString;

		/** tB[ȟ^ */
		private String fieldType;

		/** tB[ȟ^(java.sql.Types) */
		private int typesType;

		/** Tuڒnull̃tO */
		private boolean subLenWasNull;

		/** CNg */
		private boolean autoIncrement;

		/** V[PX */
		private String sequenceName;

		public DatabaseMetaInfo() {
			name = "";
			nameJp = "";
			type = FieldType.STRING;
			length = 0L;
			subLength = 0;
			primaryKey = false;
			notNull = false;
			defaultValue = "";
			variableString = "";
			fieldType = "";
			subLenWasNull = false;
			autoIncrement = false;
			sequenceName = "";
		}

		/**
		 * @return the name
		 */
		public String getName() {
			return name;
		}

		/**
		 * @return the nameJp
		 */
		public String getNameJp() {
			return nameJp;
		}

		/**
		 * @return the type
		 */
		public FieldType getType() {
			return type;
		}

		/**
		 * @return the length
		 */
		public long getLength() {
			return length;
		}

		/**
		 * @return the subLength
		 */
		public int getSubLength() {
			return subLength;
		}

		/**
		 * @return the primaryKey
		 */
		public boolean isPrimaryKey() {
			return primaryKey;
		}

		/**
		 * @return the notNull
		 */
		public boolean isNotNull() {
			return notNull;
		}

		/**
		 * @return the defaultValue
		 */
		public String getDefaultValue() {
			return defaultValue;
		}

		/**
		 * @return the variableString
		 */
		public String getVariableString() {
			return variableString;
		}

		/**
		 * @return the fieldType
		 */
		public String getFieldType() {
			return fieldType;
		}

		/**
		 * @param name
		 *            the name to set
		 */
		public void setName(String name) {
			this.name = name;
		}

		/**
		 * @param nameJp
		 *            the nameJp to set
		 */
		public void setNameJp(String nameJp) {
			this.nameJp = nameJp;
		}

		/**
		 * @param type
		 *            the type to set
		 */
		public void setType(FieldType type) {
			this.type = type;
		}

		/**
		 * @param length
		 *            the length to set
		 */
		public void setLength(long length) {
			this.length = length;
		}

		/**
		 * @param subLength
		 *            the subLength to set
		 */
		public void setSubLength(int subLength) {
			this.subLength = subLength;
		}

		/**
		 * @param primaryKey
		 *            the primaryKey to set
		 */
		public void setPrimaryKey(boolean primaryKey) {
			this.primaryKey = primaryKey;
		}

		/**
		 * @param notNull
		 *            the notNull to set
		 */
		public void setNotNull(boolean notNull) {
			this.notNull = notNull;
		}

		/**
		 * @param defaultValue
		 *            the defaultValue to set
		 */
		public void setDefaultValue(String defaultValue) {
			this.defaultValue = defaultValue;
		}

		/**
		 * @param variableString
		 *            the variableString to set
		 */
		public void setVariableString(String variableString) {
			this.variableString = variableString;
		}

		/**
		 * @param fieldType
		 *            the FieldType to set
		 */
		public void setFieldType(String fieldType) {
			this.fieldType = fieldType;
		}

		/**
		 * @return the typesType
		 */
		public int getTypesType() {
			return typesType;
		}

		/**
		 * @param typesType
		 *            the typesType to set
		 */
		public void setTypesType(int typesType) {
			this.typesType = typesType;
		}

		/**
		 * @return the subLenWasNull
		 */
		public boolean isSubLenWasNull() {
			return subLenWasNull;
		}

		/**
		 * @param subLenWasNull
		 *            the subLenWasNull to set
		 */
		public void setSubLenWasNull(boolean subLenWasNull) {
			this.subLenWasNull = subLenWasNull;
		}

		/**
		 * @return the autoIncrement
		 */
		public boolean isAutoIncrement() {
			return autoIncrement;
		}

		/**
		 * @param autoIncrement
		 *            the autoIncrement to set
		 */
		public void setAutoIncrement(boolean autoIncrement) {
			this.autoIncrement = autoIncrement;
		}

		/**
		 * @return the sequenceName
		 */
		public String getSequenceName() {
			return sequenceName;
		}

		/**
		 * @param sequenceName
		 *            the sequenceName to set
		 */
		public void setSequenceName(String sequenceName) {
			this.sequenceName = sequenceName;
		}

	}

}
