package jp.oarts.pirka.iop.tool.core.tools;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import jp.oarts.pirka.iop.tool.core.general.constants.FieldType;

/**
 * C^[tF[Xpc[
 *
 * @author ito
 *
 */
public class InterfaceTools {

	/**
	 * C^[tFXƂĎgp\`FbN
	 *
	 * @param name
	 *            C^[tF[X
	 * @return true=gp\
	 */
	public static boolean isSafeInterfaceDataNmae(String name) {
		String checkString = name.toLowerCase();
		for (int i = 0; i < checkString.length(); i++) {
			if ("abcdefghijklmnopqrstuvwxyz0123456789_".indexOf(checkString.charAt(i)) < 0) {
				return false;
			}
		}
		return true;
	}

	/**
	 * C^[tFXږƂĎgp\`FbN
	 *
	 * @param name
	 *            C^[tF[X
	 * @return true=gp\
	 */
	public static boolean isSafeInterfaceDataItemNmae(String name) {
		String checkString = name.toLowerCase();
		for (int i = 0; i < checkString.length(); i++) {
			if ("abcdefghijklmnopqrstuvwxyz0123456789".indexOf(checkString.charAt(i)) < 0) {
				return false;
			}
		}
		return true;
	}

	/**
	 * C^[tF[XƂĎgp\ɂȂ悤ɑΏۊO̕폜Oɕϊ
	 *
	 * @param name
	 *            C^tF[X
	 * @param defaultName
	 *            ϊɋ󕶎ɂȂƂɎgpC^[tF[X
	 * @return ϊ̃C^[tF[X
	 */
	public static String changeSafeInterfaceDataNmae(String name, String defaultName) {
		StringBuilder sb = new StringBuilder();
		String checkString = name.toLowerCase();
		for (int i = 0; i < checkString.length(); i++) {
			if ("abcdefghijklmnopqrstuvwxyz0123456789_".indexOf(checkString.charAt(i)) >= 0) {
				sb.append(name.charAt(i));
			}
		}
		name = sb.toString();
		if (name.length() <= 0) {
			name = defaultName;
		}
		return name;
	}

	/**
	 * C^[tF[XږƂĎgp\ɂȂ悤ɑΏۊO̕폜Oɕϊ
	 *
	 * @param name
	 *            C^tF[X
	 * @param defaultName
	 *            ϊɋ󕶎ɂȂƂɎgpC^[tF[X
	 * @return ϊ̃C^[tF[X
	 */
	public static String changeSafeInterfaceDataItemNmae(String name, String defaultName) {
		boolean nextUpper = false;
		StringBuilder sb = new StringBuilder();
		String checkString = name.toLowerCase();
		for (int i = 0; i < checkString.length(); i++) {
			if ("abcdefghijklmnopqrstuvwxyz0123456789".indexOf(checkString.charAt(i)) >= 0) {
				if (nextUpper) {
					sb.append(("" + name.charAt(i)).toUpperCase());
					nextUpper = false;
				} else {
					sb.append(name.charAt(i));
				}
			} else {
				nextUpper = true;
			}
		}
		name = sb.toString();
		if (name.length() <= 0) {
			name = defaultName;
		}
		return name;
	}

	/**
	 * CSVt@C1stB[hɕ
	 *
	 * @param line
	 *            1s
	 * @return ꂽ
	 */
	public static String[] csvLineSplit(String line) {

		List<String> list = new LinkedList<String>();

		StringBuilder sb = new StringBuilder();

		// 0=A1=_uR[e[Vn܂ł̃f[^荞݁A2=_uR[e[Vł̃f[^荞݁A3=J}܂ŃXLbv
		int mode = 0;

		for (int i = 0; i < line.length(); i++) {

			char c = line.charAt(i);
			switch (mode) {
			case 0: // 0=
				switch (c) {
				case '\"':
					mode = 1;
					break;
				case ',':
					list.add("");
					break;
				case ' ':
				case '\t':
					break;
				default:
					mode = 2;
					sb.append(c);
					break;
				}
				break;

			case 1: // _uR[e[Vn܂ł̃f[^荞
				switch (c) {
				case '\"':
					if (i < (line.length() - 1) && line.charAt(i + 1) == '\"') {
						sb.append(c);
						i++;
					} else {
						list.add(sb.toString().trim());
						sb.setLength(0);
						mode = 3;
					}
					break;
				default:
					sb.append(c);
					break;
				}
				break;

			case 2: // _uR[e[Vł̃f[^荞
				switch (c) {
				case ',':
					list.add(sb.toString().trim());
					sb.setLength(0);
					mode = 0;
					break;
				default:
					sb.append(c);
					break;
				}
				break;

			case 3: // J}܂ŃXLbv
				switch (c) {
				case ',':
					mode = 0;
					break;
				}
				break;

			}

		}

		if (mode == 1 || mode == 2) {
			list.add(sb.toString().trim());
		}

		return list.toArray(new String[0]);
	}

	/**
	 * VÕtB[h^Cv߂
	 *
	 * @param type
	 * @param length
	 * @param subLength
	 * @return VO^Cv
	 */
	public static String getVoType(FieldType type, long length, int subLength) {

		switch (type) {
		case STRING:
			return "String";

		case BOOLEAN:
			return "Boolean";

		case DATE:
			return "Date";

		case TIME:
			return "Time";

		case DATE_TIME:
			return "Timestamp";

		case IMAGE:
			return "byte[]";

		case NUM:
			if (isIntType(length, subLength)) {
				return "Integer";
			} else if (isLongType(length, subLength)) {
				return "Long";
			} else {
				return "BigDecimal";
			}
		}

		return "Object";
	}

	/**
	 * VÕtB[h̃N[߂
	 *
	 * @param type
	 * @param length
	 * @param subLength
	 * @param fieldName
	 * @return VON[
	 */
	public static String getVoClone(FieldType type, long length, int subLength, String fieldName) {

		switch (type) {
		case STRING:
		case BOOLEAN:
		case NUM:
			return fieldName;

		case DATE:
		case TIME:
		case DATE_TIME:
			return fieldName + " == null ? null : (" + getVoType(type, length, subLength) + ")" + fieldName + ".clone()";

		case IMAGE:
			return fieldName + " == null ? null : " + "java.util.Arrays.copyOf(" + fieldName + ", " + fieldName + ".length)";
		}

		return fieldName;
	}

	/**
	 * VÕtB[hr߂
	 *
	 * @param type
	 * @param length
	 * @param subLength
	 * @param targetObjectName
	 * @param fieldName
	 * @return VOr
	 */
	public static String getVoComp(FieldType type, long length, int subLength, String targetObjectName, String fieldName) {

		switch (type) {
		case STRING:
		case BOOLEAN:
		case DATE:
		case TIME:
		case DATE_TIME:
			return fieldName + ".equals(" + targetObjectName + "." + fieldName + ")";

		case IMAGE:
			return "java.util.Arrays.equals(" + fieldName + ", " + targetObjectName + "." + fieldName + ")";

		case NUM:
			if (isIntType(length, subLength)) {
				return fieldName + ".equals(" + targetObjectName + "." + fieldName + ")";
			} else if (isLongType(length, subLength)) {
				return fieldName + ".equals(" + targetObjectName + "." + fieldName + ")";
			} else {
				return "(" + fieldName + ".compareTo(" + targetObjectName + "." + fieldName + ") == 0)";
			}
		}

		return fieldName + ".equals(" + targetObjectName + "." + fieldName + ")";
	}

	/**
	 * zf[^x[Xe[ũR[h߂
	 *
	 * @param recNo
	 * @return
	 */
	public static String createRecName(int recNo) {
		return "record_" + recNo;
	}

//	/**
//	 * zf[^x[Xe[u̕R[h̃XgNX߂
//	 *
//	 * @param listNo
//	 * @return
//	 */
//	public static String createListName(int listNo) {
//		return "list_" + listNo;
//	}
//
	/**
	 * int^̐Kp邩̔
	 *
	 * @param length
	 *            ڒ
	 * @param subLength
	 *            ڒł̏_ȉ̒
	 * @return true=Kp
	 */
	public static boolean isIntType(long length, int subLength) {
		return length <= 9 && subLength == 0;
	}

	/**
	 * long^̐Kp邩̔
	 *
	 * @param length
	 *            ڒ
	 * @param subLength
	 *            ڒł̏_ȉ̒
	 * @return true=Kp
	 */
	public static boolean isLongType(long length, int subLength) {
		return length > 9 && length <= 18 && subLength == 0;
	}

	/**
	 * BigDecimal^̐Kp邩̔
	 *
	 * @param length
	 *            ڒ
	 * @param subLength
	 *            ڒł̏_ȉ̒
	 * @return true=Kp
	 */
	public static boolean isBigDecimalType(long length, int subLength) {
		return (!isIntType(length, subLength)) && (!isLongType(length, subLength));
	}

	/**
	 * ONXƂĎgpł閼Oɕϊ
	 *
	 * @param name
	 * @return O
	 */
	public static String changeClassNmae(String name) {
		if (name.length() == 1) {
			return name.toUpperCase();
		} else if (name.length() > 1) {
			return name.substring(0, 1).toUpperCase() + name.substring(1);
		}

		return "UnDefineClass";
	}

	/**
	 * Oo[ƂĎgpł閼Oɕϊ
	 *
	 * @param name
	 * @return O
	 */
	public static String changeMemberNmae(String name) {
		if (name.length() == 1) {
			return name.toLowerCase();
		} else if (name.length() > 1) {
			return name.substring(0, 1).toLowerCase() + name.substring(1);
		}

		return "UnDefineMember";
	}

	/**
	 * OQb^[\bhƂĎgpł閼Oɕϊ
	 *
	 * @param name
	 * @return O
	 */
	public static String changeGetterNmae(String name, boolean isBoolean) {

		String prefix = null;
		if (isBoolean) {
			prefix = "is";
		} else {
			prefix = "get";
		}

		if (name.length() == 1) {
			return prefix + name.toUpperCase();
		} else if (name.length() > 1) {
			return prefix + name.substring(0, 1).toUpperCase() + name.substring(1);
		}

		return prefix;
	}

	/**
	 * OZb^[\bhƂĎgpł閼Oɕϊ
	 *
	 * @param name
	 * @return O
	 */
	public static String changeSetterNmae(String name) {

		String prefix = "set";

		if (name.length() == 1) {
			return prefix + name.toUpperCase();
		} else if (name.length() > 1) {
			return prefix + name.substring(0, 1).toUpperCase() + name.substring(1);
		}

		return prefix;
	}

	/**
	 * tB[hϊ\bh擾
	 *
	 * @param name
	 * @return O
	 */
	public static String getFieldChangeMethodName(FieldType type, long length, int subLength) {

		switch (type) {
		case STRING:
			return "changeString";

		case BOOLEAN:
			return "changeBoolean";

		case DATE:
			return "changeDate";

		case TIME:
			return "changeTime";

		case DATE_TIME:
			return "changeTimestamp";

		case IMAGE:
			return "changeBytes";

		case NUM:
			if (isIntType(length, subLength)) {
				return "changeInteger";
			} else if (isLongType(length, subLength)) {
				return "changeLong";
			} else {
				return "changeBigDecimal";
			}
		}

		return "changeObject";
	}


	/**
	 * Zipt@C𓀂ă}bvɊi[
	 *
	 * @param zipImage
	 * @throws IOException
	 */
	public static void putMapFromZip(byte[] zipImage, HashMap<String, byte[]> map) throws IOException {

		byte[] buf = new byte[2048];
		ZipInputStream zis = null;

		try {
			zis = new ZipInputStream(new ByteArrayInputStream(zipImage));

			while (true) {
				ZipEntry entry = null;
				try {
					entry = zis.getNextEntry();
					if (entry == null) {
						break;
					}

					if (!entry.isDirectory()) {

						ByteArrayOutputStream baos = null;
						try {
							baos = new ByteArrayOutputStream();

							while (true) {
								int size = zis.read(buf);
								if (size < 0) {
									break;
								}
								baos.write(buf, 0, size);
							}
							baos.flush();
							baos.close();
							map.put(entry.getName(), baos.toByteArray());
							baos = null;
						} finally {
							if (baos != null) {
								baos.close();
							}
						}
					}
				} finally {
					if (entry != null) {
						zis.closeEntry();
					}
				}
			}
		} finally {
			if (zis != null) {
				zis.close();
			}
		}
	}
}
