package jp.oarts.pirka.iop.tool.core.tools.text;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import jp.oarts.pirka.iop.tool.core.business.InterfaceException;

public class HtmlTextCreator extends TextCreator {

	/**
	 * x[XeLXg̃LN^Zbg
	 */
	//private static final String CHAR_SET = "JISAutoDetect";

	private static final String MARK_START_STRING = "<!-- #! ";
	private static final String MARK_END_STRING = " -->";

	public HtmlTextCreator() throws InterfaceException {
		// Ȃ
	}

	/**
	 * RXgN^
	 *
	 * @param textName
	 * @throws InterfaceException
	 */
	public HtmlTextCreator(String resourceName, String charSet) throws InterfaceException {
		try {
			textMap = analyze(new InputStreamReader(getClass().getResourceAsStream(resourceName), charSet));
		} catch (UnsupportedEncodingException e) {
			throw new InterfaceException(e);
		}
	}

	/**
	 * RXgN^
	 *
	 * @param textName
	 * @throws InterfaceException
	 */
	public HtmlTextCreator(InputStream is, String charSet) throws InterfaceException {
		try {
			textMap = analyze(new InputStreamReader(is, charSet));
		} catch (UnsupportedEncodingException e) {
			throw new InterfaceException(e);
		}
	}

	/**
	 * RXgN^
	 *
	 * @param textName
	 * @throws InterfaceException
	 */
	public HtmlTextCreator(byte[] buff, String charSet) throws InterfaceException {
		try {
			textMap = analyze(new InputStreamReader(new ByteArrayInputStream(buff), charSet));
		} catch (UnsupportedEncodingException e) {
			throw new InterfaceException(e);
		}
	}

	/**
	 * \[Xt@C̉
	 *
	 * @param resourceName
	 *            \[Xt@C
	 * @return ͂ꂽ}bviL[F^CgAlF^Cg̍sj
	 * @throws InterfaceException
	 */
	protected HashMap<String, List<String>> analyze(Reader reader) throws InterfaceException {

		int markTotalLength = MARK_START_STRING.length() + MARK_END_STRING.length();

		List<String> lineList = new LinkedList<String>();
		String title = "";

		// InputStream is = null;
		BufferedReader br = null;

		HashMap<String, List<String>> map = new HashMap<String, List<String>>();
		lineList = new LinkedList<String>();

		try {
			try {
				br = new BufferedReader(reader);

				while (true) {
					String line = br.readLine();
					if (line == null) {
						break;
					}

					// ^CgƔ
					if (line.length() > markTotalLength
							&& MARK_START_STRING.equals(line.substring(0, MARK_START_STRING.length())) //
							&& MARK_END_STRING.equals(line.substring(line.length() - MARK_END_STRING.length()))) {

						if (title.length() > 0) {
							map.put(title, lineList);
						}
						lineList = new LinkedList<String>();
						title = line.substring(MARK_START_STRING.length(), line.length() - MARK_END_STRING.length()).trim();
						continue;
					}

					if (title.length() > 0) {
						lineList.add(line.replaceAll(CHANGE_PRE_CHAR_STRING, CHANGE_AFTER_CHAR_STRING));
					}
				}
				if (title.length() > 0) {
					map.put(title, lineList);
				}

				return map;
			} finally {
				if (br != null) {
					br.close();
				}
			}
		} catch (IOException e) {
			logger.fatal("eLXgNG[^[ɂɗ\ʃG[܂", e);
			throw new InterfaceException(e);
		}
	}

	public static void main(String[] args) {

		try {
			String resName = "/" + HtmlTextCreator.class.getName().replace('.', '/') + suffix;

			HtmlTextCreator htc = new HtmlTextCreator(resName, "UTF-8");

			System.out.println(htc.getText("html"));
			System.out.println(htc.getText("header"));
			System.out.println(htc.getText("header end"));
			System.out.println(htc.getText("body", "title", "eXgł񂪂"));
			System.out.println(htc.getText("single area"));
			for (int row = 1; row < 5; row++) {
				System.out.println(htc.getText("single area field", "row", "" + row, "field-title", "^Cg" + row, "tag-ctrl",
						"<input name=\"aaa\"", "tag-end", ">"));
			}
			System.out.println(htc.getText("single area field end"));
			System.out.println(htc.getText("single area end"));

			System.out.println(htc.getText("multi area"));
			System.out.println(htc.getText("multi area title"));
			System.out.println(htc.getText("multi area title end"));
			System.out.println(htc.getText("multi area row"));
			System.out.println(htc.getText("multi area row field"));
			System.out.println(htc.getText("multi area row field end"));
			System.out.println(htc.getText("multi area row end"));
			System.out.println(htc.getText("multi area end"));
			System.out.println(htc.getText("body end"));
			System.out.println(htc.getText("html end"));

		} catch (InterfaceException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
}
