package jp.oarts.pirka.iop.tool.core.tools.xml;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * VvXMLm[h
 *
 * @author ito
 *
 */
public class XmlCreatorNode {

	/** Agr[g}bv */
	private HashMap<String, String> attributes;

	/** m[h */
	private String nodeName;

	/** eLXgf[^ */
	private String nodeValue;

	/** em[h */
	private XmlCreatorNode parentNode;

	/** qm[hXg */
	protected ArrayList<XmlCreatorNode> childNodes;

	/** iZjm[h */
	private XmlCreatorNode nextSibling;

	/**
	 * RXgN^
	 *
	 * @param nodeName
	 *            m[h
	 */
	public XmlCreatorNode(String nodeName) {
		this(nodeName, "");
	}

	/**
	 * RXgN^
	 *
	 * @param nodeName
	 *            m[h
	 * @param nodeValue
	 *            eLXgf[^
	 */
	public XmlCreatorNode(String nodeName, String nodeValue) {
		this.nodeName = nodeName;
		this.nodeValue = nodeValue;
		this.parentNode = null;
		this.attributes = new HashMap<String, String>();
		this.childNodes = new ArrayList<XmlCreatorNode>();
	}

	/**
	 * RXgN^
	 *
	 * @param nodeName
	 *            m[h
	 * @param nodeValue
	 *            f[^
	 */
	public XmlCreatorNode(String nodeName, int nodeValue) {
		this.nodeName = nodeName;
		this.nodeValue = "" + nodeValue;
		this.parentNode = null;
		this.attributes = new HashMap<String, String>();
		this.childNodes = new ArrayList<XmlCreatorNode>();
	}

	/**
	 * RXgN^
	 *
	 * @param nodeName
	 *            m[h
	 * @param nodeValue
	 *            f[^
	 */
	public XmlCreatorNode(String nodeName, boolean nodeValue) {
		this.nodeName = nodeName;
		if (nodeValue) {
			this.nodeValue = "true";
		} else {
			this.nodeValue = "false";
		}
		this.parentNode = null;
		this.attributes = new HashMap<String, String>();
		this.childNodes = new ArrayList<XmlCreatorNode>();
	}

	/**
	 * Agr[gi[
	 *
	 * @param key
	 *            Agr[gL[
	 * @param value
	 *            Agr[gl
	 */
	public void putAttribute(String key, String value) {
		attributes.put(key, value);
	}

	/**
	 * Agr[g}bvi[
	 *
	 * @param attributes
	 *            Agr[g}bv
	 */
	public void setAttributes(HashMap<String, String> attributes) {
		this.attributes = attributes;
	}

	/**
	 * w肳ꂽL[ɑΉAgr[lo
	 *
	 * @param key
	 *            Agr[gL[
	 * @return Agr[gl
	 */
	public String getAttribute(String key) {
		return this.attributes.get(key);
	}

	/**
	 * Agr[}bvo
	 *
	 * @return Agr[g}bv
	 */
	public HashMap<String, String> getAttributes() {
		return this.attributes;
	}

	/**
	 * m[ĥi[
	 *
	 * @param nodeName	m[h
	 */
	public void setNodeName(String nodeName) {
		this.nodeName = nodeName;
	}

	/**
	 * m[ĥ擾
	 *
	 * @return m[h
	 */
	public String getNodeName() {
		return this.nodeName;
	}

	/**
	 * m[hli[
	 *
	 * @param nodeValue	m[hl
	 */
	public void setNodeValue(String nodeValue) {
		this.nodeValue = nodeValue;
	}

	/**
	 * m[hl擾
	 *
	 * @return m[hl
	 */
	public String getNodeValue() {
		return this.nodeValue;
	}

	/**
	 * em[h擾
	 *
	 * @return em[h
	 */
	public XmlCreatorNode getParentNode() {
		return this.parentNode;
	}

	/**
	 * ŏ̎qm[h擾
	 *
	 * @return ŏ̎qm[h
	 */
	public XmlCreatorNode getFirstChild() {
		if (childNodes.size() <= 0) {
			return null;
		}
		return this.childNodes.get(0);
	}

	/**
	 * ́iZjqm[h擾
	 *
	 * @return	XMLm[h
	 */
	public XmlCreatorNode getNextSibling() {
		return nextSibling;
	}

	/**
	 * qm[hǉ
	 *
	 * @param node
	 *            ǉsȂm[h
	 */
	public void appendChild(XmlCreatorNode node) {
		node.parentNode = this;
		childNodes.add(node);
		if (childNodes.size() > 1) {
			XmlCreatorNode preNode = childNodes.get(childNodes.size() - 2);
			preNode.nextSibling = node;
		}
	}

	/**
	 * qm[h݂邩̊mF
	 *
	 * @return true=qm[hL
	 */
	public boolean hasChildNodes() {
		if (childNodes.size() > 0) {
			return true;
		}
		return false;
	}

	/**
	 * ̃m[hƂ̎q̃m[h𕶎ŕԂ
	 *
	 * @return 
	 */
	public String toString(int deep) {
		String lf = "\n";
		String deepSpace = makeDeepSpace(deep);

		StringBuilder sb = new StringBuilder();
		sb.append(deepSpace);
		sb.append('<');
		sb.append(nodeName);
		for (String attributeName : attributes.keySet()) {
			sb.append(' ');
			sb.append(attributeName);
			sb.append('=');
			sb.append('\"');
			String attribute = attributes.get(attributeName);
			if (attribute != null) {
				sb.append(attribute);
			}
			sb.append('\"');
		}

		if (hasChildNodes()) {
			sb.append(">");
			sb.append(lf);
			for (XmlCreatorNode node : childNodes) {
				sb.append(node.toString(deep + 1));
			}
			sb.append(deepSpace);
			sb.append("</");
			sb.append(nodeName);
			sb.append(">");
		} else {
			if (nodeValue != null && nodeValue.length() > 0) {
				sb.append(">");
				String value = nodeValue.replaceAll("&", "&amp;");
				value = value.replaceAll("<", "&lt;");
				value = value.replaceAll(">", "&gt;");
				value = value.replaceAll("\"", "&quot;");
				value = value.replaceAll("'", "&apos;");
				sb.append(value);
				sb.append("</");
				sb.append(nodeName);
				sb.append(">");
			} else {
				sb.append(" />");
			}
		}
		sb.append(lf);

		return sb.toString();
	}

	/**
	 * [ɑΉ鎚쐬
	 *
	 * @param deep
	 *            [ 0ȏ̐l
	 * @return 
	 */
	private String makeDeepSpace(int deep) {
		if (deep <= 0) {
			return "";
		}

		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < deep; i++) {
			sb.append("    ");
		}
		return sb.toString();
	}
}
