package jp.oarts.pirka.iop.tool.core.tools.xml;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayDeque;
import java.util.Deque;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * XMLp[T[
 */
public class XmlParser {

	public static final String XML_ELEMENT_KEY_SEPARATOR = "/";
	public static final String INDEX_MARKER = XML_ELEMENT_KEY_SEPARATOR + "*";

	/**
	 * XMLp[XXMLf[^쐬
	 *
	 * @param xml
	 *            XML
	 * @return XMLf[^
	 */
	public XmlParserData parse(String xml) {
		return new XmlParserData(parseMain(xml));
	}

	/**
	 * XMLp[Xăf[^}bv쐬
	 *
	 * @param xml
	 *            XML
	 */
	protected LinkedHashMap<String, XmlParserItem> parseMain(String xml) {

		LinkedHashMap<String, XmlParserItem> retMap = null;
		// XML
		try {
			javax.xml.parsers.SAXParser parser = SAXParserFactory.newInstance().newSAXParser();
			CommandHandler handler = new CommandHandler();
			parser.parse(new InputSource(new StringReader(xml)), handler);
			retMap = handler.getDataMap();
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		return retMap;
	}

	/**
	 * CfbNXL[̍쐬
	 *
	 * @param key
	 * @param indexs
	 * @return	L[
	 */
	static public String createKey(String key, int... indexs) {

		int indexMarkerLength = INDEX_MARKER.length();
		for (int index : indexs) {
			int p = key.indexOf(INDEX_MARKER);
			if (p < 0) {
				break;
			}

			if (index <= 0) {
				if (key.length() <= (p + indexMarkerLength)) {
					key = key.substring(0, p);
				} else {
					key = key.substring(0, p) + key.substring(p + indexMarkerLength);
				}
			} else {
				if (key.length() <= (p + indexMarkerLength)) {
					key = key.substring(0, p) + "/" + index;
				} else {
					key = key.substring(0, p) + "/" + index + key.substring(p + indexMarkerLength);
				}
			}
		}

		return key;
	}

	/**
	 * SAXp[T[pnhiXMLHashMapɊi[j
	 */
	class CommandHandler extends DefaultHandler {

		/** XML̃p[^i[}bv */
		private LinkedHashMap<String, XmlParserItem> dataMap;

		/** [v`FbNpnh */
		private HashMap<String, Integer> loopMap;

		/** KwX^bN */
		private Deque<ElementItem> elementStack;

		/** f[^i[p̃XgOobt@ */
		StringBuilder sb = new StringBuilder();

		/**
		 * RXgN^
		 *
		 */
		public CommandHandler() {
			dataMap = new LinkedHashMap<String, XmlParserItem>();
			loopMap = new HashMap<String, Integer>();
			elementStack = new ArrayDeque<ElementItem>();
		}

		@Override
		public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {

			int loopIndex = 0;
			String key = getKey(qName);
			if (loopMap.containsKey(key)) {
				loopIndex = loopMap.get(key);
				loopIndex++;
			}
			loopMap.put(key, loopIndex);

			HashMap<String, String> localAttributeMap = new HashMap<String, String>();
			for (int index = 0; index < attributes.getLength(); index++) {
				localAttributeMap.put(attributes.getQName(index), attributes.getValue(index));
			}

			elementStack.push(new ElementItem(qName, loopIndex));
			sb.setLength(0);

			key = getKey();
			dataMap.put(key, new XmlParserItem(localAttributeMap));

		}

		/*
		 * ( Javadoc)
		 *
		 * @see org.xml.sax.helpers.DefaultHandler#characters(char[], int, int)
		 */
		@Override
		public void characters(char[] ch, int start, int length) throws SAXException {
			sb.append(ch, start, length);
		}

		/*
		 * ( Javadoc)
		 *
		 * @see org.xml.sax.helpers.DefaultHandler#endElement(java.lang.String,
		 * java.lang.String, java.lang.String)
		 */
		@Override
		public void endElement(String uri, String localName, String qName) throws SAXException {
			// System.out.println("endElement " + qName + " = " +
			// sb.toString());

			String key = getKey();
			if (dataMap.containsKey(key)) {
				XmlParserItem xmlItem = dataMap.get(key);
				xmlItem.setValue(sb.toString());
			} else {
				XmlParserItem xmlItem = new XmlParserItem(sb.toString());
				dataMap.put(key, xmlItem);
			}

			elementStack.pop();
			sb.setLength(0);
		}

		/**
		 * XML̃p[^i[}bv擾
		 *
		 * @return XML̃p[^i[}bv
		 */
		public LinkedHashMap<String, XmlParserItem> getDataMap() {
			return dataMap;
		}

		/**
		 * X^bNɐςłGgɌ݂̃L[쐬擾
		 *
		 * @return L[
		 */
		private String getKey() {

			StringBuilder sb = new StringBuilder();

			Iterator<ElementItem> ie = elementStack.descendingIterator();
			while (ie.hasNext()) {
				ElementItem elementItem = ie.next();
				if (sb.length() > 0) {
					sb.append(XML_ELEMENT_KEY_SEPARATOR);
				}
				sb.append(elementItem.getName());
				int loopIndex = elementItem.getLoopIndex();
				if (loopIndex > 0) {
					sb.append(XML_ELEMENT_KEY_SEPARATOR);
					sb.append(loopIndex);
				}
			}

			return sb.toString();
		}

		/**
		 * X^bNɐςłGg{p[^Ŏw肳ꂽGgL[쐬擾
		 *
		 * @param elementName
		 *            Gg
		 * @return L[
		 */
		private String getKey(String elementName) {

			StringBuilder sb = new StringBuilder();
			sb.append(getKey());
			sb.append(XML_ELEMENT_KEY_SEPARATOR);
			sb.append(elementName);
			return sb.toString();
		}

		/**
		 * GgACe
		 *
		 * @author ito
		 *
		 */
		class ElementItem {
			private String name;

			private int loopIndex;

			public ElementItem(String name, int loopIndex) {
				this.name = name;
				this.loopIndex = loopIndex;
			}

			/**
			 * @return loopIndex
			 */
			public int getLoopIndex() {
				return loopIndex;
			}

			/**
			 * @return name
			 */
			public String getName() {
				return name;
			}

		}
	}

}