package jp.oarts.pirka.iop.tool.web.in;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.TreeMap;

import jp.oarts.pirka.core.ctrl.SelectList;
import jp.oarts.pirka.core.kernel.FieldMap;
import jp.oarts.pirka.core.win.PirkaWindow;
import jp.oarts.pirka.iop.tool.core.business.AttributeItem;
import jp.oarts.pirka.iop.tool.core.business.AttributeSimpleItemAlwaysDefaultValue;
import jp.oarts.pirka.iop.tool.core.business.FileCreatorSampleModelerPlugin;
import jp.oarts.pirka.iop.tool.core.business.InterfaceData;
import jp.oarts.pirka.iop.tool.core.business.InterfaceDataManager;
import jp.oarts.pirka.iop.tool.core.business.InterfaceException;
import jp.oarts.pirka.iop.tool.core.business.InterfaceProject;
import jp.oarts.pirka.iop.tool.core.business.PluginManager;
import jp.oarts.pirka.iop.tool.core.general.constants.AttributeType;
import jp.oarts.pirka.iop.tool.core.general.vo.AttributeData;
import jp.oarts.pirka.iop.tool.core.general.vo.InterfaceDataItem;
import jp.oarts.pirka.iop.tool.web.common.IopBaseWindow;
import jp.oarts.pirka.iop.tool.web.constants.Define;
import jp.oarts.pirka.iop.tool.web.tools.IopUtil;

public class AttachAttribute extends IopBaseWindow {

	/**
	 *
	 */
	private static final long serialVersionUID = -4522943256446547395L;

	private int dataId;

	private List<Integer> includeIdList;

	private List<String> attachedList;

	private InterfaceProject interfaceProject;

	private boolean useDisableItemsAttributeFlag = false;

	/**
	 * RXgN^
	 *
	 * @param interfaceProject
	 *            C^tF[XvWFNg
	 */
	public AttachAttribute(InterfaceProject interfaceProject, int dataId) {
		super();
		this.dataId = dataId;
		this.interfaceProject = interfaceProject;
		this.includeIdList = new ArrayList<Integer>();
		this.attachedList = new ArrayList<String>();

		renew();
	}

	public void preDraw(){
		super.preDraw();
		renew();
	}

	/**
	 * ꗗXV
	 */
	public void renew() {
		try {
			includeIdList.clear();
			attachedList.clear();

			FieldMap fm = getFieldMap();
			InterfaceDataManager interfaceDataManager = interfaceProject.getInterfaceDataManager();
			InterfaceData interfaceData = interfaceDataManager.getInterfaceData(dataId);

			fm.clearFieldMap("attachNameLoop");
			fm.clearFieldMap("attachCol");
			fm.clearFieldMap("attachKeyCol");
			fm.clearFieldMap("listRow");

			SelectList selectAttachList = new SelectList();
			FileCreatorSampleModelerPlugin[] attachListWk = PluginManager.getInstance().getAttachPluginList();

			for (FileCreatorSampleModelerPlugin plugin : attachListWk) {
				if (plugin.canAttach() && !interfaceData.isAttached(plugin.getName())) {
					selectAttachList.add(plugin.getNameJp(), plugin.getName());
				}
			}
			fm.setSelectList("attachList", selectAttachList);
			fm.setValue("attachList", "");

			useDisableItemsAttributeFlag = false;

			if (interfaceData.getAttachList().length > 0) {

				for (String attachName : interfaceData.getAttachList()) {
					FileCreatorSampleModelerPlugin attribute = PluginManager.getInstance().getAttachPlugin(attachName);
					if (attribute != null) {
						FieldMap attachFieldMap = createLoopFieldMap("attachNameLoop");
						fm.addFieldMap("attachNameLoop", attachFieldMap);

						attachFieldMap.setValue("attachName", attribute.getNameJp());
						attachFieldMap.setOption("attachName", "title", attribute.getAttributeComment());
						attachedList.add(attribute.getName());

						useDisableItemsAttributeFlag |= attribute.useDisableItemsAttribute();
					}
				}
			}

			DecimalFormat fmt = new DecimalFormat("0000");
			fm.setValue("myInfo", interfaceData.getName() + " ( " + fmt.format(interfaceData.getDataId()) + " : "
					+ interfaceData.getNameJp() + " )");

			for (String attachName : interfaceData.getAttachList()) {
				FileCreatorSampleModelerPlugin attribute = PluginManager.getInstance().getAttachPlugin(attachName);
				if (attribute != null) {

					AttributeItem[] items = attribute.getAttributeItems();
					if (items.length > 0) {

						FieldMap attachColFieldMap = createLoopFieldMap("attachCol");
						fm.addFieldMap("attachCol", attachColFieldMap);

						attachColFieldMap.setOption("attachTitlecolspan", "colspan", "" + items.length);
						attachColFieldMap.setValue("attachTitleValue", attribute.getNameJp());
						attachColFieldMap.setOption("attachTitleValue", "title", attribute.getAttributeComment());

						for (AttributeItem item : items) {
							FieldMap attachKeyColFieldMap = createLoopFieldMap("attachKeyCol");
							fm.addFieldMap("attachKeyCol", attachKeyColFieldMap);
							attachKeyColFieldMap.setValue("attachTitleKeyName", item.getNameJp());
							attachKeyColFieldMap.setOption("attachTitleKeyName", "title", item.getMessage());
							if (item instanceof AttributeSimpleItemAlwaysDefaultValue) {
								attachKeyColFieldMap.setOption("colStyle", "style", "background-color:#e0f0e0");
							} else {
								attachKeyColFieldMap.setOption("colStyle", "style", "background-color:transparent");
							}
						}

					}

				}
			}

			int no = 0;

			Integer[] itemIds;
			if (useDisableItemsAttributeFlag) {
				itemIds = interfaceData.getAllItemIdList();
			} else {
				itemIds = interfaceData.getEnableItemIdList();
			}

			// for (Integer itemId : interfaceData.getEnableItemIdList()) {
			for (Integer itemId : itemIds) {
				no++;
				InterfaceDataItem item = interfaceData.getItem(itemId);

				FieldMap listRowFieldMap = createLoopFieldMap("listRow");
				fm.addFieldMap("listRow", listRowFieldMap);

				if (item.isEnable()) {
					listRowFieldMap.setOption("lineStyle", "style", "background-color:transparent");
				} else {
					listRowFieldMap.setOption("lineStyle", "style", "background-color:#c0c0c0");
				}

				listRowFieldMap.setValue("no", no);
				if (item.getIncludeId() > 0) {

					InterfaceData includeInterfaceData = interfaceDataManager.getInterfaceData(item.getIncludeDataId());
					if (includeInterfaceData != null) {
						setString(listRowFieldMap, "inc", "" + item.getIncludeId() + ":" + includeInterfaceData.getName() + "("
								+ includeInterfaceData.getNameJp() + ")", "<br>");
					} else {
						setString(listRowFieldMap, "inc", "" + item.getIncludeId() + ":폜Ă܂", "<br>");
					}

				} else {
					setString(listRowFieldMap, "inc", "", "<br>");
				}
				setString(listRowFieldMap, "name", item.getName(), "<br>");
				setString(listRowFieldMap, "nameJp", item.getNameJp(), "<br>");

				setString(listRowFieldMap, "type", item.getType().toString(), "<br>");

				if (item.getLength() > 0) {
					listRowFieldMap.setValue("length", item.getLength());
				} else {
					setString(listRowFieldMap, "length", null, "<br>");
				}

				if (item.getSubLength() > 0) {
					listRowFieldMap.setValue("subLength", item.getSubLength());
				} else {
					setString(listRowFieldMap, "subLength", null, "<br>");
				}
				setString(listRowFieldMap, "primaryKey", item.isPrimaryKey() ? "" : " ", "<br>");

				if (!item.isBaseItem()) {
					InterfaceData includeData = interfaceProject.getInterfaceDataManager().getInterfaceData(
							item.getIncludeDataId());
					InterfaceDataItem includeItem = includeData.getItem(item.getIncludeItemId());

					String includeType = "";
					if (includeData != null && includeItem.getType() != null) {
						includeType = includeItem.getType().toString();
						if (includeItem.getLength() != 0) {
							if (includeItem.getSubLength() != 0) {
								includeType = includeType + "(" + includeItem.getLength() + ", " + includeItem.getSubLength()
										+ ")";
							} else {
								includeType = includeType + "(" + includeItem.getLength() + ")";
							}
						}
					}

					setString(listRowFieldMap, "includeName", includeItem.getName(), "<br>");
					setString(listRowFieldMap, "includeNameJp", includeItem.getNameJp(), "<br>");
					setString(listRowFieldMap, "includeType", includeType, "<br>");
					setString(listRowFieldMap, "includePrimaryKey", includeItem.isPrimaryKey() ? "" : " ", "<br>");

				} else {
					setString(listRowFieldMap, "includeName", null, "<br>");
					setString(listRowFieldMap, "includeNameJp", null, "<br>");
					setString(listRowFieldMap, "includeType", null, "<br>");
					setString(listRowFieldMap, "includePrimaryKey", null, "<br>");
				}

				for (String attachName : interfaceData.getAttachList()) {
					FileCreatorSampleModelerPlugin attribute = PluginManager.getInstance().getAttachPlugin(attachName);
					if (attribute != null) {

						AttributeItem[] items = attribute.getAttributeItems();
						if (items.length > 0) {
							for (AttributeItem aItem : items) {

								String style = "background-color:transparent";
								if (aItem instanceof AttributeSimpleItemAlwaysDefaultValue) {
									style = "background-color:#e0f0e0";
								}

								FieldMap attachValueColFieldMap = createLoopFieldMap("attachValueCol");
								listRowFieldMap.addFieldMap("attachValueCol", attachValueColFieldMap);

								attachValueColFieldMap.setValue("checkSw", false);
								attachValueColFieldMap.setValue("TextSw", false);
								attachValueColFieldMap.setValue("fileSw", false);
								attachValueColFieldMap.setValue("selectSw", false);
								attachValueColFieldMap.setValue("textAreaSw", false);
								attachValueColFieldMap.setValue("textNoneSw", false);

								if (attribute.useDisableItemsAttribute() || item.isEnable()) {
									switch (aItem.getAttributeType()) {
									case TEXT:
										attachValueColFieldMap.setValue("TextSw", true);
										break;
									case INTERFACE:
									case SELECT:
										attachValueColFieldMap.setValue("selectSw", true);
										break;
									case TEXT_AREA:
										attachValueColFieldMap.setValue("textAreaSw", true);
										break;
									case CHECK:
										attachValueColFieldMap.setValue("checkSw", true);
										break;

									}

									AttributeData value = null;
									if (!aItem.isAlwaysDefaultValue()) {
										value = interfaceData.getAttribute(item.getItemId(), attachName, aItem.getName());
									}

									String fieldName = getFieldName(aItem.getAttributeType());

									switch (aItem.getAttributeType()) {
									case TEXT:
										if (value == null) {
											attachValueColFieldMap.setValue(fieldName, "");
										} else {
											attachValueColFieldMap.setValue(fieldName, value.getValue());
										}
										if (aItem.getLength() > 0) {
											attachValueColFieldMap.setOption(fieldName, "maxlength", "" + aItem.getLength());
											attachValueColFieldMap.setOption(fieldName, "size", "" + aItem.getLength() * 2);
										} else {
											attachValueColFieldMap.clearOption(fieldName, "length");
											attachValueColFieldMap.clearOption(fieldName, "size");
										}
										attachValueColFieldMap.setOption(fieldName, "title", aItem.getMessage());
										attachValueColFieldMap.setOption(getColStyleFieldName(aItem.getAttributeType()),
												"style", style);
										break;
									case INTERFACE:
										attachValueColFieldMap.setSelectList(fieldName, IopUtil
												.getInterfaceDataSelecter(interfaceProject));
										if (value == null) {
											attachValueColFieldMap.setValue(fieldName, "");
										} else {
											attachValueColFieldMap.setValue(fieldName, value.getValue());
										}
										if (aItem.getLength() > 0) {
											attachValueColFieldMap.setOption(fieldName, "size", "" + aItem.getLength());
										} else {
											attachValueColFieldMap.clearOption(fieldName, "size");
										}
										attachValueColFieldMap.setOption(fieldName, "title", aItem.getMessage());
										attachValueColFieldMap.setOption(getColStyleFieldName(aItem.getAttributeType()),
												"style", style);
										break;
									case SELECT:
										attachValueColFieldMap.setSelectList(fieldName, IopUtil.convartSelectList(aItem
												.getSelecter()));
										if (value == null) {
											attachValueColFieldMap.setValue(fieldName, "");
										} else {
											attachValueColFieldMap.setValue(fieldName, value.getValue());
										}
										if (aItem.getLength() > 0) {
											attachValueColFieldMap.setOption(fieldName, "size", "" + aItem.getLength());
										} else {
											attachValueColFieldMap.clearOption(fieldName, "size");
										}
										attachValueColFieldMap.setOption(fieldName, "title", aItem.getMessage());
										attachValueColFieldMap.setOption(getColStyleFieldName(aItem.getAttributeType()),
												"style", style);
										break;
									case TEXT_AREA:
										if (value == null) {
											attachValueColFieldMap.setValue(fieldName, "");
										} else {
											attachValueColFieldMap.setValue(fieldName, value.getValue());
										}
										attachValueColFieldMap.setOption(fieldName, "title", aItem.getMessage());
										attachValueColFieldMap.setOption(getColStyleFieldName(aItem.getAttributeType()),
												"style", style);
										break;
									case CHECK:
										if (value == null) {
											attachValueColFieldMap.setValue(fieldName, false);
										} else {
											attachValueColFieldMap.setValue(fieldName, value.getValueBoolean());
										}
										attachValueColFieldMap.setOption(fieldName, "title", aItem.getMessage());
										attachValueColFieldMap.setOption(getColStyleFieldName(aItem.getAttributeType()),
												"style", style);
										break;

									}
								} else {
									attachValueColFieldMap.setValue("textNoneSw", true);
								}
							}
						}
					}
				}
			}
		} catch (InterfaceException e) {
			throw new RuntimeException(e);
		}
	}

	public void addAttach() {
		try {
			keepData(false);

			FieldMap fm = getFieldMap();
			String name = fm.getValueString("attachList");
			if (name == null || name.length() <= 0) {
				addError("A^b`IĂ܂");
				fm.setError("attachList", true);
				return;

			}
			interfaceProject.getInterfaceDataManager().getInterfaceData(dataId).attach(name);

			renew();
		} catch (InterfaceException e) {
			throw new RuntimeException(e);
		}

	}

	/**
	 * f^b`{^NbNꂽ̏
	 */
	public void deleteAttach() {
		try {
			keepData(false);

			int[] indexs = getActionLoopIndex();
			interfaceProject.getInterfaceDataManager().getInterfaceData(dataId).dettach(attachedList.get(indexs[0]));

			renew();
		} catch (InterfaceException e) {
			throw new RuntimeException(e);
		}
	}

	private boolean keepData(boolean execCheck) {
		try {
			boolean error = false;
			int rowCount = 0;

			FieldMap fm = getFieldMap();
			InterfaceDataManager interfaceDataManager = interfaceProject.getInterfaceDataManager();
			InterfaceData interfaceData = interfaceDataManager.getInterfaceData(dataId);

			HashMap<String, TreeMap<Integer, HashMap<String, AttributeData>>> allDataMap = new HashMap<String, TreeMap<Integer, HashMap<String, AttributeData>>>();

			int index1 = 0;
			Integer[] itemIds;
			if (useDisableItemsAttributeFlag) {
				itemIds = interfaceData.getAllItemIdList();
			} else {
				itemIds = interfaceData.getEnableItemIdList();
			}

			for (Integer itemId : itemIds) {
				InterfaceDataItem item = interfaceData.getItem(itemId);

				FieldMap listRowFieldMap = fm.getFieldMap("listRow", index1++);

				rowCount++;
				int index2 = 0;
				for (String attachName : interfaceData.getAttachList()) {
					FileCreatorSampleModelerPlugin attribute = PluginManager.getInstance().getAttachPlugin(attachName);
					if (attribute != null) {

						AttributeItem[] items = attribute.getAttributeItems();
						if (items.length > 0) {
							int startInex = index2;
							boolean errorInPlugin = false;

							HashMap<String, AttributeData> data = new HashMap<String, AttributeData>();

							for (AttributeItem aItem : items) {

								FieldMap attachValueColFieldMap = listRowFieldMap.getFieldMap("attachValueCol", index2++);

								String fieldName = getFieldName(aItem.getAttributeType());
								AttributeData value = null;

								if (!attribute.useDisableItemsAttribute() && !item.isEnable()) {
									continue;
								}

								switch (aItem.getAttributeType()) {
								case TEXT:
									value = new AttributeData(attachValueColFieldMap.getValueString(fieldName));
									break;
								case INTERFACE:
									value = new AttributeData(attachValueColFieldMap.getValueString(fieldName));
									break;
								case SELECT:
									value = new AttributeData(attachValueColFieldMap.getValueString(fieldName));
									break;
								case TEXT_AREA:
									value = new AttributeData(attachValueColFieldMap.getValueString(fieldName));
									break;
								case CHECK:
									value = new AttributeData(attachValueColFieldMap.getValueBoolean(fieldName));
									break;

								}

								data.put(aItem.getName(), value);
								if (execCheck) {
									try {
										aItem.check(value.getValue(), item);
									} catch (InterfaceException e) {

										error = true;
										errorInPlugin = true;
										addError(rowCount + "sځF" + e.getMessage());
										if (fieldName.length() > 0) {
											attachValueColFieldMap.setError(fieldName, true);
										}
									}
								}
								if (!aItem.isAlwaysDefaultValue()) {
									interfaceData.setAttribute(itemId, attribute.getName(), aItem.getName(), value);
								} else {
									interfaceData.setAttribute(itemId, attribute.getName(), aItem.getName(), aItem
											.getDefaultValue());
								}
							}

							if (execCheck) {
								if (!errorInPlugin) {

									if (attribute.useDisableItemsAttribute() || item.isEnable()) {

										try {

											TreeMap<Integer, HashMap<String, AttributeData>> aData = allDataMap.get(attachName);
											if (aData == null) {
												aData = new TreeMap<Integer, HashMap<String, AttributeData>>();
												allDataMap.put(attachName, aData);
											}
											aData.put(itemId, data);

											attribute.checkAttribute(dataId, itemId, data, interfaceProject);
										} catch (InterfaceException e) {
											error = true;
											addError(rowCount + "sځF" + e.getMessage());

											int indexWk = startInex;
											for (AttributeItem aItem : items) {

												FieldMap attachValueColFieldMap = listRowFieldMap.getFieldMap("attachValueCol",
														indexWk++);
												attachValueColFieldMap.setError(getFieldName(aItem.getAttributeType()), true);
											}

										}
									}
								}
							}
						}

					}
				}

			}

			if (!error && execCheck) {

				for (String attachName : interfaceData.getAttachList()) {
					FileCreatorSampleModelerPlugin attribute = PluginManager.getInstance().getAttachPlugin(attachName);
					if (attribute != null) {
						try {
							attribute.preCheckAttribute(dataId, allDataMap, interfaceProject);
							attribute.checkAttribute(dataId, allDataMap, interfaceProject);
							attribute.afterCheckAttribute(dataId, allDataMap, interfaceProject);
						} catch (InterfaceException e) {
							error = true;
							addError(e.getMessage());
						}

					}
				}

			}

			return !error;
		} catch (InterfaceException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 *
	 * @param type
	 * @return
	 */
	private String getFieldName(AttributeType type) {
		String fieldName = "";
		switch (type) {
		case TEXT:
			fieldName = "attributeText";
			break;
		case INTERFACE:
			fieldName = "attributeSelect";
			break;
		case SELECT:
			fieldName = "attributeSelect";
			break;
		case TEXT_AREA:
			fieldName = "attributeTextArea";
			break;
		case CHECK:
			fieldName = "attributeCheckBox";
		}
		return fieldName;
	}

	/**
	 *
	 * @param type
	 * @return
	 */
	private String getColStyleFieldName(AttributeType type) {
		String fieldName = "";
		switch (type) {
		case TEXT:
			fieldName = "textStyle";
			break;
		case INTERFACE:
			fieldName = "selectStyle";
			break;
		case SELECT:
			fieldName = "selectStyle";
			break;
		case TEXT_AREA:
			fieldName = "textAreaStyle";
			break;
		case CHECK:
			fieldName = "checkStyle";
		}
		return fieldName;
	}

	public PirkaWindow changeButton() {

		int[] index = getActionLoopIndex();

		try {
			InterfaceData interfaceData = interfaceProject.getInterfaceDataManager().getInterfaceData(dataId);

			Integer[] itemIds;
			if (useDisableItemsAttributeFlag) {
				itemIds = interfaceData.getAllItemIdList();
			} else {
				itemIds = interfaceData.getEnableItemIdList();
			}

			int itemId = itemIds[index[0]];
			InterfaceDataItem item = interfaceData.getItem(itemId);

			keepData(false);
			if(item.isBaseItem()){
				return new EditInterfaceItem( dataId,  itemId, interfaceProject);
			}else{
				return new EditInterfaceIncludeItem(dataId, itemId, item.getIncludeId(), interfaceProject);
			}

		} catch (InterfaceException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * regist{^
	 *
	 * @return EBhE
	 */
	public PirkaWindow regist() {
		if (keepData(true)) {
			interfaceProject.setChanged(true);
			putSession(Define.PROJECT_SESSION_NAME, interfaceProject);
			return null;
		}

		return this;
	}

	/**
	 * I{^NbNꂽ̏
	 *
	 * @return nulliOʁj
	 */
	public PirkaWindow back() {

		return null;
	}

}
