package jp.oarts.pirka.iop.tool.web.in;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

import jp.oarts.pirka.core.kernel.FieldMap;
import jp.oarts.pirka.core.win.PirkaWindow;
import jp.oarts.pirka.iop.tool.core.business.InterfaceData;
import jp.oarts.pirka.iop.tool.core.business.InterfaceDataManager;
import jp.oarts.pirka.iop.tool.core.business.InterfaceException;
import jp.oarts.pirka.iop.tool.core.business.InterfaceProject;
import jp.oarts.pirka.iop.tool.core.general.vo.InterfaceDataItem;
import jp.oarts.pirka.iop.tool.web.common.IopBaseWindow;
import jp.oarts.pirka.iop.tool.web.constants.Define;

public class EditInterface extends IopBaseWindow {

	/**
	 *
	 */
	private static final long serialVersionUID = -4522943256446547395L;

	private int dataId;

	private List<Integer> displayItemIdList;

	/**
	 * RXgN^
	 *
	 * @param dataId
	 *            f[^ID
	 */
	public EditInterface(int dataId) {
		super();

		this.dataId = dataId;

		displayItemIdList = new ArrayList<Integer>();

		renew();
	}

	public void preDraw() {
		super.preDraw();
		renew();
	}

	/**
	 * ꗗXV
	 */
	public void renew() {
		try {
			displayItemIdList.clear();

			FieldMap fm = getFieldMap();

			InterfaceDataManager interfaceDataManager = ((InterfaceProject) getSession(Define.PROJECT_SESSION_NAME))
					.getInterfaceDataManager();
			InterfaceData interfaceData = interfaceDataManager.getInterfaceData(dataId);

			fm.clearFieldMap("listRow");

			DecimalFormat fmt = new DecimalFormat("0000");
			fm.setValue("myInfo", interfaceData.getName() + " ( " + fmt.format(interfaceData.getDataId()) + " : "
					+ interfaceData.getNameJp() + " )");

			int no = 0;
			for (Integer itemId : interfaceData.getAllItemIdList()) {
				no++;
				InterfaceDataItem item = interfaceData.getItem(itemId);

				displayItemIdList.add(item.getItemId());

				FieldMap listRowFieldMap = createLoopFieldMap("listRow");
				fm.addFieldMap("listRow", listRowFieldMap);

				if (item.isEnable()) {
					listRowFieldMap.setOption("lineStyle", "style", "background-color:transparent");
				} else {
					listRowFieldMap.setOption("lineStyle", "style", "background-color:#c0c0c0");
				}

				listRowFieldMap.setValue("no", no);
				setString(listRowFieldMap, "name", item.getName(), "<br>");
				setString(listRowFieldMap, "nameJp", item.getNameJp(), "<br>");

				// if(!item.isMultiRecordType()){
				setString(listRowFieldMap, "type", item.getType().toString(), "<br>");

				if (item.getLength() > 0) {
					listRowFieldMap.setValue("length", item.getLength());
				} else {
					setString(listRowFieldMap, "length", null, "<br>");
				}

				if (item.getSubLength() > 0) {
					listRowFieldMap.setValue("subLength", item.getSubLength());
				} else {
					setString(listRowFieldMap, "subLength", null, "<br>");
				}

				String primaryKey = "-";
				if(item.isPrimaryKey()){
					primaryKey = "";
				}
				listRowFieldMap.setValue("primaryKey", primaryKey);

				// }else{
				// setString(listRowFieldMap, "type", "LIST", "<br>");
				// setString(listRowFieldMap, "length", null, "<br>");
				// setString(listRowFieldMap, "subLength", null, "<br>");
				// }

				String includeUse = "";
				String includeNo = "";
				String includeName = "";
				String includeNameJp = "";
				String includeType = "";
				String includePrimaryKey = "";

				if (!item.isBaseItem()) {
					if (item.isEnable()) {
						includeUse = "";
					} else {
						includeUse = "-";
					}

					InterfaceData includeData = interfaceDataManager.getInterfaceData(item.getIncludeDataId());
					InterfaceDataItem includeDataItem = includeData.getItem(item.getIncludeItemId());

					if (item.getIncludeId() != 0) {
						if (includeData != null) {
							includeNo = "" + item.getIncludeId() + ":" + includeData.getName() + "(" + includeData.getNameJp()
									+ ")";
						} else {
							includeNo = "" + item.getIncludeId();
						}
					}

					if (includeData != null && includeDataItem.getName() != null) {
						includeName = includeDataItem.getName();
					}
					if (includeData != null && includeDataItem.getNameJp() != null) {
						includeNameJp = includeDataItem.getNameJp();
					}

					if (includeData != null && includeDataItem.getType() != null) {
						includeType = includeDataItem.getType().toString();
						if (includeDataItem.getLength() != 0) {
							if (includeDataItem.getSubLength() != 0) {
								includeType = includeType + "(" + includeDataItem.getLength() + ", "
										+ includeDataItem.getSubLength() + ")";
							} else {
								includeType = includeType + "(" + includeDataItem.getLength() + ")";
							}
						}
					}
					if(includeDataItem.isPrimaryKey()){
						includePrimaryKey = "";
					}else{
						includePrimaryKey = "-";
					}
				}
				setString(listRowFieldMap, "includeUse", includeUse, "<br>");
				setString(listRowFieldMap, "includeNo", includeNo, "<br>");
				setString(listRowFieldMap, "includeName", includeName, "<br>");
				setString(listRowFieldMap, "includeNameJp", includeNameJp, "<br>");
				setString(listRowFieldMap, "includeType", includeType, "<br>");
				setString(listRowFieldMap, "includePrimaryKey", includePrimaryKey, "<br>");

				if (item.isBaseItem()) {
					listRowFieldMap.setValue("deleteSw1", true);
					listRowFieldMap.setValue("deleteSw2", false);
				} else {
					listRowFieldMap.setValue("deleteSw1", false);
					listRowFieldMap.setValue("deleteSw2", true);
				}

			}
		} catch (InterfaceException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * ڒǉ{^NbNꂽ̏
	 *
	 * @return EBhE
	 */
	public PirkaWindow addItem() {
		return new EditInterfaceItem(dataId, null);
	}

	/**
	 * Abv{^NbNꂽ̏
	 */
	public void upButton() {
		try {
			int[] indexs = getActionLoopIndex();
			InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);
			InterfaceDataManager interfaceDataManager = interfaceProject.getInterfaceDataManager();
			interfaceDataManager.getInterfaceData(dataId).upInterfaceItem(displayItemIdList.get(indexs[0]));
			interfaceProject.setChanged(true);

			renew();
		} catch (InterfaceException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * _E{^NbNꂽ̏
	 */
	public void downButton() {
		try {
			int[] indexs = getActionLoopIndex();
			InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);
			InterfaceDataManager interfaceDataManager = interfaceProject.getInterfaceDataManager();
			interfaceDataManager.getInterfaceData(dataId).downInterfaceItem(displayItemIdList.get(indexs[0]));
			interfaceProject.setChanged(true);

			renew();
		} catch (InterfaceException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * ڕҏW{^NbNꂽ̏
	 *
	 * @return EBhE
	 */
	public PirkaWindow update() {
		try {
			int[] indexs = getActionLoopIndex();
			int itemId = displayItemIdList.get(indexs[0]);
			InterfaceDataManager interfaceDataManager = ((InterfaceProject) getSession(Define.PROJECT_SESSION_NAME))
					.getInterfaceDataManager();
			InterfaceDataItem item = interfaceDataManager.getInterfaceData(dataId).getItem(itemId);

			if (item.isBaseItem()) {
				return new EditInterfaceItem(dataId, itemId);
			} else {
				return new EditInterfaceIncludeItem(dataId, itemId, item.getIncludeId());
			}
		} catch (InterfaceException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * ڍ폜{^NbNꂽ̏
	 */
	public void delete() {
		try {
			int[] indexs = getActionLoopIndex();
			InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);
			InterfaceDataManager interfaceDataManager = interfaceProject.getInterfaceDataManager();
			interfaceDataManager.getInterfaceData(dataId).removeMyItem(displayItemIdList.get(indexs[0]));
			interfaceProject.setChanged(true);

			renew();
		} catch (InterfaceException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * I{^NbNꂽ̏
	 *
	 * @return nulliOʁj
	 */
	public PirkaWindow back() {
		return null;
	}

}
