package jp.oarts.pirka.iop.tool.web.in;

import java.text.DecimalFormat;

import jp.oarts.pirka.core.ctrl.SelectList;
import jp.oarts.pirka.core.kernel.FieldMap;
import jp.oarts.pirka.core.util.check.CheckAndTeller;
import jp.oarts.pirka.core.util.check.PirkaCheckerString;
import jp.oarts.pirka.core.util.check.PirkaCheckerStringChars;
import jp.oarts.pirka.core.win.PirkaWindow;
import jp.oarts.pirka.iop.tool.core.business.InterfaceData;
import jp.oarts.pirka.iop.tool.core.business.InterfaceException;
import jp.oarts.pirka.iop.tool.core.business.InterfaceProject;
import jp.oarts.pirka.iop.tool.core.general.constants.FieldType;
import jp.oarts.pirka.iop.tool.core.general.vo.IncludeInfo;
import jp.oarts.pirka.iop.tool.core.general.vo.InterfaceDataItem;
import jp.oarts.pirka.iop.tool.web.common.IopBaseWindow;
import jp.oarts.pirka.iop.tool.web.constants.Define;

public class EditInterfaceIncludeItem extends IopBaseWindow {

	/**
	 *
	 */
	private static final long serialVersionUID = -4522943256446547395L;

	private InterfaceProject interfaceProject;

	private Integer dataId;

	private InterfaceDataItem dataItem;

	/**
	 * RXgN^
	 */
	public EditInterfaceIncludeItem(Integer dataId, Integer itemId, int includeId) {
		super();
		init(dataId, itemId, includeId, ((InterfaceProject) getSession(Define.PROJECT_SESSION_NAME)));
	}

	/**
	 * RXgN^
	 */
	public EditInterfaceIncludeItem(Integer dataId, Integer itemId, int includeId, InterfaceProject interfaceProject) {
		super();
		init(dataId, itemId, includeId, interfaceProject);

		FieldMap fm = getFieldMap();
		fm.setValue("regist", "ύX");
		fm.setValue("back", "");
	}

	/**
	 * RXgN^
	 */
	protected void init(Integer dataId, Integer itemId, int includeId, InterfaceProject interfaceProject) {
		try {
			this.interfaceProject = interfaceProject;
			FieldMap fm = getFieldMap();

			// `FbJ[ݒ
			addCheck("inName", new PirkaCheckerString(PirkaCheckerStringChars.NUM, PirkaCheckerStringChars.LOW_ALPHABET,
					PirkaCheckerStringChars.HIGHT_ALPHABET));
			addCheck(new CheckAndTeller() {

				private static final long serialVersionUID = 2250657920812455835L;

				@Override
				public boolean check() {
					FieldMap fm = getFieldMap();
					if (fm.getValueBoolean("ckName")) {
						String value = fm.getValueString("inName");
						if (value == null || value.length() <= 0) {
							addError("ږ", "li[Ă");
							fm.setError("inName", true);
							return false;
						}
					}

					return true;
				}
			});
			addCheck(new CheckAndTeller() {
				private static final long serialVersionUID = -4551831504413502378L;

				@Override
				public boolean check() {
					FieldMap fm = getFieldMap();
					if (fm.getValueBoolean("ckNameJp")) {
						String value = fm.getValueString("inNameJp");
						if (value == null || value.length() <= 0) {
							addError("ږi{ꖼj", "li[Ă");
							fm.setError("inNameJp", true);
							return false;
						}
					}

					return true;
				}
			});

			addErrorCheckReturner();

			fm.setOfficialFieldName("inName", "ږ");
			fm.setOfficialFieldName("inNameJp", "ږi{ꖼj");
			fm.setOfficialFieldName("inType", "^");
			fm.setOfficialFieldName("inLength", "ڒ");
			fm.setOfficialFieldName("inSubLength", "");
			fm.setOfficialFieldName("primaryKey", "L[");

			fm.setAutoCheck("back", false);

			SelectList selectList = new SelectList();
			for (FieldType type : FieldType.values()) {
				selectList.add(type.toString(), type.toString());
			}
			fm.setSelectList("inType", selectList);

			this.dataId = dataId;

			dataItem = interfaceProject.getInterfaceDataManager().getInterfaceData(dataId).getItem(itemId);
			if (dataItem.isUseName()) {
				fm.setValue("inName", dataItem.getName());
			} else {
				fm.setValue("inName", "");
			}

			if (dataItem.isUseNameJp()) {
				fm.setValue("inNameJp", dataItem.getNameJp());
			} else {
				fm.setValue("inNameJp", "");
			}

			if (dataItem.isUseType()) {
				fm.setValue("inType", dataItem.getType().toString());
			} else {
				fm.setValue("inType", "");
			}

			if (dataItem.isUseLength()) {
				fm.setValue("inLength", dataItem.getLength());
				fm.setValue("inSubLength", dataItem.getSubLength());
			} else {
				fm.setValue("inLength", "");
				fm.setValue("inSubLength", "");
			}

			fm.setValue("ckEnable", dataItem.isEnable());
			fm.setValue("ckName", dataItem.isUseName());
			fm.setValue("ckNameJp", dataItem.isUseNameJp());
			fm.setValue("ckType", dataItem.isUseType());
			fm.setValue("ckLength", dataItem.isUseLength());
			fm.setValue("primaryKey", dataItem.isPrimaryKey());

			IncludeInfo includeInfo = interfaceProject.getInterfaceDataManager().getInterfaceData(dataId).getIncludeIfo(
					includeId);
			if (includeInfo != null) {
				fm.setValue("incNo", includeInfo.getIncludeId());

				DecimalFormat fmt = new DecimalFormat("0000");
				InterfaceData includeData = interfaceProject.getInterfaceDataManager().getInterfaceData(
						includeInfo.getIncludeDataId());
				if (includeData != null) {
					fm.setValue("includeInterfaceName", includeData.getName() + " ("
							+ fmt.format(includeInfo.getIncludeDataId()) + " : " + includeData.getNameJp() + ")");

					InterfaceDataItem includeItem = includeData.getItem(dataItem.getIncludeItemId());
					if (includeItem != null) {
						fm.setValue("name", includeItem.getName());
						fm.setValue("nameJp", includeItem.getNameJp());
						fm.setValue("type", includeItem.getType().toString());
						setString(fm, "length", includeItem.getLength() == 0 ? "" : "" + includeItem.getLength(), "<br>");
						setString(fm, "subLength", includeItem.getSubLength() == 0 ? "" : "" + includeItem.getSubLength(),
								"<br>");
					}

				}
			}

		} catch (InterfaceException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * ڒǉ{^NbNꂽ̏
	 *
	 * @return EBhE
	 */
	public PirkaWindow regist() {
		try {
			FieldMap fm = getFieldMap();

			String name = fm.getValueString("inName");
			String nameJp = fm.getValueString("inNameJp");
			FieldType type = FieldType.valueOf(fm.getValueString("inType"));
			long length = fm.getValueLong("inLength", 0);
			int subLength = fm.getValueInt("inSubLength", 0);

			dataItem.setEnable(fm.getValueBoolean("ckEnable"));
			dataItem.setUseName(fm.getValueBoolean("ckName"));
			dataItem.setUseNameJp(fm.getValueBoolean("ckNameJp"));
			dataItem.setUseType(fm.getValueBoolean("ckType"));
			dataItem.setUseLength(fm.getValueBoolean("ckLength"));

			dataItem.setPrimaryKey(fm.getValueBoolean("primaryKey"));

			if (dataItem.isUseName()) {
				dataItem.setName(name);
			} else {
				dataItem.setName("");
			}

			if (dataItem.isUseNameJp()) {
				dataItem.setNameJp(nameJp);
			} else {
				dataItem.setNameJp("");
			}

			if (dataItem.isUseType()) {
				dataItem.setType(type);
			} else {
				dataItem.setType(FieldType.STRING);
			}

			if (dataItem.isUseLength()) {
				dataItem.setLength(length);
				dataItem.setSubLength(subLength);
			} else {
				dataItem.setLength(0);
				dataItem.setSubLength(0);
			}

			interfaceProject.getInterfaceDataManager().getInterfaceData(dataId).updateItem(dataItem);
			interfaceProject.setChanged(true);
			dataItem = null;

			return null;
		} catch (InterfaceException e) {
			throw new RuntimeException(e);
		}

	}

	/**
	 * I{^NbNꂽ̏
	 *
	 * @return nulliOʁj
	 */
	public PirkaWindow back() {
		return null;
	}

}
