package jp.oarts.pirka.iop.tool.web.in;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import jp.oarts.pirka.core.kernel.FieldMap;
import jp.oarts.pirka.core.win.PirkaWindow;
import jp.oarts.pirka.iop.tool.core.business.AttributeItem;
import jp.oarts.pirka.iop.tool.core.business.FileCreatorSampleModelerPlugin;
import jp.oarts.pirka.iop.tool.core.business.InterfaceData;
import jp.oarts.pirka.iop.tool.core.business.InterfaceDataManager;
import jp.oarts.pirka.iop.tool.core.business.InterfaceException;
import jp.oarts.pirka.iop.tool.core.business.InterfaceProject;
import jp.oarts.pirka.iop.tool.core.business.PluginManager;
import jp.oarts.pirka.iop.tool.core.general.vo.AttributeData;
import jp.oarts.pirka.iop.tool.core.general.vo.IncludeInfo;
import jp.oarts.pirka.iop.tool.core.general.vo.InterfaceDataItem;
import jp.oarts.pirka.iop.tool.web.common.IopBaseWindow;
import jp.oarts.pirka.iop.tool.web.constants.Define;
import jp.oarts.pirka.iop.tool.web.tools.IopUtil;

public class InterfaceView extends IopBaseWindow {

	/**
	 *
	 */
	private static final long serialVersionUID = -4522943256446547395L;

	private int dataId;

	private HashMap<Integer, String> includeNo2IncludeInterfaceNameMap;
	private List<Integer> includeIdList;

	private List<String> attachList;

	/**
	 * RXgN^
	 *
	 * @param dataId
	 *            f[^ID
	 */
	public InterfaceView(int dataId) {
		super();
		this.dataId = dataId;

		this.includeNo2IncludeInterfaceNameMap = new HashMap<Integer, String>();
		this.includeIdList = new ArrayList<Integer>();
		this.attachList = new ArrayList<String>();

		renew();
	}

	/**
	 *
	 */
	public void preDraw() {
		super.preDraw();
		renew();
	}

	/**
	 * ꗗXV
	 */
	public void renew() {
		try {

			boolean useDisableItemsAttributeFlag = false;

			includeIdList.clear();
			attachList.clear();
			includeNo2IncludeInterfaceNameMap.clear();

			FieldMap fm = getFieldMap();
			InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);
			InterfaceDataManager interfaceDataManager = interfaceProject.getInterfaceDataManager();
			InterfaceData interfaceData = interfaceDataManager.getInterfaceData(dataId);

			fm.clearFieldMap("includeLoop");
			fm.clearFieldMap("attachNameLoop");
			fm.clearFieldMap("attachCol");
			fm.clearFieldMap("attachKeyCol");
			fm.clearFieldMap("listRow");

			DecimalFormat fmt = new DecimalFormat("0000");
			fm.setValue("myInfo", interfaceData.getName() + " ( " + fmt.format(interfaceData.getDataId()) + " : "
					+ interfaceData.getNameJp() + " )");

			if (interfaceData.getIncludeIfoList().length > 0) {
				for (IncludeInfo info : interfaceData.getIncludeIfoList()) {
					FieldMap includeColFieldMap = createLoopFieldMap("includeLoop");
					fm.addFieldMap("includeLoop", includeColFieldMap);

					InterfaceData idwk = interfaceDataManager.getInterfaceData(info.getIncludeDataId());

					String nameStr = idwk.getName() + " (" + fmt.format(info.getIncludeDataId()) + " : " + idwk.getNameJp()
							+ ")";

					includeColFieldMap.setValue("incNo", info.getIncludeId());
					includeColFieldMap.setValue("includeInterfaceName", nameStr);

					includeColFieldMap.setOption("includeInterfaceName", "title", idwk.getComment());
					includeIdList.add(info.getIncludeId());
					includeNo2IncludeInterfaceNameMap.put(info.getIncludeId(), nameStr);
				}

			}

			if (interfaceData.getAttachList().length > 0) {

				for (String attachName : interfaceData.getAttachList()) {
					FileCreatorSampleModelerPlugin attribute = PluginManager.getInstance().getAttachPlugin(attachName);
					if (attribute != null) {
						FieldMap attachFieldMap = createLoopFieldMap("attachNameLoop");
						fm.addFieldMap("attachNameLoop", attachFieldMap);

						attachFieldMap.setValue("attachName", attribute.getNameJp());
						attachFieldMap.setOption("attachName", "title", attribute.getAttributeComment());
						attachList.add(attribute.getName());

						useDisableItemsAttributeFlag |= attribute.useDisableItemsAttribute();
					}
				}

			}

			for (String attachName : interfaceData.getAttachList()) {
				FileCreatorSampleModelerPlugin attribute = PluginManager.getInstance().getAttachPlugin(attachName);
				if (attribute != null) {

					AttributeItem[] items = attribute.getAttributeItems();
					if (items.length > 0) {

						FieldMap attachColFieldMap = createLoopFieldMap("attachCol");
						fm.addFieldMap("attachCol", attachColFieldMap);

						attachColFieldMap.setOption("attachTitleColspan", "colspan", "" + items.length);
						attachColFieldMap.setValue("attachTitleValue", attribute.getNameJp());
						attachColFieldMap.setOption("attachTitleValue", "title", attribute.getAttributeComment());

						for (AttributeItem item : items) {
							FieldMap attachKeyColFieldMap = createLoopFieldMap("attachKeyCol");
							fm.addFieldMap("attachKeyCol", attachKeyColFieldMap);
							attachKeyColFieldMap.setValue("attachTitleKeyName", item.getNameJp());
							attachKeyColFieldMap.setOption("attachTitleKeyName", "title", item.getMessage());
						}

					}

				}
			}

			Integer[] itemIds;
			if (useDisableItemsAttributeFlag) {
				itemIds = interfaceData.getAllItemIdList();
			} else {
				itemIds = interfaceData.getEnableItemIdList();
			}
//			itemIds = interfaceData.getEnableItemIdList();

			int no = 0;
			for (Integer itemId : itemIds) {
				no++;
				InterfaceDataItem item = interfaceData.getItem(itemId);

				FieldMap listRowFieldMap = createLoopFieldMap("listRow");
				fm.addFieldMap("listRow", listRowFieldMap);

				if (item.isEnable()) {
					listRowFieldMap.setOption("lineStyle", "style", "background-color:transparent");
				} else {
					listRowFieldMap.setOption("lineStyle", "style", "background-color:#c0c0c0");
				}

				listRowFieldMap.setValue("no", no);
				if (item.getIncludeId() > 0) {
					setString(listRowFieldMap, "inc", "" + item.getIncludeId(), "<br>");
					String nameStr = includeNo2IncludeInterfaceNameMap.get(item.getIncludeId());
					if (nameStr != null) {
						listRowFieldMap.setOption("inc", "title", nameStr);
					}
				} else {
					setString(listRowFieldMap, "inc", "", "<br>");
				}
				setString(listRowFieldMap, "name", item.getName(), "<br>");
				setString(listRowFieldMap, "nameJp", item.getNameJp(), "<br>");

				if(!item.isMultiRecordType()){
					setString(listRowFieldMap, "type", item.getType().toString(), "<br>");

					if (item.getLength() > 0) {
						listRowFieldMap.setValue("length", item.getLength());
					} else {
						setString(listRowFieldMap, "length", null, "<br>");
					}

					if (item.getSubLength() > 0) {
						listRowFieldMap.setValue("subLength", item.getSubLength());
					} else {
						setString(listRowFieldMap, "subLength", null, "<br>");
					}

				}else{
					setString(listRowFieldMap, "type", "LIST", "<br>");
					setString(listRowFieldMap, "length", null, "<br>");
					setString(listRowFieldMap, "subLength", null, "<br>");
				}
				setString(listRowFieldMap, "primaryKey", item.isPrimaryKey() ? "" : "-", "<br>");

				if(!item.isBaseItem()){
					InterfaceData includeData = interfaceProject.getInterfaceDataManager().getInterfaceData(item.getIncludeDataId());
					InterfaceDataItem includeItem = includeData.getItem(item.getIncludeItemId());

					String includeType = "";
					if (includeData != null && includeItem.getType() != null) {
						includeType = includeItem.getType().toString();
						if (includeItem.getLength() != 0) {
							if (includeItem.getSubLength() != 0) {
								includeType = includeType + "(" + includeItem.getLength() + ", "
										+ includeItem.getSubLength() + ")";
							} else {
								includeType = includeType + "(" + includeItem.getLength() + ")";
							}
						}
					}

					setString(listRowFieldMap, "includeName", includeItem.getName(), "<br>");
					setString(listRowFieldMap, "includeNameJp", includeItem.getNameJp(), "<br>");
					setString(listRowFieldMap, "includeType", includeType, "<br>");
					setString(listRowFieldMap, "includePrimaryKey", includeItem.isPrimaryKey() ? "" : "-", "<br>");

				}else{
					setString(listRowFieldMap, "includeName", null, "<br>");
					setString(listRowFieldMap, "includeNameJp", null, "<br>");
					setString(listRowFieldMap, "includeType", null, "<br>");
					setString(listRowFieldMap, "includePrimaryKey", null, "<br>");
				}

				for (String attachName : interfaceData.getAttachList()) {
					FileCreatorSampleModelerPlugin attribute = PluginManager.getInstance().getAttachPlugin(attachName);
					if (attribute != null) {

						AttributeItem[] items = attribute.getAttributeItems();
						if (items.length > 0) {
							for (AttributeItem aIitem : items) {
								FieldMap attachValueColFieldMap = createLoopFieldMap("attachValueCol");
								listRowFieldMap.addFieldMap("attachValueCol", attachValueColFieldMap);

								AttributeData value = interfaceData
										.getAttribute(item.getItemId(), attachName, aIitem.getName());

								attachValueColFieldMap.setOption("attachValue", "title", aIitem.getMessage());
								if (value == null) {
									setString(attachValueColFieldMap, "attachValue", null, "<br>");
								} else {

									if (attribute.useDisableItemsAttribute() || item.isEnable()) {
										switch (aIitem.getAttributeType()) {
										case CHECK:
											attachValueColFieldMap.setValue("attachValue",
													value != null && value.getValueBoolean() ? "" : "-");
											attachValueColFieldMap.setOption("attachValueTd", "align", "center");
											break;
										case SELECT:
											setString(attachValueColFieldMap, "attachValue", IopUtil.searchTextFromSelectList(
													aIitem.getSelecter(), value.getValue()), "<br>");
											attachValueColFieldMap.setOption("attachValueTd", "align", "center");
											break;
										case INTERFACE:
											setString(attachValueColFieldMap, "attachValue", IopUtil.getInterfaceTitle(
													interfaceProject, value.getValue()), "<br>");
											attachValueColFieldMap.setOption("attachValueTd", "align", "left");
											break;
										default:
											setString(attachValueColFieldMap, "attachValue", value.getValue(), "<br>");
										}
									}else{
										setString(attachValueColFieldMap, "attachValue", null, "<br>");
									}
								}
							}
						}
					}
				}
			}

		} catch (InterfaceException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * C^[tF[XҏW
	 *
	 * @return		EBhE
	 */
	public PirkaWindow update() {
		return new EditInterfaceName(dataId);
	}

	/**
	 * C^[tF[XڕҏW
	 *
	 * @return	EBhE
	 */
	public PirkaWindow addItem() {
		return new EditInterface(dataId);
	}

	/**
	 * C^[tF[XCN[hҏW
	 *
	 * @return	EBhE
	 */
	public PirkaWindow addInclude() {

		InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);
		return new SelectIncludeInterface((InterfaceProject) interfaceProject.clone(), dataId);
	}

	/**
	 * A^b`ҏW{^NbNꂽ̏
	 *
	 * @return	EBhE
	 */
	public PirkaWindow addAttachUpdate() {

		InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);
		return new AttachAttribute((InterfaceProject) interfaceProject.clone(), dataId);
	}

	/**
	 * 폜{^NbNꂽ̏
	 *
	 * @return		EBhE
	 */
	public PirkaWindow delete() {
		InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);
		interfaceProject.getInterfaceDataManager().removeInterfaceData(dataId);

		interfaceProject.setChanged(true);

		return new InterfaceList();
	}

	/**
	 * I{^NbNꂽ̏
	 *
	 * @return nulliOʁj
	 */
	public PirkaWindow back() {

		return new InterfaceList();
	}

}
