package jp.oarts.pirka.iop.tool.web.plugin;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;

import jp.oarts.pirka.core.kernel.FieldMap;
import jp.oarts.pirka.core.res.PirkaFileDownloaderFromBytes;
import jp.oarts.pirka.core.res.PirkaRespons;
import jp.oarts.pirka.core.win.PirkaWindow;
import jp.oarts.pirka.iop.tool.core.business.FileCreatorSampleModelerPlugin;
import jp.oarts.pirka.iop.tool.core.business.InterfaceDesignerPlugin;
import jp.oarts.pirka.iop.tool.core.business.InterfaceException;
import jp.oarts.pirka.iop.tool.core.business.InterfaceProject;
import jp.oarts.pirka.iop.tool.core.business.ParameterItem;
import jp.oarts.pirka.iop.tool.core.business.Plugin;
import jp.oarts.pirka.iop.tool.core.general.constants.ParameterType;
import jp.oarts.pirka.iop.tool.core.general.vo.AttributeFileImage;
import jp.oarts.pirka.iop.tool.core.general.vo.ParameterData;
import jp.oarts.pirka.iop.tool.web.common.IopBaseWindow;
import jp.oarts.pirka.iop.tool.web.constants.Define;
import jp.oarts.pirka.iop.tool.web.tools.IopUtil;

public class ParameterInput extends IopBaseWindow {

	/**
	 *
	 */
	private static final long serialVersionUID = -4522943256446547395L;

	private Plugin plugin;

	private String parameterName;
	private HashMap<String, ParameterData> dataMap;
	private LinkedHashMap<String, ParameterItem> paramMap;
	private HashMap<Integer, ParameterItem> buttonParamItemMap;

	private AttributeFileImage fileImage;

	/**
	 * RXgN^
	 *
	 * @param plugin
	 *            vOC
	 */
	public ParameterInput(Plugin plugin) {
		super();
		this.plugin = plugin;

		dataMap = new HashMap<String, ParameterData>();
		paramMap = new LinkedHashMap<String, ParameterItem>();
		buttonParamItemMap = new HashMap<Integer, ParameterItem>();

		int index = 0;
		for (ParameterItem parameterItem : plugin.getParameterItems()) {
			dataMap.put(parameterItem.getName(), parameterItem.getDefaultValue());
			paramMap.put(parameterItem.getName(), (ParameterItem) parameterItem.clone());
			if (parameterItem.getParameterType() == ParameterType.BUTTON) {
				buttonParamItemMap.put(index, parameterItem);
			}
			index++;
		}
		parameterName = "";
		fileImage = null;
		renew();
	}

	/**
	 * ꗗXV
	 */
	public void renew() {

		InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);

		FieldMap fm = getFieldMap();

		if (parameterName == null || parameterName.trim().length() <= 0) {
			fm.setValue("parameterName", "");
			fm.setValue("deleteSw", false);
		} else {
			fm.setValue("parameterName", " (" + parameterName + ")");
			fm.setValue("deleteSw", true);
			// fm.setValue("parameterName", "p[^ F " + parameterName);
		}

		String[] parameterNames = interfaceProject.getParameterDataManager().getParameterFileListString(plugin.getName());
		if (parameterNames.length > 0) {
			fm.setValue("loadSw", true);
		} else {
			fm.setValue("loadSw", false);
		}

		fm.clearFieldMap("itemLoop");

		fm.setValue("title", plugin.getNameJp());
		fm.setValue("comment", plugin.getParameterComment());
		fm.setValue("saveAndLoad", plugin.useSaveAndLoadParameter());
		fm.setValue("saveAndLoad2", plugin.useSaveAndLoadParameter());

		for (ParameterItem parameterItem : paramMap.values()) {
			FieldMap itemFieldMap = createLoopFieldMap("itemLoop");
			fm.addFieldMap("itemLoop", itemFieldMap);

			itemFieldMap.setValue("checkSw", false);
			itemFieldMap.setValue("nonCheckSw", false);
			itemFieldMap.setValue("TextSw", false);
			itemFieldMap.setValue("fileSw", false);
			itemFieldMap.setValue("selectSw", false);
			itemFieldMap.setValue("textAreaSw", false);
			itemFieldMap.setValue("buttonSw", false);

			itemFieldMap.setValue("attributeItemLeftTitle", parameterItem.getNameJp());
			itemFieldMap.setValue("attributeItemCheckTitle", parameterItem.getNameJp());

			itemFieldMap.setOption("attributeItemLeftTitle", "title", parameterItem.getMessage());
			itemFieldMap.setOption("attributeItemCheckTitle", "title", parameterItem.getMessage());

			switch (parameterItem.getParameterType()) {
			case BUTTON:
				itemFieldMap.setValue("nonCheckSw", true);
				itemFieldMap.setValue("buttonSw", true);
				break;
			case TEXT:
				itemFieldMap.setValue("nonCheckSw", true);
				itemFieldMap.setValue("TextSw", true);
				break;
			case FILE:
				itemFieldMap.setValue("nonCheckSw", true);
				itemFieldMap.setValue("fileSw", true);
				break;
			case INTERFACE:
			case DB:
			case SELECT:
				itemFieldMap.setValue("nonCheckSw", true);
				itemFieldMap.setValue("selectSw", true);
				break;
			case TEXT_AREA:
				itemFieldMap.setValue("nonCheckSw", true);
				itemFieldMap.setValue("textAreaSw", true);
				break;
			case CHECK:
				itemFieldMap.setValue("checkSw", true);
				break;

			}

			ParameterData value = dataMap.get(parameterItem.getName());

			String fieldName = getFieldName(parameterItem.getParameterType());
			String subFieldName = getSubFieldName(parameterItem.getParameterType());

			switch (parameterItem.getParameterType()) {
			case BUTTON:
				itemFieldMap.setValue(fieldName, parameterItem.getNameJp());
				itemFieldMap.setOption(fieldName, "title", parameterItem.getMessage());
				itemFieldMap.setValue("attributeItemLeftTitle", "@");
				break;
			case TEXT:
				if (value == null) {
					itemFieldMap.setValue(fieldName, "");
				} else {
					itemFieldMap.setValue(fieldName, value.getValue());
				}
				if (parameterItem.getLength() > 0) {
					itemFieldMap.setOption(fieldName, "maxlength", "" + parameterItem.getLength());
					itemFieldMap.setOption(fieldName, "size", "" + parameterItem.getLength() * 2);
				} else {
					itemFieldMap.clearOption(fieldName, "length");
					itemFieldMap.setOption(fieldName, "size", "100");
				}
				itemFieldMap.setOption(fieldName, "title", parameterItem.getMessage());
				break;
			case FILE:
				itemFieldMap.setOption(fieldName, "title", parameterItem.getMessage());
				if (value == null) {
					itemFieldMap.setValue(subFieldName, "");
				} else {
					itemFieldMap.setValue(subFieldName, value.getValue());
				}
				break;
			case INTERFACE:
				itemFieldMap.setSelectList(fieldName, IopUtil.getInterfaceDataSelecter(interfaceProject, !parameterItem
						.isMultiSelect()));
				if (value == null) {
					itemFieldMap.setValue(fieldName, "");
				} else {
					itemFieldMap.setValue(fieldName, value.getValue());
				}
				if (parameterItem.getLength() > 0) {
					itemFieldMap.setOption(fieldName, "size", "" + parameterItem.getLength());
				} else {
					itemFieldMap.clearOption(fieldName, "size");
				}
				itemFieldMap.setOption(fieldName, "title", parameterItem.getMessage());
				if (parameterItem.isMultiSelect()) {
					itemFieldMap.setOption(fieldName, "multiple");
				} else {
					itemFieldMap.clearOption(fieldName, "multiple");
				}
				break;
			case DB:
				itemFieldMap.setSelectList(fieldName, IopUtil.getDbSelecter(!parameterItem.isMultiSelect()));
				if (value == null) {
					itemFieldMap.setValue(fieldName, "");
				} else {
					itemFieldMap.setValue(fieldName, value.getValue());
				}
				if (parameterItem.getLength() > 0) {
					itemFieldMap.setOption(fieldName, "size", "" + parameterItem.getLength());
				} else {
					itemFieldMap.clearOption(fieldName, "size");
				}
				itemFieldMap.setOption(fieldName, "title", parameterItem.getMessage());
				if (parameterItem.isMultiSelect()) {
					itemFieldMap.setOption(fieldName, "multiple");
				} else {
					itemFieldMap.clearOption(fieldName, "multiple");
				}
				break;
			case SELECT:
				itemFieldMap.setSelectList(fieldName, IopUtil.convartSelectList(parameterItem.getSelecter()));
				if (value == null) {
					itemFieldMap.setValue(fieldName, "");
				} else {
					itemFieldMap.setValue(fieldName, value.getValue());
				}
				if (parameterItem.getLength() > 0) {
					itemFieldMap.setOption(fieldName, "size", "" + parameterItem.getLength());
				} else {
					itemFieldMap.clearOption(fieldName, "size");
				}
				itemFieldMap.setOption(fieldName, "title", parameterItem.getMessage());
				if (parameterItem.isMultiSelect()) {
					itemFieldMap.setOption(fieldName, "multiple");
				} else {
					itemFieldMap.clearOption(fieldName, "multiple");
				}
				break;
			case TEXT_AREA:
				if (value == null) {
					itemFieldMap.setValue(fieldName, "");
				} else {
					itemFieldMap.setValue(fieldName, value.getValue());
				}
				itemFieldMap.setOption(fieldName, "title", parameterItem.getMessage());
				break;
			case CHECK:
				if (value == null) {
					itemFieldMap.setValue(fieldName, false);
				} else {
					itemFieldMap.setValue(fieldName, value.getValueBoolean());
				}
				itemFieldMap.setOption(fieldName, "title", parameterItem.getMessage());
				break;

			}
		}

	}

	public boolean remakeDataMap() {

		HashMap<String, ParameterData> keepDataMap = (HashMap<String, ParameterData>) dataMap.clone();
		dataMap.clear();

		FieldMap fm = getFieldMap();
		FieldMap[] itemFms = fm.getFieldMaps("itemLoop");

		// G[`FbN
		boolean errorFlag = false;
		int index = 0;

		for (String paramKey : paramMap.keySet()) {

			ParameterItem parameterItem = paramMap.get(paramKey);

			FieldMap itemFm = itemFms[index++];

			String fieldNmae = getFieldName(parameterItem.getParameterType());
			String text = null;
			ParameterData value = null;
			try {
				switch (parameterItem.getParameterType()) {
				case TEXT:
					text = itemFm.getValueString(fieldNmae);
					parameterItem.check(text);
					value = new ParameterData(text);
					break;
				case FILE:
					byte[] fileImage = new byte[0];
					text = itemFm.getValueString(fieldNmae);
					if (text == null || text.length() <= 0) {
						if (keepDataMap.containsKey(paramKey)) {
							ParameterData tempParamData = keepDataMap.get(paramKey);
							text = tempParamData == null ? "" : tempParamData.getValue();
							fileImage = tempParamData == null ? null : tempParamData.getFileImage();
						}
					} else {
						InputStream is = null;
						ByteArrayOutputStream baos = null;
						byte[] buff = new byte[4096];
						try {
							try {
								is = itemFm.getInputStream(fieldNmae);
								if (is != null) {
									try {
										baos = new ByteArrayOutputStream();
										while (true) {
											int length = is.read(buff);
											if (length <= 0) {
												break;
											}

											baos.write(buff, 0, length);
										}

										fileImage = baos.toByteArray();
									} finally {
										if (baos != null) {
											baos.close();
										}
									}
								}
							} finally {
								if (is != null) {
									is.close();
								}
							}

						} catch (IOException e) {
							throw new InterfaceException("Abv[ht@C̎擾Ɏs܂", e);
						}
					}
					parameterItem.check(text);
					value = new ParameterData(text, fileImage);
					break;

				case INTERFACE:
					text = itemFm.getValueString(fieldNmae);
					parameterItem.check(text);
					value = new ParameterData(text);
					break;

				case DB:
					text = itemFm.getValueString(fieldNmae);
					parameterItem.check(text);
					value = new ParameterData(text);
					break;

				case SELECT:
					text = itemFm.getValueString(fieldNmae);
					parameterItem.check(text);
					value = new ParameterData(text);
					break;

				case TEXT_AREA:
					text = itemFm.getValueString(fieldNmae);
					parameterItem.check(text);
					value = new ParameterData(text);
					break;

				case CHECK:
					value = new ParameterData((Boolean) itemFm.getValueBoolean(fieldNmae));
					break;

				}

				if (value != null) {
					dataMap.put(parameterItem.getName(), value);
				}

			} catch (InterfaceException e) {
				errorFlag = true;
				addError(parameterItem.getNameJp(), e.getMessage());
				itemFm.setError(fieldNmae, true);
			}
		}

		InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);

		if (!errorFlag) {

			List<String> errorKeyList = new LinkedList<String>();
			try {
				plugin.checkParameter(dataMap, interfaceProject, errorKeyList);
			} catch (InterfaceException e) {
				errorFlag = true;
				addError(e.getMessage());
				index = 0;
				for (ParameterItem parameterItem : plugin.getParameterItems()) {
					FieldMap itemFm = itemFms[index++];

					if (errorKeyList.contains(parameterItem.getName())) {
						itemFm.setError(getFieldName(parameterItem.getParameterType()), true);
					}
				}
			}
		}

		return errorFlag;
	}

	public boolean remakeDataMapNoCheck() {


		HashMap<String, ParameterData> keepDataMap = (HashMap<String, ParameterData>) dataMap.clone();
		dataMap.clear();

		FieldMap fm = getFieldMap();
		FieldMap[] itemFms = fm.getFieldMaps("itemLoop");

		// G[`FbN
		boolean errorFlag = false;
		int index = 0;

		for (String paramKey : paramMap.keySet()) {

			ParameterItem parameterItem = paramMap.get(paramKey);

			FieldMap itemFm = itemFms[index++];

			String fieldNmae = getFieldName(parameterItem.getParameterType());
			String text = null;
			ParameterData value = null;
			try {
				switch (parameterItem.getParameterType()) {
				case TEXT:
					text = itemFm.getValueString(fieldNmae);
					// parameterItem.check(text);
					value = new ParameterData(text);
					break;
				case FILE:
					byte[] fileImage = new byte[0];
					text = itemFm.getValueString(fieldNmae);
					if (text == null || text.length() <= 0) {
						if (keepDataMap.containsKey(paramKey)) {
							ParameterData tempParamData = keepDataMap.get(paramKey);
							text = tempParamData == null ? "" : tempParamData.getValue();
							fileImage = tempParamData == null ? null : tempParamData.getFileImage();
						}
					} else {
						InputStream is = null;
						ByteArrayOutputStream baos = null;
						byte[] buff = new byte[4096];
						try {
							try {
								is = itemFm.getInputStream(fieldNmae);
								if (is != null) {
									try {
										baos = new ByteArrayOutputStream();
										while (true) {
											int length = is.read(buff);
											if (length <= 0) {
												break;
											}

											baos.write(buff, 0, length);
										}

										fileImage = baos.toByteArray();
									} finally {
										if (baos != null) {
											baos.close();
										}
									}
								}
							} finally {
								if (is != null) {
									is.close();
								}
							}

						} catch (IOException e) {
							throw new InterfaceException("Abv[ht@C̎擾Ɏs܂", e);
						}
					}
					//parameterItem.check(text);
					value = new ParameterData(text, fileImage);
					break;

				case INTERFACE:
					text = itemFm.getValueString(fieldNmae);
					// parameterItem.check(text);
					value = new ParameterData(text);
					break;

				case DB:
					text = itemFm.getValueString(fieldNmae);
					// parameterItem.check(text);
					value = new ParameterData(text);
					break;

				case SELECT:
					text = itemFm.getValueString(fieldNmae);
					// parameterItem.check(text);
					value = new ParameterData(text);
					break;

				case TEXT_AREA:
					text = itemFm.getValueString(fieldNmae);
					// parameterItem.check(text);
					value = new ParameterData(text);
					break;

				case CHECK:
					value = new ParameterData((Boolean) itemFm.getValueBoolean(fieldNmae));
					break;

				}

				if (value != null) {
					dataMap.put(parameterItem.getName(), value);
				}

			} catch (InterfaceException e) {
				errorFlag = true;
				addError(parameterItem.getNameJp(), e.getMessage());
				itemFm.setError(fieldNmae, true);
			}
		}

		// InterfaceProject interfaceProject = (InterfaceProject)
		// getSession(Define.PROJECT_SESSION_NAME);

		// if (!errorFlag) {
		//
		// List<String> errorKeyList = new LinkedList<String>();
		// try {
		// plugin.checkParameter(dataMap, interfaceProject, errorKeyList);
		// } catch (InterfaceException e) {
		// errorFlag = true;
		// addError(e.getMessage());
		// index = 0;
		// for (ParameterItem parameterItem : plugin.getParameterItems()) {
		// FieldMap itemFm = itemFms[index++];
		//
		// if (errorKeyList.contains(parameterItem.getName())) {
		// itemFm.setError(getFieldName(parameterItem.getParameterType()),
		// true);
		// }
		// }
		// }
		// }

		return errorFlag;
	}

	/**
	 *
	 * @param type
	 * @return
	 */
	private String getFieldName(ParameterType type) {
		String fieldNmae = "";
		switch (type) {
		case BUTTON:
			fieldNmae = "attributeButton";
			break;
		case TEXT:
			fieldNmae = "attributeText";
			break;
		case FILE:
			fieldNmae = "attributeFile";
			break;
		case INTERFACE:
			fieldNmae = "attributeSelect";
			break;
		case DB:
			fieldNmae = "attributeSelect";
			break;
		case SELECT:
			fieldNmae = "attributeSelect";
			break;
		case TEXT_AREA:
			fieldNmae = "attributeTextArea";
			break;
		case CHECK:
			fieldNmae = "attributeCheckBox";
		}
		return fieldNmae;
	}

	/**
	 *
	 * @param type
	 * @return
	 */
	private String getSubFieldName(ParameterType type) {
		String subFieldNmae = "";
		switch (type) {
		case BUTTON:
			subFieldNmae = "";
			break;
		case TEXT:
			subFieldNmae = "";
			break;
		case FILE:
			subFieldNmae = "attributeFileName";
			break;
		case INTERFACE:
			subFieldNmae = "";
			break;
		case DB:
			subFieldNmae = "";
			break;
		case SELECT:
			subFieldNmae = "";
			break;
		case TEXT_AREA:
			subFieldNmae = "";
			break;
		case CHECK:
			subFieldNmae = "";
		}
		return subFieldNmae;
	}

	/**
	 * Jn{^NbNꂽ̏
	 *
	 * @return 
	 */
	public PirkaRespons start() {

		InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);
		boolean errorFlag = remakeDataMap();

		if (errorFlag) {
			return this;
		}

		if (plugin instanceof InterfaceDesignerPlugin) {

			return new ExecuteInterfaceDesigner((InterfaceDesignerPlugin) plugin, dataMap, interfaceProject);

		} else if (plugin instanceof FileCreatorSampleModelerPlugin) {

			Execute exe = new Execute(this, (FileCreatorSampleModelerPlugin) plugin, dataMap, interfaceProject);

			return exe;
		}

		return this;

	}

	/**
	 * t@CNG[^[ATvf[̎sʃZbg
	 *
	 * @param fileImage
	 * @param errorMessage
	 */
	public void setExecResult(AttributeFileImage fileImage, String errorMessage) {
		FieldMap fm = getFieldMap();

		if (errorMessage != null) {
			fm.clearOption("myBody", "onLoad");
			addError(errorMessage);
		} else {
			this.fileImage = fileImage;
			fm.setOption("myBody", "onLoad", "document.myForm.submit();");
		}
	}

	/**
	 * _E[hJAVASCRIPTĂ΂
	 *
	 * @return 
	 */
	public PirkaRespons onElse() {
		FieldMap fm = getFieldMap();
		fm.clearOption("myBody", "onLoad");

		if (fileImage != null) {
			PirkaFileDownloaderFromBytes download = new PirkaFileDownloaderFromBytes(fileImage.getFileNmae(), fileImage
					.getFileImage());
			fileImage = null;
			return download;
		} else {
			return this;
		}
	}

	@SuppressWarnings("deprecation")
	public PirkaWindow save() {

		remakeDataMap();

		if (getErrorMessageStocker().getErrorMessageList().size() > 0) {
			return this;
		}
		return new ParameterSave(plugin.getName(), parameterName, dataMap, this);
	}

	public PirkaWindow load() {
		return new ParameterLoad(plugin.getName(), this);
	}

	public void attributeButton() {

		int index = getActionLoopIndex()[0];
		remakeDataMapNoCheck();

		if (getErrorMessageStocker().getErrorMessageList().size() > 0) {
			return;
		}

		ParameterItem item = buttonParamItemMap.get(index);
		if (item != null) {
			try {
				item.clickButton(paramMap, dataMap);
				renew();
			} catch (InterfaceException e) {
				addError(e.getMessage());
			}
		}
	}

	public void setParameterName(String parameterName) {
		this.parameterName = parameterName;
		InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);
		dataMap = interfaceProject.getParameterDataManager().getParameterDataMap(plugin.getName(), parameterName);
		if (dataMap == null) {
			dataMap = new HashMap<String, ParameterData>();
			for (ParameterItem parameterItem : paramMap.values()) {
				dataMap.put(parameterItem.getName(), parameterItem.getDefaultValue());
			}
		}
		renew();
	}

	public void delete() {
		if (parameterName == null || parameterName.trim().length() <= 0) {
			return;
		}

		InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);
		interfaceProject.getParameterDataManager().removeParameterDataMap(plugin.getName(), parameterName);

		parameterName = "";
		renew();
		interfaceProject.setChanged(true);

	}

	/**
	 * ߂{^NbNꂽ̏
	 *
	 * @return nulliOʁj
	 */
	public PirkaWindow back() {

		return null;
	}

}
