package jp.oarts.pirka.iop.tool.web.start;

import java.util.ArrayList;
import java.util.List;

import jp.oarts.pirka.core.kernel.FieldMap;
import jp.oarts.pirka.core.res.PirkaFileDownloaderFromBytes;
import jp.oarts.pirka.core.res.PirkaRespons;
import jp.oarts.pirka.core.win.PirkaWindow;
import jp.oarts.pirka.iop.tool.core.business.FileCreatorSampleModelerPlugin;
import jp.oarts.pirka.iop.tool.core.business.InterfaceDesignerPlugin;
import jp.oarts.pirka.iop.tool.core.business.InterfaceException;
import jp.oarts.pirka.iop.tool.core.business.InterfaceProject;
import jp.oarts.pirka.iop.tool.core.business.PluginManager;
import jp.oarts.pirka.iop.tool.core.general.constants.PluginType;
import jp.oarts.pirka.iop.tool.web.common.Alert;
import jp.oarts.pirka.iop.tool.web.common.Confirm;
import jp.oarts.pirka.iop.tool.web.common.ConfirmAction;
import jp.oarts.pirka.iop.tool.web.common.IopBaseWindow;
import jp.oarts.pirka.iop.tool.web.constants.Define;
import jp.oarts.pirka.iop.tool.web.in.InterfaceList;
import jp.oarts.pirka.iop.tool.web.plugin.ParameterInput;
import jp.oarts.pirka.iop.tool.web.project.EditProject;
import jp.oarts.pirka.iop.tool.web.project.LoadProject;

/**
 * j[ʁiʁj
 *
 * @author ito
 *
 */
@SuppressWarnings("serial")
public class StartMenu extends IopBaseWindow implements ConfirmAction {

	private static final String MSG_PROJECT_EXISTS_TITLE = "vWFNgۑĂ܂";
	private static final String MSG_PROJECT_EXISTS = "vWFNgۑ܂͕Ă炱̑sĂ";

	private static final String MSG_PROJECT_NOT_EXISTS_TITLE = "vWFNg݂܂";
	private static final String MSG_PROJECT_NOT_EXISTS = "vWFNg̍쐬܂͓Ǎɂ̑sĂ";
	private static final String MSG_PROJECT_NOT_EXISTS_CLOSE = "ɃvWFNg͕Ă܂";

	private List<String> interfaceDesignerList;
	private List<String> fileCreatorList;
	private List<String> sampleModelerList;

	/**
	 * RXgN^
	 */
	public StartMenu() {
		super();
		interfaceDesignerList = new ArrayList<String>();
		fileCreatorList = new ArrayList<String>();
		sampleModelerList = new ArrayList<String>();

		renew();
	}

	public void renew() {
		FieldMap fm = getFieldMap();

		FieldMap interfaceDesignerLoopMap;
		for (InterfaceDesignerPlugin plugin : PluginManager.getInstance().getInterfaceDesignerPluginList()) {
			interfaceDesignerList.add(plugin.getName());
			interfaceDesignerLoopMap = createLoopFieldMap("interfaceDesignerLoop");
			fm.addFieldMap("interfaceDesignerLoop", interfaceDesignerLoopMap);
			interfaceDesignerLoopMap.setValue("interfaceDesignerName", plugin.getNameJp());
			interfaceDesignerLoopMap.setOption("interfaceDesignerName", "title", plugin.getComment());
		}

		for (FileCreatorSampleModelerPlugin plugin : PluginManager.getInstance().getAttachPluginList()) {

			if (plugin.canExecute()) {

				if (plugin.getPluginType() == PluginType.FILE_CREATOR) {
					fileCreatorList.add(plugin.getName());

					interfaceDesignerLoopMap = createLoopFieldMap("fileCreatorLoop");
					fm.addFieldMap("fileCreatorLoop", interfaceDesignerLoopMap);
					interfaceDesignerLoopMap.setValue("fileCreatorName", plugin.getNameJp());
					interfaceDesignerLoopMap.setOption("fileCreatorName", "title", plugin.getComment());

				} else if (plugin.getPluginType() == PluginType.SAMPLE_MODELER) {
					sampleModelerList.add(plugin.getName());

					interfaceDesignerLoopMap = createLoopFieldMap("sampleModelerLoop");
					fm.addFieldMap("sampleModelerLoop", interfaceDesignerLoopMap);
					interfaceDesignerLoopMap.setValue("sampleModelerName", plugin.getNameJp());
					interfaceDesignerLoopMap.setOption("sampleModelerName", "title", plugin.getComment());
				}
			}

		}

	}

	/**
	 * ۑ̃vWFNg`FbNs
	 *
	 * @return true=ۑ
	 */
	protected boolean isExistsNonSaveProject() {
		InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);
		if (interfaceProject != null && interfaceProject.isChanged()) {
			return true;
		}
		return false;
	}

	/**
	 * VKvWFNgNbNꂽƂ̏
	 *
	 * @return VKvWFNg
	 */
	public PirkaWindow newProject() {

		if (isExistsNonSaveProject()) {
			return new Alert(MSG_PROJECT_EXISTS_TITLE, MSG_PROJECT_EXISTS);
		}

		return new EditProject(null);
	}

	/**
	 * vWFNgXVNbNꂽƂ̏
	 *
	 * @return VKvWFNg
	 */
	public PirkaWindow updateProject() {

		InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);
		if (interfaceProject == null) {
			return new Alert(MSG_PROJECT_NOT_EXISTS_TITLE, MSG_PROJECT_NOT_EXISTS);
		}

		return new EditProject(interfaceProject);
	}

	/**
	 * vWFNgǂݍ݂NbNꂽƂ̏
	 *
	 * @return IuWFNg
	 */
	public PirkaWindow loadProject() {

		if (isExistsNonSaveProject()) {
			return new Alert(MSG_PROJECT_EXISTS_TITLE, MSG_PROJECT_EXISTS);
		}

		return new LoadProject(this);
	}

	/**
	 * vWFNg݂NbNꂽƂ̏
	 *
	 * @return IuWFNg
	 */
	public PirkaRespons saveProject() {

		try {
			InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);
			if (interfaceProject == null) {
				return new Alert(MSG_PROJECT_NOT_EXISTS_TITLE, MSG_PROJECT_NOT_EXISTS);
			}

			interfaceProject.setChanged(false);

			PirkaFileDownloaderFromBytes downloader;
			downloader = new PirkaFileDownloaderFromBytes(interfaceProject.getName(), interfaceProject.toXmlByteArray());
			return downloader;
		} catch (InterfaceException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * vWFNgNbNꂽƂ̏
	 *
	 * @return IuWFNg
	 */
	public PirkaWindow closeProject() {
		InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);
		if (interfaceProject == null) {
			return new Alert(MSG_PROJECT_NOT_EXISTS_TITLE, MSG_PROJECT_NOT_EXISTS_CLOSE);
		}

		return new Confirm(this, "vWFNg", "vWFNg܂H");
	}

	/**
	 * C^[tF[XfUCWEBNbNꂽƂ̏
	 *
	 * @return		EBhE
	 */
	public PirkaWindow interfaceDesignerWeb() {
		InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);
		if (interfaceProject == null) {
			return new Alert(MSG_PROJECT_NOT_EXISTS_TITLE, MSG_PROJECT_NOT_EXISTS);
		}

		return new InterfaceList();
	}

	/**
	 * C^[tF[XNbNꂽƂ̏
	 *
	 * @return		EBhE
	 */
	public PirkaWindow interfaceDesigner() {
		InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);
		if (interfaceProject == null) {
			return new Alert(MSG_PROJECT_NOT_EXISTS_TITLE, MSG_PROJECT_NOT_EXISTS);
		}

		int[] indexs = getActionLoopIndex();

		String interfaceDesignerName = interfaceDesignerList.get(indexs[0]);

		if (InterfaceList.class.getName().equals(interfaceDesignerName)) {
			return new InterfaceList();
		}

		return new ParameterInput(PluginManager.getInstance().getInterfaceDesignerPlugin(interfaceDesignerList.get(indexs[0])));

	}

	/**
	 * t@CNG[^[NbNꂽƂ̏
	 *
	 * @return	EBhE
	 */
	public PirkaWindow fileCreator() {
		InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);
		if (interfaceProject == null) {
			return new Alert(MSG_PROJECT_NOT_EXISTS_TITLE, MSG_PROJECT_NOT_EXISTS);
		}

		int[] indexs = getActionLoopIndex();

		return new ParameterInput(PluginManager.getInstance().getAttachPlugin(fileCreatorList.get(indexs[0])));

	}

	/**
	 * Tvf[NbNꂽƂ̏
	 *
	 * @return		EBhE
	 */
	public PirkaWindow sampleModeler() {
		InterfaceProject interfaceProject = (InterfaceProject) getSession(Define.PROJECT_SESSION_NAME);
		if (interfaceProject == null) {
			return new Alert(MSG_PROJECT_NOT_EXISTS_TITLE, MSG_PROJECT_NOT_EXISTS);
		}

		int[] indexs = getActionLoopIndex();

		return new ParameterInput(PluginManager.getInstance().getAttachPlugin(sampleModelerList.get(indexs[0])));

	}

	@Override
	public PirkaRespons no() {
		return null;
	}

	@Override
	public PirkaRespons yes() {
		putSession(Define.PROJECT_SESSION_NAME, null);
		return null;
	}

}
